#!/bin/env python3
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   16-Aug-2012

import os
import tkinter as tk
from tkinter import ttk

import tkFlair
import tkExtra
import tkDialogs
import FlairRibbon

try:
	import pydicom
except ImportError:
	try:
		import dicom as pydicom
	except ImportError:
		pydicom = None
import Dicom

#===============================================================================
# Info Tab
#===============================================================================
class InfoTab(FlairRibbon.FlairTab):
	#----------------------------------------------------------------------
	def __init__(self, master, page, **kw):
		super().__init__(master, page, "Dicom", **kw)

		topf = tk.Frame(self)
		topf.pack(side=tk.TOP, fill=tk.X)

		# ---
		row = 0
		tk.Label(topf, text="Global:", foreground="DarkRed").grid(
				row=row, column=0, sticky=tk.W)

		# ---
		row += 1
		tk.Label(topf, text="Directory:").grid(row=row, column=0, sticky=tk.E)
		self.directory = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				width=20,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.directory.grid(row=row, column=1, sticky=tk.EW)

		# ---
		row += 1
		tk.Label(topf, text="File(s):").grid(row=row, column=0, sticky=tk.E)
		self.file = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				width=20,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.file.grid(row=row, column=1, sticky=tk.EW)

		# ---
		row += 1
		tk.Label(topf, text="UID:").grid(row=row, column=0, sticky=tk.E)
		self.uid = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				width=20,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.uid.grid(row=row, column=1, sticky=tk.EW)

		# ---
		row += 1
		tk.Label(topf, text="Patients Name:").grid(row=row, column=0, sticky=tk.E)
		self.patientname = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.patientname.grid(row=row, column=1, sticky=tk.EW)

		# ---
		row += 1
		tk.Label(topf, text="Modality:").grid(row=row, column=0, sticky=tk.E)
		self.modality = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.modality.grid(row=row, column=1, sticky=tk.EW)

		# ---
		row += 1
		tk.Label(topf, text="Study Date/Time:").grid(row=row, column=0, sticky=tk.E)
		self.datetime = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.datetime.grid(row=row, column=1, sticky=tk.EW)

		# ---
		row += 1
		tk.Label(topf, text="Patients Age:").grid(row=row, column=0, sticky=tk.E)
		self.age = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.age.grid(row=row, column=1, sticky=tk.EW)

		# ---
		row += 1
		tk.Label(topf, text="Patients Sex:").grid(row=row, column=0, sticky=tk.E)
		self.sex = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.sex.grid(row=row, column=1, sticky=tk.EW)

		# ---
		row += 1
		tk.Label(topf, text="Institution:").grid(row=row, column=0, sticky=tk.E)
		self.institution = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.institution.grid(row=row, column=1, sticky=tk.EW)

		# ---
		row += 1
		tk.Label(topf, text="Study:").grid(row=row, column=0, sticky=tk.E)
		self.study = tk.Label(topf, justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.study.grid(row=row, column=1, sticky=tk.EW)

		topf.grid_columnconfigure(1, weight=1)

		# Tree view listbox
		# show first the scrollbar to be always visible
		sb = ttk.Scrollbar(self, orient=tk.VERTICAL,
				takefocus=False)
				#command=self.tree.yview)
		sb.pack(side=tk.RIGHT, fill=tk.Y)

		self.tree = tkExtra.Treeview(self)
		self.tree.pack(side=tk.LEFT, fill=tk.BOTH, expand=tk.YES)
		self.tree.column("#0",  width=240*tkFlair.dpiscale,
					minwidth=80*tkFlair.dpiscale,
					stretch=tk.NO)
		self.tree.heading("#0", text="Name", anchor=tk.W)
		self.tree.sortColumn("#0")
		self.tree["yscrollcommand"] = sb.set
		sb["command"] = self.tree.yview

		# Tree columns based on mdality
		self.modalityColumns = {
				"*" : ("Value",),
				"RTSTRUCT" : (
					 "Dicom Volume (cm3)",
					 "Voxel Volume (cm3)",
					 "Polygon Volume (cm3)",
					 "Dicom/Polygon (+%)",
					 "Voxel/Polygon (+%)")
			}

		self._inrefresh = False

	# ----------------------------------------------------------------------
	@property
	def dicom(self):	return self.page.dicominfo.dicom

	# ----------------------------------------------------------------------
	def showModality(self, modality):
		try:
			columns = self.modalityColumns[modality]
		except KeyError:
			columns = self.modalityColumns["*"]
		self.tree["columns"] = [f"#{i}" for i in range(1,len(columns)+1)]
		for i,column in enumerate(columns):
			col = f"#{i+1}"
			self.tree.column(col,  stretch=tk.YES)
			self.tree.heading(col, text=column, anchor=tk.CENTER)
			self.tree.sortColumn(col, float)

	# ----------------------------------------------------------------------
	def activate(self):
		self.refresh()

	# ----------------------------------------------------------------------
	def deactivate(self):
		self.page.showContextGroup()

	# --------------------------------------------------------------------
	def refresh(self):
		if self.page.dicominfo is None or self.dicom is None:
			self.clearInformation()
			return
		if self._inrefresh: return	# To avoid a second call from the update_idletasks
		self._inrefresh = True		# called from the setStatus in analyzeSlices
		if not self.page.dicominfo.cache:
			err = self.page.analyzeSlices()
			if err:
				self.flair.notify("Error analyzing dicom",
						err,
						tkFlair.NOTIFY_ERROR)
				self._inrefresh = False
				return
		self.clearInformation()
		self.showModality(self.dicom.modality)
		if self.page.dicominfo is None:
			self.flair.notify("No DICOM selected",
				"Please select a DICOM set first")
			self._inrefresh = False
			return
		self.headerInformation()
		if self.dicom.modality == "RTSTRUCT":
			self.page.showContextGroup()
			self.updateRtStruct()
		#elif self.dicom.modality == "RTIMAGE":
		elif self.dicom.modality in ("CT","NM","PT","MR","RTDOSE"):
			self.page.showContextGroup("USRBIN")
			self.updateVoxelInformation()
		else:
			self.page.showContextGroup()
		self.treeFill()
		self._inrefresh = False

	# --------------------------------------------------------------------
	# Clear voxel information
	# --------------------------------------------------------------------
	def clearInformation(self):
		self.directory["text"]   = ""
		self.file["text"]        = ""
		self.uid["text"]         = ""
		self.datetime["text"]    = ""
		self.patientname["text"] = ""
		self.modality["text"]    = ""
		self.age["text"]         = ""
		self.sex["text"]         = ""
		self.institution["text"] = ""
		self.study["text"]       = ""
		self.tree.delete(*self.tree.get_children())

	# --------------------------------------------------------------------
	def headerInformation(self):
		fn = self.page.dicominfo.filename()
		header = Dicom.dicomHeader(self.page.dicominfo.name, fn)
		filestr = "\n".join([os.path.basename(x) for x in self.page.dicomfile[:3]])
		if len(self.page.dicomfile)>3: filestr += "\n..."
		if len(filestr)>160: filestr = f"{filestr[:160]}..."
		self.directory["text"]   = self.page.dicominfo("directory")
		self.file["text"]        = filestr
		self.uid["text"]         = header.uid
		self.modality["text"]    = "%s : %s" % \
				(header.modality, Dicom.MODALITY.get(header.modality,"?"))
		self.datetime["text"]    = header.datetime
		self.patientname["text"] = header.name
		self.age["text"]         = header.age
		self.sex["text"]         = header.sex
		self.institution["text"] = header.institution
		self.study["text"]       = header.study

	# --------------------------------------------------------------------
	# update voxel information
	# --------------------------------------------------------------------
	def updateVoxelInformation(self):
		if self.page.dicominfo.cache: return
		cache = self.page.dicominfo.cache
		d = self.dicom

		section = "Dataset"
		if d.modality in ("CT", "MR") and d.hasSlices:
			cache[f"{section}:# slices"] = str(len(self.page.dicominfo.files))
		cache[f"{section}:Patient Position"] = d.patientPosition
		cache[f"{section}:Position"] = f"[{d.x}, {d.y}, {d.z}] mm"
		cache[f"{section}:Origin"]   = f"[{d.origin().x}, {d.origin().y}, {d.origin().z}] mm"
		cache[f"{section}:Orientation"] = str(d.iop)
		cache[f"{section}:Dimensions"] = f"{d.nx*d.dx} x {d.ny*d.dy} x {d.nz*d.dz} mm^3"
		cache[f"{section}:# voxels [Nx x Ny x Nz]"] = f"{d.nx} x {d.ny} x {d.nz}"
		cache[f"{section}:Voxel size [dx x dy x dz]"] = f"{d.dx} x {d.dy} x {d.dz} mm^3"

		m = "" if d.m == 1 else f"{d.m} * "
		if d.b > 0:
			cache[f"{section}:Rescale"] = f"{m}value + {d.b}"
		elif d.b < 0:
			cache[f"{section}:Rescale"] = f"{m}value - {-d.b}"
		elif d.b == 0:
			cache[f"{section}:Rescale"] = f"{m}value"

		# Cropped area
		nx = d.ncolumns()
		ny = d.nrows()
		nz = d.nslices()

		if d.modality != "RTDOSE":
			section = "Selected Volume"
			cache[f"{section}:Lower Corner"] = "[%g, %g, %g] mm" % tuple(d.origin())
			cache[f"{section}:# voxels [Nx x Ny x Nz]"] = f"{nx} x {ny} x {nz}"
			cache[f"{section}:Dimensions [Dx x Dy x Dz]"] = \
				f"{nx*d.dx} x {ny*d.dy} x {nz*d.dz} mm^3"
		else:
			try:
				dataset = pydicom.read_file(self.page.dicominfo.filename(), force=True)
				cache[f"{section}:Dose Units"] = dataset.DoseUnits
				cache[f"{section}:Dose Type"] = dataset.DoseType
				cache[f"{section}:Dose Summation Type"] = dataset.DoseSummationType
			except pydicom.filereader.InvalidDicomError:
				pass
		if d.hasPixel:
			cache[f"{section}:Unit range [Min..Max]"] = f"{d.minHU} .. {d.maxHU}"
			cache[f"{section}:Unique Units [Regions]"] = str(len(d.reg2hu))

	# --------------------------------------------------------------------
	# Update RT structure information
	# --------------------------------------------------------------------
	def updateRtStruct(self):
		if self.page.dicominfo.cache: return
		cache = self.page.dicominfo.cache

		try:
			dataset = pydicom.read_file(self.page.dicominfo.filename(), force=True)
		except pydicom.filereader.InvalidDicomError:
			return
		ctinfo = None
		for cnt in dataset.ReferencedFrameOfReferenceSequence[0]\
				.RTReferencedStudySequence[0]\
				.RTReferencedSeriesSequence[0]\
				.ContourImageSequence:
			ctinfo = self.project.dicomInfoFromSOPInstanceUID(cnt.ReferencedSOPInstanceUID)
			if ctinfo is not None:
				break

		# Load first file
		if ctinfo:
			ctfilename = ctinfo.filename()
		else:
			ctfilename = None

		rt = Dicom.RTStruct(dataset, ctfilename) #, x0, y0, dx, dy, dz)

		# Display progress dialog
		dlg = tkDialogs.ProgressDialog(self, "Calculate volumes")
		dlg.setLimits(0.0, rt.nstruct)
		try:
			rt.calcVolumes(lambda i,d=dlg:d.show(i))
		except tk.TclError:
			pass
		dlg.stop()

		for idx in sorted(list(rt.idx.values())):
			name  = rt.names[idx]
			d,v,p = rt.volumes[idx]
			v /= 1000.0
			p /= 1000.0
			if d>0.0:
				dp = 100.0*(d/p-1.0)
			else:
				dp = 0.0
			try:
				vp = 100.0*(v/p-1.0)
			except ZeroDivisionError:
				vp = 0.0
			cache[f"#{idx} {name}"] = (round(d,4),
						round(v,4),
						round(p,4),
						round(dp,2),
						round(vp,2))

	# --------------------------------------------------------------------
	# Fill tree with information
	# --------------------------------------------------------------------
	def treeFill(self):
		cache = self.page.dicominfo.cache
		sections = {""}
		for key,value in cache.items():
			if key[0] == "@": continue
			try:
				section,name = key.split(":")
			except ValueError:
				section = ""
				name = key
			if section not in sections:
				self.tree.insert('',tk.END, section, text=section,
							tag="section", open=True)
				sections.add(section)
			if isinstance(value, str):
				value = (value,)
			self.tree.insert(section, tk.END, text=name, values=value)
		self.tree.tag_configure("section", foreground="DarkRed")
