#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   12-Sep-2013

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import os
import time
import tkinter as tk

import icons
import Ribbon
import tkExtra

#import Global
import Manual
import tkFlair
import Project

#===============================================================================
# Tab pages inside a Page with a dedicated Ribbon
#===============================================================================
class FlairTab(tk.Frame):
	#----------------------------------------------------------------------
	def __init__(self, master, page, name, **kw):
		super().__init__(master, **kw)
		self.page  = page
		self.name  = name
		self.flair = page.flair
		# FIXME replace with property
		self.project = page.project

	#----------------------------------------------------------------------
	#@property
	#def project(self):
	#	return self.flair.project

	#----------------------------------------------------------------------
	def createRibbon(self):
		self.ribbon = tk.Frame(self.page.ribbon,
					padx=0,
					pady=0,
					background=Ribbon._BACKGROUND)
		return self.ribbon

	#----------------------------------------------------------------------
	# Called when tab is activated
	#----------------------------------------------------------------------
	def activate(self):
		pass

	#----------------------------------------------------------------------
	# Called when tab is de-activated
	#----------------------------------------------------------------------
	def deactivate(self):
		pass

	#----------------------------------------------------------------------
	def execute(self):
		pass

#===============================================================================
# Flair dynamic menu
#===============================================================================
class FlairMenu(Ribbon.MenuButton):
	def __init__(self, master, page, **kw):
		super().__init__(master, None, **kw)
		self.page = page

#===============================================================================
# Flair menu group
#===============================================================================
class FlairMenuGroup(Ribbon.MenuGroup):
	def __init__(self, master, page, name, menulist=None, **kw):
		super().__init__(master, name, menulist, **kw)
		self.page = page

#===============================================================================
# Input template menu button
#===============================================================================
class _TemplateMenuButton(FlairMenu):
	#----------------------------------------------------------------------
	def createMenu(self):
		# Find templates
		files = {}
		for path in (os.path.join(tkFlair.prgDir, tkFlair._TEMPLATES_DIR),
			     os.path.join(tkFlair.iniDir, tkFlair._TEMPLATES_DIR)):
			try:
				for filename in os.listdir(path):
					(fn, ext) = os.path.splitext(filename)
					if ext == Project.EXT:
						files[fn] = path
			except OSError:
				pass

		menu = tk.Menu(self, tearoff=0, activebackground=Ribbon._ACTIVE_COLOR)
		for fn in sorted(files):
			f = os.path.join(files[fn], fn+Project.EXT)
			cmd = lambda f=f,p=self.page : p.openTemplate(f)
			menu.add_command(label=fn,
					image=icons.get(Project.Project.projectMode(f)),
					compound=tk.LEFT,
					command=cmd)
		return menu

#===============================================================================
# Flair Tab Page
#===============================================================================
class FlairPage(Ribbon.Page):		# <--- should be possible to be a toplevel as well
	active = None

	_name_ = "Page"			# To be overridden by the end class
	_icon_ = "empty"

	#----------------------------------------------------------------------
	def __init__(self, flair, name, icon, show, **kw):
		super().__init__(name, icon, show, **kw)
		self.flair    = flair
		self.project  = flair.project	# FIXME replace with property
		self._title   = []
		self._tabs    = []
		self.tabVar   = tk.IntVar()
		self.tabVar.set(0)
		self.activeTab   = None
		# Interrupt long process
		self._interrupt  = False
		self._longNotify = False

	#----------------------------------------------------------------------
	#@property
	#def project(self):
	#	return self.flair.project

	#----------------------------------------------------------------------
	def createPage(self, scrollx=False, scrolly=False):
		super().createPage()
		#scrollx = scrolly = False
		self._title = []
		self._tabs  = []
		self.tabVar.set(0)
		self.activeTab = None

		# Do not show title when undocked
		if not self._undock:
			f = tk.Frame(self.page, relief=tk.GROOVE)
			f.grid(row=0, column=0, columnspan=2, sticky=tk.NSEW)
			self._title.append(f)

			# --- Pin --- and close button ---
			self._pinbtn = tk.Button(f, text="\N{BULLSEYE}",
				command=self.pinButton,
				font=tkFlair._TITLE_FONT,
				border=0, highlightthickness=0,
				foreground=tkFlair._TITLE_FOREGROUND_COLOR,
				width=3,
				takefocus=False, padx=1, pady=1)
			tkExtra.Balloon.set(self._pinbtn, "Pin page to remain always visible")
			self.showPin()
			self._title.append(self._pinbtn)
			self._title[-1].pack(side=tk.LEFT, fill=tk.Y)

			# --- title ---
			if FlairPage.active is self:
				bg=tkFlair._TITLE_BACKACTIVE_COLOR
			else:
				bg=tkFlair._TITLE_BACKINACTIVE_COLOR

			self._title.append(tk.Label(f, text=self.name,
					font=tkFlair._TITLE_FONT,
					foreground=tkFlair._TITLE_FOREGROUND_COLOR,
					background=bg,
					justify="center",
					padx=10,
					pady=1))
			self._title[-1].bind("<Button-1>",        self._dragStart)
			self._title[-1].bind("<B1-Motion>",       self._dragMove)
			self._title[-1].bind("<ButtonRelease-1>", self._dragEnd)
			self._title[-1].bind("<Double-1>",        self.minmaxButton)
			self._title[-1].pack(side=tk.LEFT, expand=tk.YES, fill=tk.X)

			# --- close ---
			self._title.append(tk.Button(f, text="\N{VECTOR OR CROSS PRODUCT}",
					command=self.close,
					font=tkFlair._TITLE_FONT,
					border=0, highlightthickness=0,
					width=2,
					foreground=tkFlair._TITLE_FOREGROUND_COLOR,
					takefocus=False, padx=1, pady=1))
			self._title[-1].pack(side=tk.RIGHT, fill=tk.Y)
			tkExtra.Balloon.set(self._title[-1], "Close page and display the previous one")

			# --- max/min ---
			self._minmax = tk.Button(f, text="\N{BLACK UP-POINTING TRIANGLE}",
				command=self.minmaxButton,
				font=tkFlair._TITLE_FONT,
				border=0, highlightthickness=0,
				width=2,
				foreground=tkFlair._TITLE_FOREGROUND_COLOR,
				takefocus=False, padx=1, pady=1)
			self._title.append(self._minmax)
			self._title[-1].pack(side=tk.RIGHT, fill=tk.Y)
			tkExtra.Balloon.set(self._title[-1], "Maximize/Minimize page")

		else:
			self._pinbtn = None
			self._minmax = None

		# Do we need a scroll frame?
		if scrollx or scrolly:
			self._scrollFrame = tkExtra.ScrollFrame(self.page, takefocus=True)
			self._scrollFrame.grid(row=1, column=0, sticky=tk.NSEW, padx=2, pady=2)

			self.frame = self._scrollFrame()
			self.frame.config(takefocus=True)

			if scrollx:
				sbx=tkExtra.AutoScrollbar(self.page,
						orient=tk.HORIZONTAL,
						command=self._scrollFrame.xview)
				sbx.grid(row=2,column=0,sticky=tk.NSEW)
				self._scrollFrame["xscrollcommand"] = sbx.set

			if scrolly:
				sby=tkExtra.AutoScrollbar(self.page,
						orient=tk.VERTICAL,
						command=self._scrollFrame.yview)
				sby.grid(row=1,column=1,sticky=tk.NSEW)
				self._scrollFrame["yscrollcommand"] = sby.set

		else:	# Or fixed
			self._scrollFrame = None
			self.frame = tk.Frame(self.page, takefocus=True)
			self.frame.grid(row=1, column=0, columnspan=2, sticky=tk.NSEW)

		self.page.grid_columnconfigure(0, weight=1)
		self.page.grid_rowconfigure(1, weight=1)

		self.frame.bind("<FocusIn>",	self._focusIn)
		self.page.bind("<Map>",		self.mapped)

	#----------------------------------------------------------------------
	def createRibbon(self):
		super().createRibbon()
		for tab in self._tabs:
			tab.createRibbon()

	#----------------------------------------------------------------------
	def createStatus(self):
		super().createStatus()

		self.inputLabel = tk.Label(self.status, relief=tk.SUNKEN,
				width=32,
				justify=tk.LEFT, anchor=tk.W, takefocus=0)
		self.inputLabel.pack(side=tk.LEFT, fill=tk.X)

		b = tk.Button(self.status,image=icons.get("config"),
				command=self.preferences)
		b.pack(side=tk.RIGHT)
		tkExtra.Balloon.set(b, "Open configuration dialog")
		b = tk.Button(self.status,image=icons.get("output"),
				command=self.showOutput)
		b.pack(side=tk.RIGHT)
		tkExtra.Balloon.set(b, "Show output [Ctrl-o]")

		self.statusLabel = tk.Canvas(self.status,
						relief=tk.SUNKEN,
						border=1)
		self._statusText = self.statusLabel.create_text(2,2,
						text="",
						anchor=tk.NW)
		bounds = self.statusLabel.bbox(self._statusText)
		height = bounds[3]-bounds[1]
		self.statusLabel.config(height=height)
		self._statusPercBar  = self.statusLabel.create_rectangle(0, 0, 0, 0,
					fill="green3",
					width=0)
		self.statusLabel.tag_lower(self._statusPercBar)
		self.statusLabel.pack(side=tk.RIGHT, expand=tk.YES, fill=tk.BOTH)
		self.resetPercent()

		return self.status

	#----------------------------------------------------------------------
	def preferences(self):
		self.master.mastertab.event_generate("<<Preferences>>")

	#----------------------------------------------------------------------
	def showOutput(self):
		self.master.mastertab.event_generate("<<Output>>")

	#----------------------------------------------------------------------
	def addTab(self, tab):
		self._tabs.append(tab)

	#----------------------------------------------------------------------
	def changeTab(self, tab=None):
		#Global.dump("*-* FlairPage.changeTab",tab,self._tabs)
		if not self._tabs: return
		if tab is not None:
			self.tabVar.set(tab)
		if self.activeTab and self.activeTab is not self._tabs[self.tabVar.get()]:
			self.activeTab.deactivate()
			self.activeTab.pack_forget()
			self.activeTab.ribbon.pack_forget()
		self.activeTab = self._tabs[self.tabVar.get()]
		self.activeTab.pack(expand=tk.YES, fill=tk.BOTH)
		self.activeTab.ribbon.pack(side=tk.LEFT, fill=tk.BOTH)
		self.activeTab.activate()

	# ----------------------------------------------------------------------
	def cut(self, event=None):
		f = self.page.focus_get()
		if event and f is event.widget: return	# Stop infinite loop
		if f: f.event_generate("<<Cut>>")

	# ----------------------------------------------------------------------
	def copy(self, event=None):
		f = self.page.focus_get()
		if event and f is event.widget: return	# Stop infinite loop
		if f: f.event_generate("<<Copy>>")

	# ----------------------------------------------------------------------
	def paste(self, event=None):
		f = self.page.focus_get()
		if event and f is event.widget: return	# Stop infinite loop
		if f: f.event_generate("<<Paste>>")

	# ----------------------------------------------------------------------
	def clone(self, event=None):
		self.copy(event)
		self.paste(event)

	# ----------------------------------------------------------------------
	def help(self):
		Manual.show("",self.name)

	# ----------------------------------------------------------------------
	def undo(self, event=None):
		f = self.page.focus_get()
		if event and f is event.widget: return	# Stop infinite loop
		if f: f.event_generate("<<Undo>>")

	# ----------------------------------------------------------------------
	def redo(self, event=None):
		f = self.page.focus_get()
		if event and f is event.widget: return	# Stop infinite loop
		if f: f.event_generate("<<Redo>>")

	# ----------------------------------------------------------------------
	def busy(self):
		self.flair.busy()

	# ----------------------------------------------------------------------
	def notBusy(self):
		self.flair.notBusy()

	# ----------------------------------------------------------------------
	def minmaxButton(self, event=None):
		self.activate()
		self.master.maximize(self)
		self.showminmax()

	# ----------------------------------------------------------------------
	def showminmax(self):
		if self._minmax is None: return
		if self.page is self.master._pageFrame.maxchild():
			self._minmax["text"] = "\N{BLACK DOWN-POINTING TRIANGLE}"
		else:
			self._minmax["text"] = "\N{BLACK UP-POINTING TRIANGLE}"

	# ----------------------------------------------------------------------
	def pinButton(self, event=None):
		self.pinned = not self.pinned
		#if not self.pinned: self.master.hide(self)
		self.showPin()

	# ----------------------------------------------------------------------
	def setpin(self, pin):
		super().setpin(pin)
		self.showPin()

	# ----------------------------------------------------------------------
	def close(self):
		self.master.remove(self)

	# ----------------------------------------------------------------------
	def hide(self, replace):
		self.master.hide(self, replace)

	#----------------------------------------------------------------------
	def afterDock(self):
		super().afterDock()
		self.refreshTitle()

	# ----------------------------------------------------------------------
	def showPin(self):
		if self._pinbtn:
			if self.pinned:
				self._pinbtn.config(text="\N{FISHEYE}")
			else:
				self._pinbtn.config(text="\N{BULLSEYE}")

	# ----------------------------------------------------------------------
	# Show busy cursor and execute
	# ----------------------------------------------------------------------
	def execute(self):
		#self._setFocus()
		self.busy()
		self._execute()
		self.notBusy()

	# ----------------------------------------------------------------------
	# Override execute function
	# ----------------------------------------------------------------------
	def _execute(self):
		if self.activeTab is not None:
			self.activeTab.execute()

	# ----------------------------------------------------------------------
	# Event handlers for dragging and placing
	# Bind to <Button-1>
	# ----------------------------------------------------------------------
	def _dragStart(self, event):
		self.activate()
		self.master._pageFrame.dragStart(event)

	# ----------------------------------------------------------------------
	# Bind to <B1-Motion>
	# ----------------------------------------------------------------------
	def _dragMove(self, event):
		self.master._pageFrame.dragMove(event)

	# ----------------------------------------------------------------------
	# Bind to <ButtonRelease-1>
	# ----------------------------------------------------------------------
	def _dragEnd(self, event):
		wasdragging = self.master._pageFrame._drag is not None
		if not self.master._pageFrame.dragEnd(event) and wasdragging:
			# Check if outside to undock it
			if event.x < -10 or event.y <-10 or \
			   event.x > self.master.winfo_width() + 10 or \
			   event.y > self.master.winfo_height() + 10:
				self.master.undock(self.name, event.x_root, event.y_root)

	# ----------------------------------------------------------------------
	def reset(self): pass

	# ----------------------------------------------------------------------
	# Refresh window contents
	# ----------------------------------------------------------------------
	def refresh(self, what=None):
		if self.page is None: return
		self.changeTab()
		self.refreshUndoButton()
		self._invalid = None

	# ----------------------------------------------------------------------
	# Redraw window contents
	# ----------------------------------------------------------------------
	redraw = refresh

	#----------------------------------------------------------------------
	def refreshTitle(self):
		if self.page is None: return
		if self.project.filename == "":
			title = "[untitled]"
		else:
			title = self.project.filename
		try: self.master.master.title(f"{title} - {self.name}" )
		except tk.TclError: pass

	#----------------------------------------------------------------------
	def refreshStatus(self):
		if self.page is None: return
		mode = self.project.mode().capitalize()
		name = os.path.basename(self.project.filename)
		if self.project.external:
			ext = "\N{BLACK RIGHT-POINTING TRIANGLE} %s" % \
				(os.path.basename(self.project.external.filename))
		else:
			ext = ""
		self.inputLabel.config(text=f"{mode}: {name}{ext}")

	# ----------------------------------------------------------------------
	# Set status message
	# ----------------------------------------------------------------------
	def setStatus(self, txt="", fg="#000000", bg=None, update=False):
		if self.page is None: return
		try:
			self.statusLabel.itemconfig(self._statusText, text=txt, fill=fg)
			if bg is None:
				self.statusLabel.config(
					bg=self.statusLabel.winfo_toplevel().cget("bg"))
			else:
				self.statusLabel.config(bg=bg)
			if update:
				self.statusLabel.update_idletasks()
		except tk.TclError:
			pass

	# ----------------------------------------------------------------------
	def resetPercent(self, percent_max=100.0, update=False):
		self._percent     = 0
		self._percentTime = 0
		self._percentMax  = percent_max
		self.setStatusPercent(0, update)

	# ----------------------------------------------------------------------
	# Set percentage of the status label
	# ----------------------------------------------------------------------
	def setStatusPercent(self, percent, update=False):
		width  = self.statusLabel.winfo_width()
		height = self.statusLabel.winfo_height()

		if (self._percent != percent and time.time()-self._percentTime > 0.1) or \
		   percent == 0:
			self._percent = min(percent, self._percentMax)
			self._percentTime = time.time()
			w = int((width*percent) / self._percentMax)
			self.statusLabel.coords(self._statusPercBar, 0, 0, w, height)
			if update:
				self.statusLabel.update_idletasks()

	# ----------------------------------------------------------------------
	def getStatusPercent(self):
		return self._percent

	# ----------------------------------------------------------------------
	# Update contents
	# ----------------------------------------------------------------------
	def get(self):
		pass

	# ----------------------------------------------------------------------
	def modified(self, m=True):
		self.flair.modified(m)

	# ----------------------------------------------------------------------
	# Set focus, activate page if it is mapped
	# ----------------------------------------------------------------------
	def _focusIn(self, event=None):
		if self.page and self.page.winfo_ismapped():
			self.activate()

	# ----------------------------------------------------------------------
	def isactive(self):
		return FlairPage.active is self

	# ----------------------------------------------------------------------
	@staticmethod
	def clearActive():
		FlairPage.active = None

	# ----------------------------------------------------------------------
	def activate(self):
		#Global.dump("*-* FlairPage.activate",self.name)
		if FlairPage.active is self: return
		super().activate()

		if FlairPage.active is not None:
			FlairPage.active.deactivate()
		FlairPage.active = self

		for w in self._title:
			try: w["background"] = tkFlair._TITLE_BACKACTIVE_COLOR
			except tk.TclError: pass
		self.showminmax()
		#if self._scrollFrame is not None:
		#	self._scrollFrame.update_scrollregion()
		if self.master.activePage.get() != self.name:
			self.master.changePage(self)

		# Get the focus if needed
		try:
			found = False
			focus = self.page.focus_get()
			while focus is not None:
				focus = focus.master
				if focus is self.page:
					found = True
					break
			if not found: self.page.focus_set()
		except tk.TclError:
			pass

		self.refreshUndoButton()
		self.refreshStatus()

		if self.invalid(): self.refresh(self._invalid)

		try: os.waitpid(-1,os.WNOHANG)	# Cleanup child processes
		except OSError: pass

	# ----------------------------------------------------------------------
	def deactivate(self):
		#print("\ndeactivate",self.name, self.page)
		for w in self._title:
			try: w["background"] = tkFlair._TITLE_BACKINACTIVE_COLOR
			except tk.TclError: pass
		#if self.page and self.page.winfo_ismapped(): self.get()
		if self.page: self.get()

	# ----------------------------------------------------------------------
	def saveConfig(self, config):
		pass

	# ----------------------------------------------------------------------
	# Reset interrupt
	# ----------------------------------------------------------------------
	def resetInterrupt(self):
		if self._longNotify:
			self.flair.notifyHide()
		self._interrupt  = False
		self._longNotify = False

	# ----------------------------------------------------------------------
	# Interrupt a long process from notify
	# ----------------------------------------------------------------------
	def interrupt(self):
		if self._longNotify:
			self.flair.notifyHide()
		self._interrupt  = True
		self._longNotify = False

	# ----------------------------------------------------------------------
	# Notify if process takes too long time
	# ----------------------------------------------------------------------
	def interruptNotify(self, timeout=5):
		if not self.flair.notifyIsVisible():
			self._longNotify = True
			self.flair.notify("Too long",
				"Process takes too long",
				tkFlair.NOTIFY_NORMAL,
				"Interrupt?", self.interrupt)
		return self._interrupt

#===============================================================================
# Flair Dockable Dialog in the main window?
#===============================================================================
#class FlairDialog(FlairPage):
#	active = None
#
#	_name_ = "Page"			# To be overridden by the end class
#	_icon_ = "empty"
#
#	#----------------------------------------------------------------------
#	def __init__(self, flair, name, icon, show, **kw):
#		super().__init__(flair, name, icon, show, **kw)
