#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   27-Jan-2012

#from enum import Enum, unique
import ansi

_ansi        = True		# show ansi coloring in dump
developer    = False		# Activate development mode compatibility
experimental = False		# Activate experimental beta features
profiler     = False		# Activate profiling/timing options in code
verbose      = 0		# Activate verbose output
tmpPrefix    = "flair_"		# Commands and tmp files
DND_TIME     = 250		# time for starting DND

#-------------------------------------------------------------------------------
# dump debug messages when develop flag is on
#-------------------------------------------------------------------------------
def dump(*args, bold=False, level=1):
	if verbose>=level:
		if bold and _ansi: print(ansi.BOLD, end="")
		print(*args)	# dump
		if bold and _ansi: print(ansi.NORMAL, end="")

#===============================================================================
# Viewport constants
#===============================================================================
class VIEWER:
	NAME  = ["Red",   "Green",  "Blue",   "Magenta"]
	COLOR = [0xFF7070, 0x70FF70, 0x7070FF, 0xFF70FF]

#===============================================================================
# Bit flags for object selection
# XXX WARNING should be the same as in the geoviewer XXX
#===============================================================================
class BIT:
	SELECT      = 1
	FREEZE      = 1<<1
	LOCK        = 1<<2
	VISIBLE     = 1<<3
	WIREFRAME   = 1<<4
	MOVE        = 1<<5
	BBOX        = 1<<6

	SELFREEZE   =  SELECT | FREEZE
	UNSELECT    = ~SELECT
	UNLOCK      = ~LOCK
	UNFREEZE    = ~FREEZE
	UNVISIBLE   = ~VISIBLE
	UNWIREFRAME = ~WIREFRAME
	UNBBOX      = ~BBOX

#===============================================================================
# Draw flags
# XXX WARNING should be the same as in the geoviewer XXX
#===============================================================================
#@unique
class DRAW: #(Enum):
	CLEAR     = 1
	SEGMENTS  = 1<<1
	REGIONS   = 1<<2
	GRID      = 1<<3
	AXES      = 1<<4
	TITLE     = 1<<5
	STATUS    = 1<<6
	LABELS    = 1<<7
	IMAGE     = 1<<8
	USRBIN    = 1<<9
	_3D       = 1<<10
	EDGE      = 1<<11
	VERTICES  = 1<<12
	MGDRAW    = 1<<13
	COLORBOX  = 1<<14
	LATTICES  = 1<<15
	VOXEL     = 1<<16
	WIREFRAME = 1<<17
	BBOX      = 1<<18
	MESSAGE   = 1<<19
	TEST      = 1<<30
	FAST      = _3D|WIREFRAME|BBOX|CLEAR|GRID

#===============================================================================
# Flags for selecting objects with the mouse
#===============================================================================
class SELECT:
	UNSELECT = -1		# unselect
	SELECT   =  0		# default: select items
	TOGGLE   =  1		# toggle selection

#===============================================================================
# Actions
# **Negative** is for sub-task, not to hide the task dialog
#===============================================================================
#@unique
class ACTION: #(Enum):
	# Selections
	SELECT          =   0	# Default action (select)
	AREA_SELECT     =   1	# Select bodies or regions using rectangle
	SEL_OR_MOVE     =   2	# Still fuzzy if we want to select or to move
	SEL_OR_AREA     =   3	# Still fuzzy if we want to select on point or an area
	SELECT_REGION   =  -4	# Select a region
	SELECT_ZONE     =  -5	# Select an existing zone from a region
	SELECT_BODY     =  -6	# Select bodies
	SELECT_OBJECT   =  -7	# Select objects
	SELECT_SPLINE   =  -8	# Select spline object
	SELECT_VIEWPORT =  -9	# Viewport selection
	SELECT_NODE     = -10	# Select a node from object

	# Display
	PAN             =  10	# Pan display
	PAN_SLOWDOWN    =  11	# Pan slowdown
	ORBIT           =  12	# Orbit (trackball) movement
	PIVOT           =  13	# Set orbit pivot point
	ZOOM_IN         =  14	# Increase zoom level
	ZOOM_OUT        =  15	# Decrease zoom level
	ZOOM_ON         =  16	# Zoom on selected objects
	CENTER          =  17	# Center viewports to location

	# Viewport lines
	VIEW_ORIGIN     =  20	# Move viewport center
	VIEW_MOVE       =  21	# Move viewport line leaving center unchanged in other axis
	VIEW_ROTATE     =  22	# Rotate viewport line

	# Addition
	ADD             =  30	# Add an object
	ADD_NEXT        = -31	# Add next point of object
	ADD_ZONE        = -32	# Add/Edit a new zone
	ADD_ZONEPAINT   = -33	# Add multiple zones
	ADD_SPLINE_NODE = -34	# Spline add node
	DEL_SPLINE_NODE = -35	# Spline delete node

	# Objects manipulation
	EDIT            =  40	# Edit/Correct overlaps
	MOVE            =  41	# Move selected objects
	MOVE_KEYBOARD   =  42	# Move object with keyboard
	ROTATE          =  43	# Rotate selected objects
	REPEAT          =  44	# Repeat and distribute selected objects
	LINK            =  45	# Link node with spline

	# Other editing
	ZONE            =  50	# Select zone master action
	PEN             =  52	# Draw using pen
	#PAINT           =  53	# Paint region properties
	INFO            =  54	# Display info at cursor
	VOLUME          =  55	# Display volume calculator tool
	EXPORT          =  56	# Export selected viewport
	MOVIE           =  57	# Create movie from selected viewport
	PLANNER         =  58	# Dose planner reporting tool

	STOP            = 999	# Stop any action and return to selection
