#!/bin/env python3
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Wioletta.Kozlowska@cern.ch
# Date:   Feb-2017

import tkinter as tk

import icons
import Ribbon
import tkFlair
import tkExtra
import FlairRibbon
import bFileDialog
import OptimizerProcess

try:
	import geoviewer
except ImportError:
	geoviewer = None

OPTALG={
		"Difference Dose"               : 1,
		"Difference Dose + RBE"         : 2,
		"Plain Gradient Dose"           : 3,
		"Plain Gradient Dose + RBE"     : 4,
		"Difference Dose + RBE v.2"     : 5,
		"Plain Gradient Dose + RBE v.2" : 6,
		"Plain Gradient RBE only"       : 7
	}

CELL_LINE={
		"Add cell line file"                        :     0,
		"Chordoma LEM"                              :     1,
		"Furusawa HSG"                              :     2,
		"Chordoma LEM IV"                           :     3,
		"Liver Dt-30Gy"                             :     4,
		"Prostate Dt-20Gy"                          :     5,
		"V79 Dt-4Gy"                                :     6,
		"Prostate Dt-30Gy"                          :     7,
		"Liver Dt-40Gy"                             :     8,
		"Nt Dt-20Gy"                                :     9,
		"Furusawa HSG with alpha scaled"            :    20,
		"Furusawa HSG with beta scaled"             :    21,
		"Furusawa HSG with a/b=2 by varying alpha"  : 22002,
		"Furusawa HSG with a/b=10 by varying alpha" : 22010
	   }

#===============================================================================
# Optimizer Page
#===============================================================================
class OptimizerPage(FlairRibbon.FlairPage):
	"""Optimize Treatment Plan"""
	_name_ = "Optimizer"
	_icon_ = "pet"

	#---------------------------------------------------------------------
	def init(self):
		if geoviewer is not None:
			self.opt = geoviewer.Optimizer()
		else:
			self.opt = None

	#----------------------------------------------------------------------
	def createRibbon(self):
		super().createRibbon()

		# ========== Executable ===========
		groupInfo = Ribbon.LabelGroup(self.ribbon, "RunOptimizer")
		groupInfo.label["background"] = Ribbon._BACKGROUND_GROUP2
		groupInfo.pack(side=tk.LEFT, fill=tk.Y, padx=0, pady=0)

		col, row = 0,0
		b = Ribbon.LabelButton(groupInfo.frame, image=icons.get("gear32"),
				text="Run",
				compound=tk.TOP,
				command=self.execute,
				background=Ribbon._BACKGROUND)
		b.grid(row=row, column=col, sticky=tk.E)
		tkExtra.Balloon.set(b, "Run Optimizer")

		self._updated = False

	#----------------------------------------------------------------------
	# Create Optimizer page
	#----------------------------------------------------------------------
	def createPage(self):
		super().createPage()

		self.lastPath =""

		self.splitter = tkExtra.HSplitter(self.frame,
						tkFlair.config.getInt(tkFlair._PAGE_SECTION,
						self.name+".split", 240),
						True)
		self.splitter.pack(expand=tk.YES, fill=tk.BOTH)

		#-----------------------Output----------------------------------
		self.output=tkExtra.tk.LabelFrame(self.splitter, text = "Output")
		self.output.pack(fill=tk.BOTH, expand=tk.YES, side=tk.LEFT)

		self.output = tk.Text(self.output, bg='black', bd=3, fg='white', height = 40)

		#-----------------------Input-----------------------------------
		inputFrame=tkExtra.tk.LabelFrame(self.splitter, text="Input")
		inputFrame.pack(fill=tk.BOTH, expand=tk.YES)

		#-------------------------Input/Output---------------------------------
		rowL,colL=0,0
		fileLabel=tkExtra.tk.LabelFrame(inputFrame, text="Input/Output")
		fileLabel.grid(row=rowL, column=colL,sticky=tk.EW)

		row,col=0,0
#               tk.Label(fileLabel, text="Input Directory:").grid(
#                       row=row, column=col, sticky=tk.W)
#               col+=1
#               self.inputDir = tk.StringVar()
#               inputLabel= tk.Label(fileLabel,
#                                 width=30,relief=tk.SUNKEN, anchor=tk.W,
#                                 textvariable=self.inputDir).grid(
#                       row=row, column=col, sticky=tk.W)
#                col+=1
#                b=tk.Button(fileLabel,image=icons.get("load"), pady=0,
#				command=self.loadInputDir)
#		b.grid(row=row, column=col)

		col =0
		row+=1
		tk.Label(fileLabel, text="Output Directory:").grid(
			row=row, column=col, sticky=tk.W)
		col+=1
		self.outputDir = tk.StringVar()
		tk.Label(fileLabel,
				  width=30,relief=tk.SUNKEN, anchor=tk.W,
				  textvariable=self.outputDir).grid(
			row=row, column=col, sticky=tk.W)
		col+=1
		b=tk.Button(fileLabel,image=icons.get("load"), pady=0,
				command=self.loadOutputDir)
		b.grid(row=row, column=col)

		col =0
		row+=1
		self.PBFile=tk.StringVar()
		tk.Label(fileLabel, text="Pencil Beam:").grid(row=row,column=col, sticky=tk.W)
		col+=1
		self.PBFile = tk.StringVar()
		tk.Label(fileLabel,
				width=30,relief=tk.SUNKEN, anchor=tk.W,
				textvariable=self.PBFile).grid(
			row=row, column=col, sticky=tk.W)
		col+=1
		b=tk.Button(fileLabel,image=icons.get("load"), pady=0,
			 command=self.loadPBFile)
		b.grid(row=row, column=col)

		#----------------------------Files--------------------------------
		rowL+=1
		scoreLabel=tkExtra.tk.LabelFrame(inputFrame, text="Scoring files")
		scoreLabel.grid(row=rowL,column=colL, sticky=tk.EW)

		row,col=0,0
		self.DoseFile=tk.StringVar()
		tk.Label(scoreLabel, text="Dose File:").grid(row=row,column=col, sticky=tk.W)
		tk.Label(scoreLabel,
				width=30,relief=tk.SUNKEN, anchor=tk.W,
				textvariable=self.DoseFile).grid(
					row=row, column=col+1, sticky=tk.W)
		b=tk.Button(scoreLabel,image=icons.get("load"), pady=0,
			 command=self.loadDoseFile)
		b.grid(row=row, column=col+2)

		row+=1
		self.AlphaFile=tk.StringVar()
		tk.Label(scoreLabel, text="Alpha File:").grid(row=row,column=col, sticky=tk.W)
		tk.Label(scoreLabel,
				width=30,relief=tk.SUNKEN, anchor=tk.W,
				textvariable=self.AlphaFile).grid(
					row=row, column=col+1, sticky=tk.W)
		b=tk.Button(scoreLabel,image=icons.get("load"), pady=0,
			 command=self.loadAlphaFile)
		b.grid(row=row, column=col+2)

		row+=1
		self.BetaFile=tk.StringVar()
		tk.Label(scoreLabel, text="Beta File:").grid(row=row,column=col, sticky=tk.W)
		tk.Label(scoreLabel,
				width=30,relief=tk.SUNKEN, anchor=tk.W,
				textvariable=self.BetaFile).grid(
					row=row, column=col+1, sticky=tk.W)
		b=tk.Button(scoreLabel,image=icons.get("load"), pady=0,
			 command=self.loadBetaFile)
		b.grid(row=row, column=col+2)

		row+=1
		self.LetFile=tk.StringVar()
		tk.Label(scoreLabel, text="Let File:").grid(row=row,column=col, sticky=tk.W)
		tk.Label(scoreLabel,
				width=30,relief=tk.SUNKEN, anchor=tk.W,
				textvariable=self.LetFile).grid(
					row=row, column=col+1, sticky=tk.W)
		b=tk.Button(scoreLabel,image=icons.get("load"), pady=0,
			 command=self.loadLetFile)
		b.grid(row=row, column=col+2)

		#------------------------Voxel Files--------------------------------
		rowL+=1
		planLabel=tkExtra.tk.LabelFrame(inputFrame, text="Plan files")
		planLabel.grid(row=rowL,column=colL, sticky=tk.EW)

		inpFileLabel=tkExtra.tk.LabelFrame(planLabel, borderwidth=0)
		inpFileLabel.grid(row=rowL,column=colL, sticky=tk.EW)

		row,col=0,0

		self.VxlFile=tk.StringVar()
		tk.Label(inpFileLabel, text="Voxel File:").grid(row=row,column=col, sticky=tk.W)
		tk.Label(inpFileLabel,
				width=30,relief=tk.SUNKEN, anchor=tk.W,
				textvariable=self.VxlFile).grid(
					row=row, column=col+1, sticky=tk.W)
		b=tk.Button(inpFileLabel,image=icons.get("load"), pady=0,
			 command=self.loadVxlFile)
		b.grid(row=row, column=col+2)

		#----------------------------------------------------
		rowL+=1
		posLabel=tkExtra.tk.LabelFrame(planLabel, borderwidth=0)
		posLabel.grid(row=rowL,column=colL, sticky=tk.EW)

		row+=1
		tk.Label(posLabel, text="Xmin:").grid(row=row,column=col, sticky=tk.W)
		col+=1
		self.nXmin = tkExtra.FloatEntry(posLabel, width=5, background="White")
		self.nXmin.insert(tk.END,0.0)
		self.nXmin.grid(row=row, column=col, sticky=tk.W)

		col+=1
		tk.Label(posLabel, text="Ymin:").grid(row=row,column=col, sticky=tk.W)
		col+=1
		self.nYmin = tkExtra.FloatEntry(posLabel, width=5, background="White")
		self.nYmin.insert(tk.END,0.0)
		self.nYmin.grid(row=row, column=col, sticky=tk.W)

		col+=1
		tk.Label(posLabel, text="Zmin:").grid(row=row,column=col, sticky=tk.W)
		col+=1
		self.nZmin = tkExtra.FloatEntry(posLabel, width=5, background="White")
		self.nZmin.insert(tk.END,0.0)
		self.nZmin.grid(row=row, column=col, sticky=tk.W)

		#---------------------------------------------------
		rowL+=1
		roiLabel=tkExtra.tk.LabelFrame(planLabel, borderwidth=0)
		roiLabel.grid(row=rowL,column=colL, sticky=tk.EW)

		rowR = 0
		colR = 0

		roiTable=tkExtra.tk.LabelFrame(roiLabel, borderwidth=0)
		roiTable.grid(row=rowR,column=colR, sticky=tk.EW)

		row = 0
		col = 0

		self.roiList = tkExtra.MultiListbox(roiLabel,
						(("RoiId", 3, None),
						 ("IsPtv", 2, None),
						 ("Dose",  3, None),
						 ("RBE",   3, None),
						("LET",    3, None)))
		self.roiList.grid(row=row, column=col, stick=tk.EW)
		self.roiList.delete(0,tk.END)

		colR+=1
		rowR =0

		row = 0
		col = 0

		roiButton=tkExtra.tk.LabelFrame(roiLabel, borderwidth=0)
		roiButton.grid(row=rowR,column=colR, sticky=tk.EW)

		tk.Button(roiButton, text="Add", command=self.addRoi).grid(
			row=row, column=col, stick=tk.EW)
		row+=1
		tk.Button(roiButton, text="Edit", command=self.editRoi).grid(
			row=row, column=col, stick=tk.EW)
		row+=1
		tk.Button(roiButton, text="Delete", command=self.deleteRoi).grid(
			row=row, column=col, stick=tk.EW)
		row+=1
		tk.Button(roiButton, text="Process", command=self.processRoi).grid(
			row=row, column=col, stick=tk.EW)
		rowL+=1

		#-----------------------Optimization Methods--------------------------------
		optLabel=tkExtra.tk.LabelFrame(inputFrame, text="Optimization Methods")
		optLabel.grid(row=rowL, column=colL,sticky=tk.EW)

		row,col =0,0
#                self.nOptAlg=tk.IntVar()
		self.optAlg = tk.StringVar()
		tk.Label(optLabel, text="Algorithm:").grid(row=row,column=col, sticky=tk.W)
		self.optAlg = tkExtra.Combobox(optLabel, width=30,
				command=self.setOptAlg)
		self.optAlg.grid(row=row,column=col+1,sticky=tk.W)
		self.optAlg.fill(sorted(OPTALG))
		self.optAlg.select(0)

		row+=1
		self.nAction = tk.IntVar()
		tk.Label(optLabel, text="Iteration Intervals:").grid(row=row,column=col, sticky=tk.W)
		self.nAction = tkExtra.IntegerEntry(optLabel, width=10, background="White")
		self.nAction.insert(tk.END, 20)
		self.nAction.grid(row=row, column=col+1, sticky=tk.W)

		row+=1
		tk.Label(optLabel, text="Convergence Criterium:").grid(row=row,column=col, sticky=tk.W)
		self.nConvCrit = tkExtra.FloatEntry(optLabel, width=10, background="White")
		self.nConvCrit.insert(tk.END,0.01)
		self.nConvCrit.grid(row=row, column=col+1, sticky=tk.W)

		row+=1
		tk.Label(optLabel, text="Scale Step:").grid(row=row,column=col, sticky=tk.W)
		self.nScaleStep = tkExtra.FloatEntry(optLabel, width=10, background="White")
		self.nScaleStep.insert(tk.END,1.0)
		self.nScaleStep.grid(row=row, column=col+1, sticky=tk.W)

		row+=1
		tk.Label(optLabel, text="Min Dose Threshold:").grid(row=row,column=col, sticky=tk.W)
		self.nMinDoseT = tkExtra.FloatEntry(optLabel, width=10, background="White")
		self.nMinDoseT.insert(tk.END,0.0)
		self.nMinDoseT.grid(row=row, column=col+1, sticky=tk.W)

		row+=1
		tk.Label(optLabel, text="Min Particles Per Beam:").grid(row=row,column=col, sticky=tk.W)
		self.nMinPartT = tkExtra.FloatEntry(optLabel, width=10, background="White")
		self.nMinPartT.insert(tk.END, 0)
		self.nMinPartT.grid(row=row, column=col+1, sticky=tk.W)

		#------------------------Biological Optimization----------------------
		rowL+=1
		bioLabel=tkExtra.tk.LabelFrame(inputFrame, text="Biological optimization")
		bioLabel.grid(row=rowL, column=colL,sticky=tk.EW)

		#TODO: forbid to add bio files/ let files if this is not checked in or check in automatically
		row,col =0, 0
		self.isBioopt = tk.IntVar()
		tk.Checkbutton(bioLabel, text ="Biological optimization",
			variable=self.isBioopt, onvalue=tk.TRUE, command=self.setBioopt
			).grid(row=row,column=col, sticky=tk.W)

		row+=1
		self.isLetopt = tk.IntVar()
		tk.Checkbutton(bioLabel, text ="LET optimization",
			variable=self.isLetopt, onvalue=tk.TRUE, command=self.setLetopt
			).grid(row=row,column=col, sticky=tk.W)

		row+=1
		self.cellLine = tk.StringVar()
		tk.Label(bioLabel, text="Cell:").grid(row=row,column=col, sticky=tk.W)
		self.cellLine = tkExtra.Combobox(bioLabel, width=30, command = self.setCellLine)
		self.cellLine.grid(row=row,column=col+1)
		self.cellLine.fill(sorted(CELL_LINE))
		self.cellLine.select(0)

		#TODO: set up Alpha/Beta scale
		row+=1
		tk.Label(bioLabel, text="Alpha Scale:").grid(row=row,column=col, sticky=tk.W)
		self.nAlphaScale = tkExtra.FloatEntry(bioLabel, width=10,
					background="White", state=tk.DISABLED)
		self.nAlphaScale.grid(row=row, column=col+1, sticky=tk.W)

		row+=1
		tk.Label(bioLabel, text="Beta Scale:").grid(row=row,column=col, sticky=tk.W)
		self.nBetaScale = tkExtra.FloatEntry(bioLabel, width=10,
					background="White", state=tk.DISABLED)
		self.nBetaScale.grid(row=row, column=col+1, sticky=tk.W)
		#end TODO

		row+=1
		self.CellFile=tk.StringVar()
		tk.Label(bioLabel, text="Cell Line File:").grid(row=row,column=col, sticky=tk.W)
		tk.Label(bioLabel,
				width=30,relief=tk.SUNKEN, anchor=tk.W,
				textvariable=self.CellFile).grid(
				row=row, column=col+1, sticky=tk.EW)
		self.cellFileb=tk.Button(bioLabel,image=icons.get("load"), pady=0,
				command=self.loadCellFile)
		self.cellFileb.grid(row=row, column=col+2)

		self.nBetaScale.configure(state="disabled")
		self.nAlphaScale.configure(state="disabled")
		self.cellFileb.configure(state="disabled")

		col=0
		row+=1
		tk.Label(bioLabel, text="Exp scaling of RBE:").grid(row=row,column=col, sticky=tk.W)
		self.nRbeFactor = tkExtra.FloatEntry(bioLabel, width=10, background="White")
		self.nRbeFactor.grid(row=row, column=col+1, sticky=tk.W)
		self.nRbeFactor.insert(tk.END, 1.0)

		row+=1
		tk.Label(bioLabel, text="Fixed RBE value:").grid(row=row,column=col, sticky=tk.W)
		self.nRbeFixed = tkExtra.FloatEntry(bioLabel, width=10, background="White")
		self.nRbeFixed.grid(row=row, column=col+1, sticky=tk.W)
		self.nRbeFixed.insert(tk.END, 1.0)

#                row+=1
#                tk.Label(bioLabel, text="Fixed target mean RBE:").grid(row=row,column=col, sticky=tk.W)
#                self.nRbeInPtv = tkExtra.FloatEntry(bioLabel, width=10, background="White")
#		self.nRbeInPtv.grid(row=row, column=col+1, sticky=tk.W)
#                self.nRbeInPtv.insert(tk.END, 0.0)

		#--------------------Additional info--------------------------------
		rowL+=1
#                addLabel=tkExtra.tk.LabelFrame(inputFrame, text="Additional options")
#                addLabel.grid(row=rowL,column=colL, sticky=tk.EW)
#
#                row,col=0,0
#                self.isInitialEvaluation = tk.IntVar()
#                c = tk.Checkbutton(addLabel, text ="Initial evaluation",
#                                variable=self.isInitialEvaluation, onvalue=tk.TRUE)
#                c.grid(row=row,column=col, sticky=tk.W)
#
#                row+=1
#                self.isCheckPb = tk.IntVar()
#                c = tk.Checkbutton(addLabel, text ="Initial check of PB",
#                                variable=self.isCheckPb, onvalue=tk.TRUE)
#                c.grid(row=row,column=col, sticky=tk.W)
#
#                row+=1
#                self.isPhotonEquiv = tk.IntVar()
#                c = tk.Checkbutton(addLabel, text ="Compute Photon Equivalent Surivival",
#                                variable=self.isPhotonEquiv, onvalue=tk.TRUE)
#                c.grid(row=row,column=col, sticky=tk.W)
#
#                row+=1
#                self.isSelectiveEval = tk.IntVar()
#                c = tk.Checkbutton(addLabel, text ="Do Selective Evaluation",
#                                variable=self.isSelectiveEval, onvalue=tk.TRUE)
#                c.grid(row=row,column=col, sticky=tk.W)
#
#                row+=1
#                tk.Label(addLabel, text="Scale Initial Intensities:").grid(
#                        row=row,column=col, sticky=tk.W)
#                self.nScaleInt = tkExtra.FloatEntry(addLabel, width=10,
#                                background="White")
#		self.nScaleInt.grid(row=row, column=col+1,sticky=tk.W)
#                self.nScaleInt.insert(tk.END, 1)
#
		#------------------------------------------------------
#                rowL+=1

		#FIXME tutaj?

	# ----------------------------------------------------------------------
	# Record input/output directory
	# ----------------------------------------------------------------------
#	def loadInputDir(self, event=None):
#		"""load input directory"""
#
#		fn=bFileDialog.askdirectory(parent=self,
#			title="Input directory",
#			initialdir=self.lastPath)
#
#		if len(fn) == 0:
#                        self.inputDir.set(self.lastPath)
#                        return
#                self.inputDir.set(fn)
#		self.opt.setInputDir(fn)

	#-----------------------------------------------------------------------
	def loadOutputDir(self, event=None):
		"""load output directory"""

		fn=bFileDialog.askdirectory(parent=self,
				title="Output directory",
				initialdir=self.lastPath)

		if not fn:
			self.outputDir.set(self.lastPath)
			return
		rel_fn=self.project.relpath(fn)
		self.outputDir.set(rel_fn)
		self.opt.cleanOutputDir(fn)

	#-----------------------------------------------------------------------
	def loadPBFile(self,event=None):
		"""load pencil beam file"""
		#TODO: load Pencil beam from the input file

		fn=bFileDialog.askopenfilename(parent=self,
				title="Import PB file",
				filetypes=[("txt files",("*.txt")),
					   ("PB fluka files",("*.pb")),
					   ("All","*")],
				initialdir=self.inputDir.get(),
				initialfile=self.PBFile.get())

		if not fn: return
		self.PBFile.set(fn)
		rel_fn=self.project.relpath(fn)
		self.opt.setPencilBeamFile(rel_fn)
		#TODO: load PB file from the input file

	#-----------------------------------------------------------------------
	def loadVxlFile(self,event=None):
		fn=bFileDialog.askopenfilename(parent=self,
				title="Import Voxel file",
				filetypes=[("vxl files",("*.vxl")),
					("All","*")],
				initialdir=self.inputDir.get(),
				initialfile=self.VxlFile.get())

		if not fn: return
		rel_fn=self.project.relpath(fn)
		self.VxlFile.set(rel_fn)
		#TODO: load Vxl file from the input file -> moved to the init optimizer

	#-----------------------------------------------------------------------
	def loadCellLine(self,event=None):
		cellline=CELL_LINE[self.cellLine.get()]
		self.nCellLine=str(cellline)
		self.opt.setCellLine(cellline)

	#-----------------------------------------------------------------------
	def loadCellFile(self,event=None):
		fn=bFileDialog.askopenfilename(parent=self,
				title="Import Cell Line file",
				filetypes=[("txt files",("*.txt")),
					("All","*")],
				initialdir=self.inputDir.get(),
				initialfile=self.CellFile.get())

		if not fn: return
		rel_fn=self.project.relpath(fn)
		self.CellFile.set(rel_fn)
		self.opt.setCellFile(fn)

	#-----------------------------------------------------------------------
	def loadDoseFile(self,event=None):
		fn=bFileDialog.askopenfilename(parent=self,
				title="Import Dose Eventbin file",
				filetypes=[("All","*")],
				initialdir=self.inputDir.get(),
				initialfile=self.DoseFile.get())

		if not fn: return
		rel_fn=self.project.relpath(fn)
		self.DoseFile.set(rel_fn)
		self.opt.setDoseFile(fn)

	#-----------------------------------------------------------------------
	def loadAlphaFile(self,event=None):
		fn=bFileDialog.askopenfilename(parent=self,
				title="Import Alpha Eventbin file",
				filetypes=[("All","*")],
				initialdir=self.inputDir.get(),
				initialfile=self.AlphaFile.get())
		if not fn: return
		rel_fn=self.project.relpath(fn)
		self.AlphaFile.set(rel_fn)
		self.opt.setAlphaFile(fn)

	#-----------------------------------------------------------------------
	def loadBetaFile(self,event=None):
		fn=bFileDialog.askopenfilename(parent=self,
				title="Import Beta Eventbin file",
				filetypes=[("All","*")],
				initialdir=self.inputDir.get(),
				initialfile=self.BetaFile.get())
		if not fn: return
		rel_fn=self.project.relpath(fn)
		self.BetaFile.set(rel_fn)
		self.opt.setBetaFile(fn)

	#-----------------------------------------------------------------------
	def loadLetFile(self,event=None):
		fn=bFileDialog.askopenfilename(parent=self,
				title="Import Let Eventbin file",
				filetypes=[("All","*")],
				initialdir=self.inputDir.get(),
				initialfile=self.LetFile.get())
		if not fn: return
		rel_fn=self.project.relpath(fn)
		self.LetFile.set(rel_fn)
		self.opt.setLetFile(fn)

	#-----------------------------------------------------------------------
	def readPBfromInput(self):
		pass
		#inp=self.flair.project.input
		#rtplan=inp["include"]
		#print rtplan
		#self.page.get()

	#-----------------------------------------------------------------------
	def setBioopt(self, event=None):
		#TODO: it needs to be called always or what?
		self.opt.setBioopt(self.isBioopt.get())

	#-----------------------------------------------------------------------
	def setLetopt(self, event=None):
		#TODO: it needs to be called always or what?
		self.opt.setLetopt(self.isLetopt.get())

	#-----------------------------------------------------------------------
	def setOptAlg(self, event=None):
		algNo=OPTALG[self.optAlg.get()]
		self.nOptAlg=str(algNo)
		self.opt.setOptimizationAlgorithm(algNo)
		print("Chosen algorithm no:",algNo)

	#-----------------------------------------------------------------------
	def setCellLine(self, event=None):
		#self.cellFileb.configure(state="disabled")
		self.nCellLine=str(CELL_LINE[self.cellLine.get()])

		#This needs to be corrected to give back the right version
		#needs validation for max 3 digits
		if self.nCellLine=='20':
			self.nAlphaScale.configure(state="normal")
			self.nCellLine+=str(self.nAlphaScale.get())
		elif self.nCellLine=='21':
			self.nBetaScale.configure(state="normal")
			self.nCellLine+=str(self.nBetaScale.get())
		elif self.nCellLine=='0':
			pass
			#self.cellFileb.configure(state="normal")

	#-----------------------------------------------------------------------
	def addRoi(self, event=None):
		roiInfo=AddDialog(self.page,
				"Add Roi","Add new ROI objectives").show()
		if roiInfo:
			self.roiList.selectClear()
			self.roiList.insert(tk.END,roiInfo)
			self.roiList.selection_set(tk.END)

	#-----------------------------------------------------------------------
	def editRoi(self,event=None):
		idx=self.roiList.curselection()[0]
		startInfo=self.roiList.get(idx)
		roiInfo=AddDialog(self.page,
				"Edit Roi","Edit ROI objectives",startInfo).show()
		if roiInfo:
			self.roiList.selectClear()
			self.roiList.set(idx,roiInfo)
			self.roiList.selection_set(idx)

	#-----------------------------------------------------------------------
	def deleteRoi(self,event=None):
		idx=self.roiList.curselection()
		self.roiList.delete(idx[0],idx[-1])

	#---------------------------Test function-------------------------------
	def processRoi(self, event=None):
		rois=list(zip(*(self.roiList.get(0,tk.END))))
		self.opt.setVxlFile(self.VxlFile.get()
				     , self.nXmin.getfloat(), self.nYmin.getfloat(), self.nZmin.getfloat()
				     , self.DoseFile.get(), rois[0], rois[1]
				     , rois[2], rois[3], rois[4] )

	# ----------------------------------------------------------------------
	def run(self):
		"""Run Optimizer"""
		rois=list(zip(*(self.roiList.get(0,tk.END))))
		self.opt.setVxlFile(self.VxlFile.get()
				     , self.nXmin.getfloat(), self.nYmin.getfloat(), self.nZmin.getfloat()
				     , self.DoseFile.get(), rois[0], rois[1]
				     , rois[2], rois[3], rois[4] )
		#TODO: Remove is PTV change to + or minus Roinb
		self.opt.setIterationActionInterval(self.nAction.getint())
		self.opt.setConvergenceCriterium(self.nConvCrit.getfloat())
		self.opt.setScaleOptimizationStep(self.nScaleStep.getfloat())
		self.opt.setMinDoseThreshold(self.nMinDoseT.getfloat())
		self.opt.setMinParticlesPerPb(self.nMinPartT.getfloat())
#		self.opt.RunOptimizer()

	# --------------------------------------------------------------------
	# Process
	# --------------------------------------------------------------------
	def execute(self):
		self.run()
#		self.process = OptimizerProcess.OptimizerProcess()
		if self.process.isAlive():
			if not tkFlair.askyesno("Optimizing is active",
					"An optimizing process is still running.\n"
					"Do you want to kill it?",
					parent=self.ribbon):
				return
			self.process.kill()
			self.process.wait()

		self.log = self.flair.newLog("Optimize", "Process")

#		rois=zip(*(self.roiList.get(0,tk.END)))
#		self.process.setProcess(self.opt.setVxlFile(self.VxlFile.get()
#				     , self.nXmin.getfloat(), self.nYmin.getfloat(), self.nZmin.getfloat()
#				     , self.DoseFile.get(), rois[0], rois[1]
#				     , rois[2], rois[3], rois[4] ))

		#TODO: Remove is PTV change to + or minus Roinb
#		self.opt.setIterationActionInterval(self.nAction.getint())
#		self.opt.setConvergenceCriterium(self.nConvCrit.getfloat())
#		self.opt.setScaleOptimizationStep(self.nScaleStep.getfloat())
#		self.opt.setMinDoseThreshold(self.nMinDoseT.getfloat())
#		self.opt.setMinParticlesPerPb(self.nMinPartT.getfloat())

		self.process.init()
		self.flair.addProcess(self.process, self.log, self.opt)
#		self.page.setStatus("Optimizing started...")

	# ----------------------------------------------------------------------
	def refresh(self, what=None):
		super.refresh(what)
		self.opt.init()
		self.process = OptimizerProcess.OptimizerProcess()

	# ----------------------------------------------------------------------
	def activate(self):
		return

	# ----------------------------------------------------------------------
	def clear(self):
		return

#================================================================================
# Add/Edit Roi Dialog
#================================================================================
class AddDialog(tk.Toplevel):
	"""
	More complex Add/EditInput dialog for Roi purposes:
	valid types:
		str = any string
		int = any integer
		spin  = Spin box with limits from_, to_
		float = any float
	"""

	#-----------------------------------------------------------------------
	def __init__(self, master, title=None, message=None, inp=None):
		super().__init__(master)
		self.transient(master)

		if title:
			self.title(title)

		self.result = None
		self.input  = inp if inp else ["",0,"",0,0]

		body = tk.Frame(self)
		self.initial_focus = self.body(body,message)
		body.pack(padx=5, pady=20)

		self.buttonbox()

		self.focus_set()
		self.initial_focus.focus_set()
#		self.wait_window()

	# --------------------------------------------------------------------
	def body(self,master,message):
		row, col = 0, 0
		tk.Label(master, text=message).grid(row=row)

		col += 1
		self.c1=tk.IntVar()
		c=tk.Checkbutton(master, text ="PTV", variable=self.c1,
		      onvalue=tk.TRUE)
		c.grid(	row=row,column=col, sticky=tk.W)
		if self.input[1]:
			c.select()

		col =0
		row += 2
		tk.Label(master, text="ROI Id", justify=tk.LEFT).grid(
			row=row, column = col, sticky = tk.W)

		col += 1
		self.e1=tkExtra.FloatEntry(master)
		self.e1.insert(0,self.input[0])
		self.e1.grid(row=row, column = col)

		row += 1
		col =  0
		tk.Label(master, text="Target Dose", justify=tk.LEFT).grid(
			row=row, column=col, sticky=tk.W)

		col+=1
		self.e2=tkExtra.FloatEntry(master)
		self.e2.insert(0,self.input[2])
		self.e2.grid(row=row, column =col)

		row += 1
		col =  0
		tk.Label(master, text="Target Rbe", justify=tk.LEFT).grid(
			row=row, column=col, sticky=tk.W)

		col+=1
		self.e3=tkExtra.FloatEntry(master)
		self.e3.insert(0,self.input[3])
		self.e3.grid(row=row, column=col)

		row += 1
		col =  0
		tk.Label(master, text="Target Let", justify=tk.LEFT).grid(
			row=row, column=col, sticky=tk.W)

		col+=1
		self.e4=tkExtra.FloatEntry(master)
		self.e4.insert(0,self.input[4])
		self.e4.grid(row=row, column=col)

		return self.e1

	# --------------------------------------------------------------------
	def buttonbox(self):
		box = tk.Frame(self)
		b = tk.Button(box, text="Cancel", command=self.cancel)
		b.pack(side=tk.RIGHT, pady=5)
		b = tk.Button(box, text="Ok", command=self.ok)
		b.pack(side=tk.RIGHT, pady=5)
		box.pack(fill=tk.X)

		self.bind("<Return>", self.ok)
		self.bind("<Escape>", self.cancel)

	# --------------------------------------------------------------------
	def apply(self):
		self.result = (self.e1.getint(), int(self.c1.get()), self.e2.getfloat(
			),self.e3.getfloat(),self.e4.getfloat())

	# --------------------------------------------------------------------
	def show(self):
		grab_window = self.grab_current()
		if grab_window is not None:
			grab_window.grab_release()
		self.wait_window()
		if grab_window is not None:
			grab_window.grab_set()
		return self.result

	# --------------------------------------------------------------------
	def ok(self, event=None):
		try:
			self.apply()
			print("ok",self.result)
			self.cancel()
		except ValueError:
			pass

	#-----------------------------------------------------------------------
	def cancel(self, event=None):
		self.destroy()
