#!/bin/env python3
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Wioletta.Kozlowska@cern.ch
# Date:   22-Oct-2015

__author__  = "Vasilis Vlachoudis"
__email__   = "Vasilis.Vlachoudis@cern.ch"

import os
import select
import subprocess

import tkFlair
from process import Process

#===============================================================================
# Optimizer Tab
#===============================================================================
class OptimizerProcess(Process):
	#----------------------------------------------------------------------
	@tkFlair.exception
	def execute(self, opt):
		super().execute()
		self.p = subprocess.Popen(opt.RunOptimizer(),
					stdout=subprocess.PIPE,
					stderr=subprocess.STDOUT,
					close_fds=True)
		ready = select.select([self.p.stdout], [], [], 0)
		while ready[0]:
			# Use the os.read instead of the pout.readline
			# to avoid blocking situations
			# readline waits for the \n or EOF
			data = os.read(self.p.stdout.fileno(), 1024)
			if not data: break
			self.output(data)
			ready = select.select([self.p.stdout], [], [], 0)
		while True:
			line = self.p.stdout.readline()
			if not line: break
			self.output(line)
		return 0

	#----------------------------------------------------------------------
	def kill(self):
		super().kill()
		self.p.terminate()
