#!/bin/env python3
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch, Adonai.Herrera.Martinez@cern.ch
# Date:   31-Oct-2006

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import tkinter as tk

import Data

import tkFlair
import tkExtra

import log
import Plot
import Manual
import BasePlot
from fluka import FlukaLayout

#===============================================================================
# Residual Nuclei Plot Frame
#===============================================================================
class ResnucleiPlotFrame(BasePlot.BasePlotFrame):
	def __init__(self, master, flair):
		super().__init__(master, flair)

		# === File & Detector ===
		frame = tk.LabelFrame(self, text="Residuals Detector ")
		frame.pack(side=tk.TOP, expand=tk.YES, fill=tk.BOTH, padx=3)

		row, col = 0, 0
		tk.Label(frame, text="File:", anchor=tk.E).grid(row=row,
				column=col, sticky=tk.E)
		col += 1; span = 4
		self.datafile = tkExtra.FileEntry(frame,
					title="Load Residual Nuclei file",
					initialdir=self.project.dir(),
					filetypes=[
						("Fluka RESNUCLEi files",("*resnucle*","*.rnc")),
						("All","*")],
					foreground="DarkBlue", background="White")
		self.datafile.bind("<<FileChanged>>", self.load)
		tkExtra.Balloon.set(self.datafile, "RESNUCLEi binary output file")
		self.datafile.grid(row=row, column=col, columnspan=span, sticky=tk.EW)
		self.widget["datafile"] = self.datafile

		# ---
		col += span
		tk.Label(frame, text="Title:", anchor=tk.E).grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		self.usr_title = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_title.grid(row=row, column=col, columnspan=3, sticky=tk.EW)

		# ---
		row, col = row+1, 0
		tk.Label(frame, text="Cycles:", anchor=tk.E).grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		self.usr_cycles = tk.Label(frame, anchor=tk.E,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_cycles.grid(row=row, column=col, sticky=tk.EW)

		# ---
		col += 1
		tk.Label(frame, text="Primaries:", anchor=tk.E).grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		self.usr_ncase = tk.Label(frame, anchor=tk.E,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_ncase.grid(row=row, column=col, columnspan=2, sticky=tk.EW)

		# ---
		col += 2
		tk.Label(frame, text="Weight:", anchor=tk.E).grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		self.usr_weight = tk.Label(frame, anchor=tk.E,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_weight.grid(row=row, column=col, sticky=tk.EW)

		# ---
		col += 1
		tk.Label(frame, text="Time:", anchor=tk.E).grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		self.usr_time = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_time.grid(row=row, column=col, sticky=tk.EW)

		# ---
		frame.grid_columnconfigure(1, weight=1)	# <<<<-------
		frame.grid_columnconfigure(3, weight=1)
		frame.grid_columnconfigure(6, weight=1)
		frame.grid_columnconfigure(8, weight=5)

		# === Info ===
		frame = tk.LabelFrame(self, text="Detector Info ")
		frame.pack(expand=tk.YES, fill=tk.BOTH, padx=3)

		# ---
		row, col = 0, 0
		tk.Label(frame, text="Det:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W, pady=0)
		col += 1
		self.det = tkExtra.Combobox(frame, width=10, command=self.fillDetectorInfo)
		self.det.grid(row=row, column=col, sticky=tk.EW, pady=0)
		self.widget["@det"] = self.det

		# ---
		col += 1
		tk.Label(frame, text="Type:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W)
		col += 1
		self.usr_type = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_type.grid(row=row, column=col, sticky=tk.EW)

		# ---
		col += 1
		tk.Label(frame, text="Region:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W)
		col += 1
		self.usr_region = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_region.grid(row=row, column=col, sticky=tk.EW)

		# ---
		col += 1
		tk.Label(frame, text="Volume:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W)
		col += 1
		self.usr_volume = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_volume.grid(row=row, column=col, sticky=tk.EW)

		# ---
		col += 1
		tk.Label(frame, text="Min:", anchor=tk.E).grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		self.usr_min = tk.Label(frame, anchor=tk.E, width=10,
				foreground=tkFlair._CLABEL_FOREGROUND_COLOR,
				background=tkFlair._CLABEL_BACKGROUND_COLOR)
		self.usr_min.grid(row=row, column=col, sticky=tk.EW)

		# ---
		row, col = row+1, 0
		tk.Label(frame, text="Plot:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W)

		col += 1
		self.plotType = tkExtra.Combobox(frame, width=5)
		self.plotType.fill(Plot.RESNUCLEI_TYPES)
		self.plotType.set(Plot.RESNUCLEI_TYPES[0])
		self.plotType.grid(row=row, column=col, sticky=tk.EW)
		self.widget["plot"] = self.plotType

		# ---
		col += 1
		tk.Label(frame, text="Zhigh:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W)
		col += 1
		self.usr_zhigh = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_zhigh.grid(row=row, column=col, sticky=tk.EW)

		# ---
		col += 1
		tk.Label(frame, text="Ahigh:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W)
		col += 1
		self.usr_ahigh = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_ahigh.grid(row=row, column=col, sticky=tk.EW)

		# ---
		col += 1
		tk.Label(frame, text="Mhigh:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W)
		col += 1
		self.usr_mhigh = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.usr_mhigh.grid(row=row, column=col, sticky=tk.EW)

		# ---
		col += 1
		tk.Label(frame, text="Max:", anchor=tk.E).grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		self.usr_max = tk.Label(frame, anchor=tk.E, width=10,
				foreground=tkFlair._CLABEL_FOREGROUND_COLOR,
				background=tkFlair._CLABEL_BACKGROUND_COLOR)
		self.usr_max.grid(row=row, column=col, sticky=tk.EW)

		# ---
		row, col = row+1, 0
		tk.Label(frame, text="Norm:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W)

		col += 1; span=3
		b = tk.Entry(frame, background="White")
		tkExtra.Balloon.set(b, BasePlot._NORM_BALLOON)
		b.grid(row=row, column=col, columnspan=span, sticky=tk.EW)
		self.widget["norm"] = b

		# --- Options ---
		self._styleRow = row+1
		self.style = tk.LabelFrame(frame, text="Options ")
		self.styleFrame(self.style)
		self.plotType.command = self.typeChange

		# ---
		frame.grid_columnconfigure(1, weight=1)	# <<<<-------
		frame.grid_columnconfigure(3, weight=1)
		frame.grid_columnconfigure(5, weight=1)
		frame.grid_columnconfigure(7, weight=1)

		# ---- variables -----
		self.usr = Data.Resnuclei()
		self.previous = []
		self._datafile = None

		self.bindToggle()

	# ----------------------------------------------------------------------
	def typeChange(self):
		if self.plotType.get() in (Plot.RESNUCLEI_Z, Plot.RESNUCLEI_A):
			# Show style frame
			self.style.grid(row=self._styleRow, column=0, columnspan=10, sticky=tk.NSEW)
		else:
			# Hide style frame
			self.style.grid_forget()

	# ----------------------------------------------------------------------
	def type(self): return "RESNUCLE"

	# ----------------------------------------------------------------------
	# Set plot
	# ----------------------------------------------------------------------
	def set(self, plot):
		super().set(plot)
		if plot is None: return

		name = self.project.name()
		self.datafile.initialdir = self.project.dir()
		self.datafile.setFiletypes([
				("Fluka RESNUCLEi files",("*.rnc", "*resnucle*")),
				(f"{name} RESNUCLEi files", f"{name}*.rnc"),
				("All","*")])
		self._datafile = plot("datafile")
		self.loadDataFile(self._datafile)
		try: det = int(plot("det"))
		except: det = 1
		self.detector(det)
		self.fillDetectorInfo()

	# ----------------------------------------------------------------------
	def get(self, plot):
		super().get(plot)
		if plot is None: return

		if self.det.cget("state") != tk.DISABLED:
			det = self.detector()
			if det>=0: plot["det"] = str(det)

	# ----------------------------------------------------------------------
	def clearInfo(self, all_=True):
		if all_:
			self.usr_title.config(text="")
			self.usr_time.config(text="")
			self.usr_weight.config(text="")
			self.usr_ncase.config(text="")
			self.usr_cycles.config(text="")

		self.usr_type.config(text="")
		self.usr_region.config(text="")
		self.usr_volume.config(text="")
		self.usr_zhigh.config(text="")
		self.usr_ahigh.config(text="")
		self.usr_mhigh.config(text="")

		self.usr_min.config(text="")
		self.usr_max.config(text="")

	# ----------------------------------------------------------------------
	def loadDataFile(self, datafile):
		# --- Load RESNUCLEi ---
		if not datafile:
			self._datafile = None
			self.usr.reset()
			self.clearInfo()
			self.det.clear()
			return

		#if self.usr.file!=datafile or self.usr.ncase==0:
		try:
			self.usr.readHeader(datafile)
		except (IOError, FileNotFoundError) as ex:
			log.error(ex)
			self.clearInfo()
			self.det.clear()
			return

		self.usr_title.config(text=self.usr.title)
		self.usr_time.config(text=self.usr.time)
		self.usr_weight.config(text=self.usr.weight)
		self.usr_ncase.config(text=self.usr.ncase)
		self.usr_cycles.config(text=self.usr.nbatch)

		# Populate the detector menu
		self.det.delete(0,tk.END)
		for det in self.usr.detector:
			self.det.insert(tk.END,f"{det.num} {det.name}")

	# --------------------------------------------------------------------
	def fillDetectorInfo(self):
		# --- Display Detector ---
		d = self.detector()
		if d<=0:
			self.clearInfo(False)
			return

		det = self.usr.detector[d-1]
		for (n,i) in FlukaLayout.RESNUCLE_type:
			if i==det.type:
				self.usr_type.config(
					text=f"{i}: {n}")
				break
		self.usr_region.config(text=det.region)
		self.usr_volume.config(text="%-11g"%(det.volume))
		self.usr_zhigh.config(text=det.zhigh)
		self.usr_ahigh.config(text=str(2*det.zhigh + det.mhigh + det.nmzmin))
		self.usr_mhigh.config(text=det.mhigh)

	# ----------------------------------------------------------------------
	def detector(self, val=None):
		if val is None:
			# Get the value
			try:
				return int(self.det.get().split()[0])
			except:
				return -1
		else:
			if 0 < val <= len(self.usr.detector):
				det = self.usr.detector[val-1]
				self.det.set(f"{det.num} {det.name}")
			else:
				self.clearInfo(False)

	# ----------------------------------------------------------------------
	def load(self, event=None):
		filename = self.datafile.filename()
		if filename:
			self._datafile = self.project.relpath(filename)
			self.datafile.set(self._datafile)
			self.loadDataFile(self._datafile)
			self.detector(1)
			self.fillDetectorInfo()

	# ----------------------------------------------------------------------
	def help(self):
		Manual.show("","Residual Nuclei Plot")
