#
# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   06-Sep-2006

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import os
import time
import random
import struct
import tkinter as tk

import log
import tool
import icons
import Global
import Manual
import Ribbon
import Project
import tkExtra
import tkFlair
import FlairRibbon
import bFileDialog
from deck import Card
from fluka import FlukaConfig

#===============================================================================
# Run Tab in Page Frame
#===============================================================================
class RunTab(FlairRibbon.FlairTab):
	#----------------------------------------------------------------------
	def __init__(self, master, page, **kw):
		super().__init__(master, page, "Run", **kw)

		frame = tk.LabelFrame(self, text="Override",
				foreground="DarkBlue")
		frame.grid(row=0, column=0, sticky=tk.NSEW)
		#frame.pack(fill=tk.BOTH, side=tk.TOP, expand=tk.YES)
		self.grid_columnconfigure(0, weight=1)
		self.grid_rowconfigure(0, weight=1)

		# -----
		row = 0
		tk.Label(frame, text="Title:").grid(row=row, column=0, sticky=tk.E)
		self.title = tk.Entry(frame, background="White", justify=tk.LEFT)
		tkExtra.Balloon.set(self.title, "Override title of run")
		self.title.grid(row=row, column=1, columnspan=5, sticky=tk.EW)
		self.title.bind('<Button-3>', self.toggleField)

		# -----
		row += 1
		col  = 0
		tk.Label(frame, text="Primaries:").grid(
				row=row, column=col, sticky=tk.NE)
		col += 1
		self.primaries = tkExtra.FloatEntry(frame, background="White", width=5)
		tkExtra.Balloon.set(self.primaries,
				"Override number of starting particles")
		self.primaries.grid(row=row, column=col, sticky=tk.EW)
		self.primaries.bind('<Button-3>', self.toggleField)

		# ---
		col += 1
		tk.Label(frame, text="Time:").grid(
				row=row, column=col, sticky=tk.E)
		col += 1
		self.time = tkExtra.FloatEntry(frame, background="White", width=5)
		tkExtra.Balloon.set(self.time, "Time limit of the run")
		self.time.grid(row=row, column=col, sticky=tk.EW)
		self.time.bind('<Button-3>', self.toggleField)

		# -----
		col += 1
		tk.Label(frame, text="Rnd:").grid(
				row=row, column=col, sticky=tk.NE)
		col += 1
		self.rnd = tkExtra.IntegerEntry(frame, background="White", width=5)
		tkExtra.Balloon.set(self.rnd, "Override random number seed")
		self.rnd.grid(row=row, column=col, sticky=tk.EW)
		self.rnd.bind('<Button-3>', self.toggleField)

		col += 1
		self.rndButton = tk.Button(frame, image=icons.get("RANDOMIZ"),
				command=self.getRandomNumberAsSeed)
		tkExtra.Balloon.set(self.rndButton, "Random number")
		self.rndButton.grid(row=row, column=col, sticky=tk.NSEW)
		self.rndButton.bind('<Button-3>', self.toggleField)

		# -----
		row += 1
		col  = 0
		tk.Label(frame, text="Mode:").grid(
				row=row, column=col, sticky=tk.NE)
		col += 1
		self.mode = tkExtra.Combobox(frame, foreground="Blue")
		tkExtra.Balloon.set(self.mode, "Simulation program to use")
		self.mode.grid(row=row, column=col, sticky=tk.NSEW)
		self.mode.bindWidgets('<Button-3>', self.toggleField)
		modes = list(sorted(Card.modes()))
		self.mode.fill(modes)

		# ---
		if Global.developer:
			col += 1
			tk.Label(frame, text="Threads:").grid(
					row=row, column=col, sticky=tk.E)
			col += 1
			self.threads = tk.Spinbox(frame,
						from_=0, to=1000,
						background="White",
						state=tk.DISABLED,
						width=4)
			tkExtra.Balloon.set(self.threads,
					"Number of threads to use for multithreaded run")
			self.threads.grid(row=row, column=col, sticky=tk.EW)
			self.threads.bind('<Button-3>', self.toggleField)
		else:
			col += 2

		# ---
		col += 1
		tk.Label(frame, text="Exe:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.exe = tkExtra.Combobox(frame, foreground="DarkBlue")
		tkExtra.Balloon.set(self.exe, "Override executable file")
		self.exe.grid(row=row, column=col, sticky=tk.EW)
		self.exe.bindWidgets('<Button-3>', self.toggleField)

		col += 1
		self.bel = tk.Button(frame, image=icons.get("load"),
				command=self.loadExecutable)
		tkExtra.Balloon.set(self.bel, "Load Executable")
		self.bel.grid(row=row, column=col, sticky=tk.NSEW)
		self.bel.bind('<Button-3>', self.toggleField)

		# -----
		row += 1
		tk.Label(frame, text="Defines:").grid(row=row, column=0, sticky=tk.E)
		self.bd = tk.Button(frame, text="Default Defines",
					pady=0, padx=2,
					takefocus=False,
					command=self.selectDefaultDefines)
		tkExtra.Balloon.set(self.bd, "Reset to default defines")
		self.bd.grid(row=row, column=1, sticky=tk.EW)
		self.bd.bind('<Button-3>', self.toggleField)

		# -----
		row += 1
		self.defineList = tkExtra.MultiListbox(frame,
				((' ',     3, None),
				 ('Name', 16, None),
				 ('Value',32, None)),
				 background="White",
				 stretch="last",
				 height=5)
		self.defineList.bindList("<space>",		self.toggleDefine)
		self.defineList.bindList("<Double-1>",		self.toggleDefine)
		self.defineList.bindList("<Return>",		self.editDefine)
		self.defineList.bindList("<KP_Enter>",		self.editDefine)
		self.defineList.listbox(0).bind("<Button-1>",	self.defineListClick)
		self.defineList.bindList("<ButtonRelease-1>",	self.defineListRelease)
		self.defineList.bindLeftRight()
		#self.defineList.setPopupMenu([('Default', 0,	self.selectDefaultDefines)])
		# unbind popupmenu
		self.defineList.unbindList("<Button-3>")
		self.defineList.bindList('<Button-3>',		self.toggleField)
		self.defineList.grid(row=row,  column=0, columnspan=8, sticky=tk.NSEW)

		frame.grid_columnconfigure(1, weight=1)
		frame.grid_columnconfigure(3, weight=1)
		frame.grid_columnconfigure(5, weight=1)
		frame.grid_rowconfigure(4, weight=1)

		# ---------------------------------------------------------------------------
		# Progress
		# ---------------------------------------------------------------------------
		frame = tk.LabelFrame(self, text="Progress",
				foreground="DarkBlue")
		#frame.pack(fill=tk.BOTH, side=tk.BOTTOM)
		frame.grid(row=1, column=0, stick=tk.NSEW)

		row = 0
		col = 0
		tk.Label(frame, text="Status:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.pstatus = tk.Label(frame, justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.pstatus.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.pstatus, "Run status")

		col += 1
		tk.Label(frame, text="Input:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.pinput = tk.Label(frame, text="", justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.pinput.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.pinput, "Input file name")

		col += 1
		tk.Label(frame, text="Dir:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.pdir = tk.Label(frame, text="", justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.pdir.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.pdir, "Temporary directory where run takes place")

		row += 1
		col = 0
		tk.Label(frame, text="Started:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.pstarted = tk.Label(frame, text="", justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.pstarted.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.pstarted, "Time when run started")

		col += 1
		tk.Label(frame, text="ETA:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.peta = tk.Label(frame, text="", justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.peta.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.peta,
			"Estimated time of accomplishment assuming no other "
			"process are running on CPU")

		col += 1
		tk.Label(frame, text="Time/prim:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.ptime = tk.Label(frame, text="", justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.ptime.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.ptime, "CPU Time per primary")

		row += 1
		col = 0
		tk.Label(frame, text="Elapsed:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.pelapsed = tk.Label(frame, text="", justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.pelapsed.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.pelapsed,
				"Time elapsed since the start of the current cycle")

		col += 1
		tk.Label(frame, text="Cycle:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.premain = tk.Label(frame, text="", justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.premain.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.premain, "CPU Time remaining for present cycle")

		col += 1
		tk.Label(frame, text="Run:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.prunremain = tk.Label(frame, text="", justify=tk.LEFT, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.prunremain.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.prunremain, "Total CPU Time remaining for run")

		frame.grid_columnconfigure(1, weight=1)
		frame.grid_columnconfigure(3, weight=1)
		frame.grid_columnconfigure(5, weight=1)

		# ---------------------------------------------------------------------------
		# Progress bar
		# ---------------------------------------------------------------------------
		row += 1
		col = 0
		tk.Label(frame, text="Cycles:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.cyclePBar = tkExtra.ProgressBar(frame, background="DarkGray", height=20)
		self.cyclePBar.grid(row=row, column=col, columnspan=5, sticky=tk.NSEW)
		self.cyclePBar.setShowTime(False)

		row += 1
		col = 0
		tk.Label(frame, text="Primaries:", pady=2).grid(row=row, column=col,
				sticky=tk.E)
		col += 1
		self.primaryPBar = tkExtra.ProgressBar(frame, background="DarkGray", height=20)
		self.primaryPBar.grid(row=row, column=col, columnspan=5, sticky=tk.NSEW)
		self.primaryPBar.setShowTime(False)

		# Sometimes the last refresh is lost if we move out of the window
		# and come back again later
		self.bind("<Map>",		self.checkRefresh)
		self.bind("<Control-Key-t>",	self.attach)

		# Set initial values
		self.pcycle_perc = (0.0, 0.0)
		self.pprim_perc  = (0.0, 0.0)
		self._lastDt     = Project.refreshInterval
		self._timer      = None
		self._updating   = False	# Semaphore for updating the values
		self._varSetting = True		# Semaphore for setting the cycles
		self.hoframes    = []
		self.hocnt       = 0

	# ----------------------------------------------------------------------
	# create the ribbon buttons for the Run tab
	# ----------------------------------------------------------------------
	def createRibbon(self):
		super().createRibbon()

		# ========== Input ===========
		group = FlairRibbon.FlairMenuGroup(self.ribbon, self, "Input",
				[("Export", "export", self.export)])
		group.label["background"] = Ribbon._BACKGROUND_GROUP2
		group.pack(side=tk.LEFT, fill=tk.Y, padx=0, pady=0)
		group.grid3rows()

		# ---
		col,row = 0,0
		b = Ribbon.LabelButton(group.frame,
				image=icons.get("add32"),
				command=self.page.add)
		b.grid(row=row, rowspan=2, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Add run [Ins]")

		row = 2
		menulist = [	("Folder","folder",self.page.addGroup),
				("Run",   "run",   self.page.add)]
		b = Ribbon.MenuButton(group.frame, menulist,
				text="Add \N{BLACK DOWN-POINTING SMALL TRIANGLE}",
				anchor=tk.W)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Add Run or Group")

		# ---
		col += 1
		row  = 0
		b = Ribbon.LabelButton(group.frame,
				text="Remove",
				image=icons.get("x"),
				command=self.page.delete)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Remove run [Del]")

		# ---
		row += 1
		b = Ribbon.LabelButton(group.frame,
				text="Clone",
				image=icons.get("clone"),
				command=self.page.clone)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Clone run [Ctrl-D]")

		# ---
		row += 1
		b = Ribbon.LabelButton(group.frame,
				text="Loop",
				image=icons.get("loop"),
				command=self.page.loop)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Loop runs over a variable")

		# ========== Move ===========
		group = Ribbon.LabelGroup(self.ribbon, "Move")
		group.label["background"] = Ribbon._BACKGROUND_GROUP2
		group.pack(side=tk.LEFT, fill=tk.Y, padx=0, pady=0)
		group.grid3rows()

		# ---
		col  = 0
		row  = 1
		b = Ribbon.LabelButton(group.frame,
				image=icons.get("left"),
				command=self.page.moveLeft)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Move selected runs up-one parent [Ctrl-Left]")

		# ---
		col += 1
		row  = 0
		b = Ribbon.LabelButton(group.frame,
				image=icons.get("up"),
				command=self.page.moveUp)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Move selected runs up [Ctrl-Up]")

		# ---
		row += 1
		b = Ribbon.LabelButton(group.frame,
				#text="Rename",
				image=icons.get("rename"),
				command=self.page.rename)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Rename run [F2]")

		# ---
		row += 1
		b = Ribbon.LabelButton(group.frame,
				image=icons.get("down"),
				command=self.page.moveDown)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Move selected runs down [Ctrl-Down]")

		# ---
		col += 1
		row  = 1
		b = Ribbon.LabelButton(group.frame,
				image=icons.get("right"),
				command=self.page.moveRight)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Move selected runs down-one child [Ctrl-Right]")

		# ========== Job ===========
		group = Ribbon.LabelGroup(self.ribbon, "Job")
		group.label["background"] = Ribbon._BACKGROUND_GROUP2
		group.pack(side=tk.LEFT, fill=tk.Y, padx=0, pady=0)
		group.grid3rows()

		# Cycles
		self.prev   = tk.IntVar()
		self.cycles = tk.IntVar()
		self.last   = tk.IntVar()

		self.prev.trace(  "w", self.prevChange)
		self.cycles.trace("w", self.prevChange)	# Use the same function
		self.last.trace(  "w", self.lastChange)

		# ---
		col,row = 0,0
		b = tk.Label(group.frame,
				image=icons.get("queue"),
				font=Ribbon._FONT)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.E)
		self.queueList = Ribbon.LabelCombobox(group.frame, width=10)
		self.queueList.grid(row=row, column=col+1, padx=0, pady=0, sticky=tk.EW)
		self.queueList.bindWidgets('<Button-3>', self.toggleField)
		tkExtra.Balloon.set(self.queueList, "Submitting queue")

		# ----
		col,row = 0,1
		b = Ribbon.LabelButton(group.frame,
				text="Continue",
				image=icons.get("continue"),
				command=self.continueFromLast)
		b.grid(row=row, column=col, columnspan=2, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Continue from last cycle")

		# ----
		col,row = 0,2
		b = Ribbon.LabelButton(group.frame,
				text="Attach",
				image=icons.get("attach"),
				command=self.attach)
		b.grid(row=row, column=col, columnspan=2, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Attach to a running process [Ctrl-t]")

		# --- Cycles
		col,row = 2,0

		b = tk.Label(group.frame, text="Prev:",
				anchor=tk.E,
				font=Ribbon._FONT,
				background=Ribbon._BACKGROUND)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		self.prevbox = tk.Spinbox(group.frame,
					text=self.prev,
					from_=0, to=1000,
					font=Ribbon._FONT,
					background="White", #Ribbon._BACKGROUND,
					width=4)
		self.prevbox.grid(row=row, column=col+1, padx=0, pady=0, sticky=tk.NSEW)
		self.prevbox.bind('<Button-3>', self.toggleField)
		tkExtra.Balloon.set(self.prevbox, "Previous run last cycle number."
						"0: a new input will be created;"
						">0: it will reuse the existing input file")

		# ---
		col,row = 2,1
		b = tk.Label(group.frame, text="No:",
				anchor=tk.E,
				font=Ribbon._FONT,
				background=Ribbon._BACKGROUND)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		self.cyclesbox = tk.Spinbox(group.frame,
					text=self.cycles,
					from_=1, to=1000,
					font=Ribbon._FONT,
					background="White", #Ribbon._BACKGROUND,
					width=4)
		self.cyclesbox.grid(row=row, column=col+1, padx=0, pady=0, sticky=tk.NSEW)
		self.cyclesbox.bind('<Button-3>', self.toggleField)
		tkExtra.Balloon.set(self.cyclesbox, "Number of cycles to run")

		# ---
		col,row = 2,2
		b = tk.Label(group.frame, text="To:",
				anchor=tk.E,
				font=Ribbon._FONT,
				background=Ribbon._BACKGROUND)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		self.lastbox = tk.Spinbox(group.frame,
					text=self.last,
					from_=1, to=1000,
					font=Ribbon._FONT,
					background="White", #Ribbon._BACKGROUND,
					width=4)
		self.lastbox.grid(row=row, column=col+1, padx=0, pady=0, sticky=tk.NSEW)
		self.lastbox.bind('<Button-3>', self.toggleField)
		tkExtra.Balloon.set(self.lastbox, "Last cycle number")

		# ==================
		group = Ribbon.LabelGroup(self.ribbon, "Action")
		group.label["background"] = Ribbon._BACKGROUND_GROUP2
		group.pack(side=tk.LEFT, fill=tk.Y, padx=0, pady=0)
		group.grid3rows()

		# ---
		col,row = 0,0
		b = Ribbon.LabelButton(group.frame,
				text="Clean",
				image=icons.get("clean32"),
				compound=tk.TOP,
				command=self.clean)
		b.grid(row=row, column=col, rowspan=3, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Remove all generated files")

		# ---
		col,row = 1,0
		b = Ribbon.LabelButton(group.frame,
				text="Cycle",
				image=icons.get("STOP"),
				command=self.stopCycle)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Stop current cycle by creating a fluka.stop")

		# ---
		col,row = 1,1
		b = Ribbon.LabelButton(group.frame,
				text="Run",
				image=icons.get("STOP"),
				command=self.stopRun)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Stop run by creating a rfluka.stop")

		# ---
		col,row = 1,2
		b = Ribbon.LabelButton(group.frame,
				text="Kill",
				image=icons.get("x"),
				command=self.killRun)
		b.grid(row=row, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Kill run process if possible")

		# ----
		col,row = 2,0
		b = Ribbon.LabelButton(group.frame,
				text="Refresh",
				image=icons.get("refresh32"),
				compound=tk.TOP,
				command=self.refreshButton)
		b.grid(row=row, column=col, rowspan=3, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Refresh progress information")

		# ---
		col,row = 3,0
		b = Ribbon.LabelButton(group.frame,
				image=icons.get("play32"),
				text="Start",
				compound=tk.TOP,
				command=self.execute)
		b.grid(row=row, rowspan=3, column=col, padx=0, pady=0, sticky=tk.NSEW)
		tkExtra.Balloon.set(b, "Execute run [Ctrl-Enter]")

		self._refreshRecord = []

		return self.ribbon

	# ----------------------------------------------------------------------
	# select default defines
	# ----------------------------------------------------------------------
	def selectDefaultDefines(self, runs=None):
		self.defineList.delete(0,tk.END)

		# Update defines
		lst = []
		selected = []
		for card in self.project.input.cardsSorted("#define",2).values():
			if card._indent != 0: continue
			name = card.name()
			for n,_ in lst:
				if n==name: break
			else:
				lst.append((name, ""))
				if runs is None and card.enable:
					selected.append((name, "", False))

		# Highlight
		if runs:
			selected = []
			for n,v in lst:
				value = None
				count = 0
				for run in runs:
					for rn,rv in run.defineList:
						if n!=rn: continue	# Skip
						if value is None:
							value = rv	# First time set the value
							count = 1
						elif value == rv:
							count += 1	# Count same occurrences
							break
				if value is not None:
					selected.append((n,value,count!=len(runs)))

		# Show defines
		for n,v in lst:
			# Search in selected ones
			for sn,sv,m in selected:
				if n==sn:
					if m:
						self.defineList.insert(tk.END,(tkFlair._MULTI,sn,sv))
					else:
						self.defineList.insert(tk.END,(tkFlair._ON,sn,sv))
					break
			else:
				self.defineList.insert(tk.END,(tkFlair._OFF,n,v))

	# ----------------------------------------------------------------------
	def defineListClick(self, event):
		if len(event.widget.curselection())>1:
			active = event.widget.nearest(event.y)
			if event.widget.selection_includes(active):
				self.toggleDefine()
				return "break"

	# ----------------------------------------------------------------------
	# Button Release on the define listbox
	# ----------------------------------------------------------------------
	def defineListRelease(self, event):
		if len(self.defineList.curselection())>1: return
		active = self.defineList.nearest(event.y)
		if active is not None:
			self.defineList.activate(active)
			try:
				idx = self.defineList.listboxIndex(event.widget)
			except ValueError:
				return
			if idx == 0:
				self.toggleDefine()
			elif idx == 2:
				self.editDefine()

	# ----------------------------------------------------------------------
	# toggle state
	# ----------------------------------------------------------------------
	def toggleDefine(self, event=None):
		lst = self.defineList.listbox(0)
		sel = lst.curselection()
		if not sel: return

		status = lst.get(sel[0])
		if status == tkFlair._ON:		# [X] -> [ ]
			status = tkFlair._OFF
		elif len(self.page.runList.selection())>1:
			if status == tkFlair._OFF:	# [ ] -> [%]
				status = tkFlair._MULTI
			else:
				status = tkFlair._ON	# [%] -> [X]
		else:
			status = tkFlair._ON		# [ ] -> [X]

		for i in sel:
			lst.set(i,status)
			self.defineList.selection_set(i)
			self.defineList._updateSelect()
		self.get()

	# ----------------------------------------------------------------------
	def editDefine(self, event=None):
		if self.defineList["state"] == tk.DISABLED: return
		sel = list(map(int,self.defineList.curselection()))
		idx = self.defineList.index(tk.ACTIVE)
		self.defineList.activate(idx)
		oldValue = self.defineList.listbox(2).get(idx)
		# Protect against accidental after() updates
		self._updating = True
		edit = tkExtra.InPlaceEdit(self.defineList.listbox(2))
		if edit.value is not None and edit.value != oldValue:
			# Activate define
			for i in sel:
				self.defineList.listbox(0).set(i,tkFlair._ON)
				self.defineList.listbox(2).set(i,edit.value)
			self.get()
		self._updating = False

	# ----------------------------------------------------------------------
	# set executable to use
	# ----------------------------------------------------------------------
	def loadExecutable(self, event=None):
		if self.exe["state"] == tk.DISABLED: return
		if not self.page.selected: return
		run = self.page.selected[0]
		fn = bFileDialog.askopenfilename(master=self,
					initialfile=run("exe"),
					title="Load executable")
		if fn:
			run.setExecutable(fn)
			self.page.modified()
			self.exe.set(run("exe"))
			self.flair.refresh("exe", self)

	# ----------------------------------------------------------------------
	def fillQueueList(self):
		queues = tkFlair.config.options(tkFlair._BATCH_SECTION)
		try: queues.remove("spawnname")
		except ValueError: pass
		queues.sort()
		self.queueList.fill(queues)
		if self.queueList.get() == "":
			if queues: self.queueList.set(queues[0])

	# ----------------------------------------------------------------------
	# set Random parameter in the Run Frame
	#   from Fluka manual: WHAT(2) : any number < 9.E8 (see Note 5)
	#                      Default: 54217137
	# ----------------------------------------------------------------------
	def getRandomNumberAsSeed(self):
		if self.rnd["state"] != tk.NORMAL: return
		for run in self.page.runList.selectedRuns():
			if run.isdefault(): continue
			run.setRandomSeed()

		s = self.rnd["state"]
		self.rnd.config(state=tk.NORMAL)
		self.rnd.set(self.page.selected[0]("rnd"))
		self.rnd.config(state=s)

	# ----------------------------------------------------------------------
	# disable overrides
	# ----------------------------------------------------------------------
	def setState(self, state, all_=False):
		# Run / Info frame
		#self.renameButton.config(state=state)
		#self.upButton.config(state=state)
		#self.downButton.config(state=state)
		#self.cloneButton.config(state=state)

		# Override option frame
		self.title.config(state=state)
		self.defineList.config(state=state)
		self.bd.config(state=state)
		self.rnd.config(state=state)
		#self.rndButton.config(state=state)
		self.primaries.config(state=state)
		self.time.config(state=state)
		self.bel.config(state=state)
		self.mode.config(state=state)
		#self.threads.config(state=state)
		self.exe.config(state=state)
		self.exe["background"] = "LightGray" if state==tk.DISABLED else "White"

		# Enable the remaining buttons if not requested
		if not all_: state = tk.NORMAL
		#self.cont.config(state=state)
		self.queueList.config(state=state)
		self.prevbox.config(state=state)
		self.cyclesbox.config(state=state)
		self.lastbox.config(state=state)

	# ----------------------------------------------------------------------
	def toggleField(self, event):
		if len(self.page.selected)==1 and self.page.selected[0].isdefault():
			return

		run = self.page.selected[0]

		if event.widget["state"] != tk.DISABLED:
			state = tk.DISABLED
		else:
			state = tk.NORMAL

		if event.widget in (self.rnd, self.rndButton):
			state = tk.NORMAL
			self.rnd.config(state=state)
			self.rndButton.config(state=state)

		elif event.widget in (self.exe, self.bel):
			self.exe.config(state=state)
			self.bel.config(state=state)

		elif isinstance(event.widget.master, tkExtra.Combobox):
			event.widget.master.config(state=state)

		elif event.widget in self.defineList.listboxes():
			# do not allow for childs
			if not run("parent"):
				self.defineList.config(state=state)

		elif event.widget is self.bd:
			# do not allow for childs
			if not run("parent"):
				event.widget.config(state=state)

		else:
			event.widget.config(state=state)

	# ----------------------------------------------------------------------
	# select item
	# ----------------------------------------------------------------------
	def select(self, event=None):
		#print("*-* RunTab.select")
		if self.page.selected: self.get()

		selected = self.page.runList.selectedRuns()
		if selected == self.page.selected and self.page.valid(): return
		if not selected: return
		self.page.selected = selected

		run = selected[0]
		if not isinstance(run,Project.RunInfo): return

		# Update Information information
		# enable first to allow inserting info
		self.setState(tk.NORMAL)

		self._varSetting = True
		self.last.set(run("last"))
		self.prev.set(run("prev"))
		queue = run("queue")
		if not queue: self.queue = self.project.submit
		if queue:
			self.queueList.set(queue)
		else:
			self.queueList.select(0)

		self.mode.set(run("mode"))
		self.cycles.set(run("last")-run("prev"))
		self._varSetting = False

		self.rnd.set(run("rnd"))
		self.primaries.set(run("primaries"))
		self.time.set(run("time"))

		self.exe.set(run("exe"))
		self.title.delete(0, tk.END)
		if self.page.selected[0].isdefault() and len(self.page.selected)==1:
			self.title.insert(0, self.project.title)
			self.selectDefaultDefines()
		else:
			self.title.insert(0, run("title"))
			self.selectDefaultDefines(self.page.selected)
		#self.refresh()

	# ----------------------------------------------------------------------
	# Change in the cycles
	# ----------------------------------------------------------------------
	def prevChange(self, *abc):
		if self._varSetting: return
		self._varSetting = True
		try: self.last.set(self.prev.get() + self.cycles.get())
		except: pass
		self._varSetting = False

	# ----------------------------------------------------------------------
	# Change in the cycles
	# ----------------------------------------------------------------------
	def lastChange(self, *abc):
		if self._varSetting: return
		self._varSetting = True
		try: self.cycles.set(self.last.get() - self.prev.get())
		except: pass
		self._varSetting = False

	# ----------------------------------------------------------------------
	# Update all fields
	# ----------------------------------------------------------------------
	def get(self, event=None):
		# executable can change from Compile
		# do not save anything if invalid
		if not self.page.selected or self.page.invalid():
			return

		try:
			s = self.queueList.get()
			if self.project.submit != s:
				self.project.submit = s
				self.page.modified()
		except tk.TclError:
			return

		for run in self.page.selected:
			if not isinstance(run,Project.RunInfo): continue

			oldhash = hash(run)
			if self.prevbox["state"] != tk.DISABLED:
				try: run["prev"] = self.prev.get()
				except: pass
				for r in run.family: r["prev"] = run("prev")
			if self.cyclesbox["state"] != tk.DISABLED or \
			   self.lastbox["state"] != tk.DISABLED:
				try: run["last"] = self.last.get()
				except: pass
				for r in run.family: r["last"] = run("last")
			if self.queueList["state"] != tk.DISABLED:
				run["queue"] = self.queueList.get()
				for r in run.family: r["queue"] = run("queue")
			if self.exe["state"] != tk.DISABLED:
				exe = self.exe.get()
				run["exe"] = exe
				for r in run.family: r["exe"] = run("exe")

			# skip the rest for the main run
			if not run.isdefault():
				if self.defineList["state"] != tk.DISABLED:
					olddefs = run.defineList
					run.defineList = []
					for s,n,v in self.defineList.get(0,tk.END):
						if s==tkFlair._ON:
							# add define
							run.defineList.append((n,v))
						elif s==tkFlair._MULTI:
							# Append the existing one if any
							for nn,vv in olddefs:
								if n==nn:
									run.defineList.append((nn,vv))
									break
					for r in run.family: r.defineList = run.defineList
				if self.title["state"] != tk.DISABLED:
					run["title"] = self.title.get().strip()
					for r in run.family: r["title"] = run("title")
				if self.rnd["state"] != tk.DISABLED:
					try:	run["rnd"] = int(self.rnd.get())
					except: run["rnd"] = 0
					# No propagation of rnd in the family
				if self.primaries["state"] != tk.DISABLED:
					try:	run["primaries"] = float(self.primaries.get())
					except: run["primaries"] = 0
					if run["primaries"] < 0:
						self.flair.notify("Error",
							"Invalid number of primaries specified",
							tkFlair.NOTIFY_ERROR)
					else:
						for r in run.family:
							r["primaries"] = run("primaries")
				if self.time["state"] != tk.DISABLED:
					try:	run["time"] = float(self.time.get())
					except: run["time"] = 0
					for r in run.family: r["time"] = run("time")
				if self.mode["state"] != tk.DISABLED:
					run["mode"] = self.mode.get()
					for r in run.family: r["mode"] = run("mode")

			if hash(run) != oldhash:
				#self.flair.redraw() or refresh()???
				self.page.modified()

	# ----------------------------------------------------------------------
	# Continue from last run
	# ----------------------------------------------------------------------
	def continueFromLast(self):
		# Scan directory to find last random number seed
		for run in self.page.selected:
			if not isinstance(run,Project.RunInfo): continue
			last = run.lastRun()
			run["last"] = last + (run("last") - run("prev"))
			run["prev"] = last
			if self.page.selected[0] is run: self.prev.set(last)

	# ----------------------------------------------------------------------
	# Export selected runs to file
	# ----------------------------------------------------------------------
	def export(self):
		ok = 0
		err = 0
		for run in self.page.selected:
			if not isinstance(run,Project.RunInfo): continue
			if run.family:
				for child in run.family:
					if self._exportRun(child):
						ok += 1
					else:
						err += 1
			else:
				if self._exportRun(run):
					ok += 1
				else:
					err += 1
		if err==0:
			self.flair.notify("Exporting runs",
				f"{ok} inputs exported ok",
				tkFlair.NOTIFY_NORMAL)
		else:
			self.flair.notify("Exporting runs",
				f"{ok} inputs exported ok,\n{err} with errors",
				tkFlair.NOTIFY_WARNING)

	#-----------------------------------------------------------------------
	# Export a single run
	# @return False on error, True on Ok
	#-----------------------------------------------------------------------
	def _exportRun(self, run):
		if not isinstance(run,Project.RunInfo): return False
		cwd = run.dir()
		# Check if directory exist
		if not os.path.isdir(cwd):
			os.makedirs(cwd)
		# Prepare input file
		errors = self.project.writeInput(run,cwd)
		if errors:
			for card,err in errors:
				if card:
					self.flair.log(f"{log.ERROR}{card}")
					prefix = log.ERRORCONT
				else:
					prefix = log.ERROR
				self.flair.log(f"{prefix}{err}")
			return False
		return True

	# ----------------------------------------------------------------------
	# Start all runs
	# ----------------------------------------------------------------------
	def execute(self):
		#print("*-* RunTab.execute")
		self.page.busy()
		self.get()
		if not self.page.runList.selection():
			self.flair.notify("Warning",
					"No run selected",
					tkFlair.NOTIFY_WARNING)
			return

		for run in self.page.selected:
			if not isinstance(run,Project.RunInfo): continue
			if run.inputName2()==Project.RunInfo.UNTITLED:
				self.flair.notify("Warning",
					"Please save the project before, "
					"or open the flair project",
					tkFlair.NOTIFY_WARNING)
				break
			if run.family:
				for child in run.family:
					try:
						rc = self.startRun(child)
					except (OSError, IOError) as err:
						self.flair.notify("Error",
							str(err),
							tkFlair.NOTIFY_ERROR)
						rc = 1
						break
			else:
				try:
					rc = self.startRun(run)
				except (OSError, IOError) as err:
					self.flair.notify("Error",
						str(err),
						tkFlair.NOTIFY_ERROR)
					rc = 1
					break

			if rc: break

		self.setState(tk.DISABLED, True)
		self.flair.inputModified()
		self.page.modified()
		if self._timer is None: self.timer()

		# Show present information
		self.refresh()
		self.page.notBusy()

	# ----------------------------------------------------------------------
	# Start one run and try to attach
	# ----------------------------------------------------------------------
	def startRun(self, run):
		#print("*-* RunTab.startRun")
		if run("status") == Project.RunInfo.STATUS_WAIT2ATTACH or \
		   run("status") == Project.RunInfo.STATUS_RUNNING: return

		try:
			log = self.flair.newLog("Run", run.inputBaseName())
			rc = run.start(output=log)
			if rc:
				self.flair.notify("Run Errors",
					"Errors or warnings during writing run input.",
					tkFlair.NOTIFY_ERROR, "Output", self.flair.showOutput)
			self.flair.restoreLog()
			if self.project.submit == "null":
				self.flair.notify("Run -> null",
					"'null' queue was specified\n"
					"Input files were only written.")
			return rc
		except Project.RunException as err:
			self.flair.notify("Error submitting run",
				str(err),
				tkFlair.NOTIFY_ERROR)
			return 1

	# ----------------------------------------------------------------------
	def activate(self):
		#print("*-* RunTab.activate")
		# find all runs and populate exe listbox
		runs = {run("exe") for run in self.project.runs() if run("exe")}
		runs.update({"", f"<{FlukaConfig.DEFAULT_EXE}>", f"<{FlukaConfig.DEFAULT_EXE2}>"})
		runs.add(self.project.compile.name)
		self.exe.fill(list(sorted(runs)))
		self.select()
		self.refreshForce()
		self.refreshStatus()
		if self._timer is None: self.timer()

	# ----------------------------------------------------------------------
	# Clean files
	# ----------------------------------------------------------------------
	def clean(self):
		nfiles = 0
		if not self.page.selected:
			self.flair.notify("Clean",
					"No runs selected",
					tkFlair.NOTIFY_WARNING,
					"Output",
					self.flair.showOutput)
			return
		if tkFlair.askyesno("Warning",
				"Do you want to remove all generated files?",
				parent=self.ribbon):
			warn = False
			for run in self.page.selected:
				if not isinstance(run,Project.RunInfo): continue
				if run.family:
					for child in run.family:
						n = self._cleanRun(child)
						if n < 0:
							warn = True
							break
						nfiles += n
				n = self._cleanRun(run)
				if n < 0:
					warn = True
					break
				nfiles += n
			if warn:
				self.flair.notify("Clean",
						f"{nfiles} run files removed with warnings",
						tkFlair.NOTIFY_WARNING,
						"Output",
						self.flair.showOutput)
			else:
				self.flair.notify("Clean",
						f"{nfiles} run files removed",
						tkFlair.NOTIFY_NORMAL,
						"Output",
						self.flair.showOutput)
			self.page.setStatus("Run files cleaned","DarkBlue")
		self.refresh()
		if nfiles>0: self.page.runList.updateStatus()

	# ----------------------------------------------------------------------
	def _cleanRun(self, run):
		if run("status") in (Project.RunInfo.STATUS_WAIT2ATTACH,
					Project.RunInfo.STATUS_RUNNING):
			self.flair.notify("Running",
				f"Input '{run.name}' is running. "
				"Please stop before deleting all files",
				tkFlair.NOTIFY_WARNING)
			return -1

		err,nfiles = run.clean()
		if err:
			self.flair.notify("Errors during run cleaning",
				"Please check the output for error messages during cleaning",
				tkFlair.NOTIFY_ERROR)
		run["status"] = Project.RunInfo.STATUS_NOT_RUNNING
		return nfiles

	# ----------------------------------------------------------------------
	def _stop(self, cycle=True):
		for run in self.page.selected:
			run.stop(cycle)

	# ----------------------------------------------------------------------
	# Stop smoothly all runs
	# ----------------------------------------------------------------------
	def stopCycle(self):
		if not tkFlair.askyesno("Stop Running Cycle(s)",
				"Are you sure you want to stop the running "
				"cycle of the selected run(s)",
				parent=self.ribbon):
			return
		self._stop(True)

	# ----------------------------------------------------------------------
	# Stop smoothly all runs
	# ----------------------------------------------------------------------
	def stopRun(self):
		if not tkFlair.askyesno("Stop Run(s)",
			"Are you sure you want to stop the selected run(s)",
			parent=self.ribbon):
			return
		self._stop(False)

	# ----------------------------------------------------------------------
	# Kill all runs
	# ----------------------------------------------------------------------
	def killRun(self):
		if not tkFlair.askyesno("Stop Run(s)",
			"Are you sure you want to KILL the selected run(s)",
			parent=self.ribbon):
			return

		#---------------------------------------------------------------
		def _killRun(run):
			try:
				run.kill()
			except:
				self.flair.notify("Kill Failed",
					f"Kill Run '{run.name}' has failed. "
					"Probably is running through a batch system. "
					"Please kill it manually.",
					tkFlair.NOTIFY_ERROR)
		#---------------------------------------------------------------

		for run in self.page.selected:
			if not isinstance(run,Project.RunInfo): continue
			if run.family:
				for child in run.family:
					_killRun(child)
			else:
				_killRun(run)
		self.refresh()
		self.page.runList.updateStatus()
		self.setState(tk.DISABLED)

	# ----------------------------------------------------------------------
	# Show a small Olympic animation
	# ----------------------------------------------------------------------
	def animate(self):
		if not self.hoframes:
			with open(os.path.join(tkFlair.prgDir,"icons","ho.pak"),"rb") as fin:
				while True:
					try:
						s = struct.unpack("i",fin.read(4))[0]
					except struct.error:
						break
					data = fin.read(s)
					self.hoframes.append(tk.PhotoImage(data=data, format="gif"))
				self.holabel  = tk.Label(self)
				self.hoframe  = 0
		if not self.holabel.winfo_viewable():
			self.holabel.place(anchor=tk.S, relx=0.5, rely=1)
		self.holabel.configure(image=self.hoframes[self.hoframe])
		self.hoframe += 1
		if self.hoframe == len(self.hoframes):
			self.hoframe = 0
		if self.hocnt>0:
			self.after(self.hodt, self.animate)
			self.hocnt -= 1
		else:
			self.holabel.place_forget()

	# ----------------------------------------------------------------------
	# Refresh button clicked...
	# ----------------------------------------------------------------------
	def refreshButton(self):
		#print("*-* RunTab.refresh")
		self._refreshRecord.append(time.time())
		if len(self._refreshRecord)>5:
			del self._refreshRecord[0]
			dt    = self._refreshRecord[-1] - self._refreshRecord[0]
			speed = (len(self._refreshRecord)-1) / dt
			if dt < 1.0:
				shown = int(self.flair.notifyIsVisible())
				try:
					shown += 2*int(self.holabel.winfo_viewable())
				except AttributeError:
					pass
				if shown==0: shown = random.randint(1,2)
				if shown==1:
					self.flair.notify("Bravo!!!!",
						"Good work!\n"
						"Keep clicking to run faster... :)\n"
						f"Average speed: {speed:.3} clicks/s")
				else:
					self.hodt = min(500, max(10, int(250/speed)))
					if self.hocnt==0:
						self.hocnt = 50
						self.animate()
					else:
						self.hocnt = 50
		self.refreshForce()

	# ----------------------------------------------------------------------
	# Force refresh of selected runs
	# ----------------------------------------------------------------------
	def refreshForce(self):
		#print("\n*-* RunTab.refreshForce")
		for run in self.page.runList.selectedRuns():
			run.refreshAsThread()
		# display refreshed information
		self.refresh()

	# ----------------------------------------------------------------------
	# Update run progress information
	# and Enable or Disable the fields
	# ----------------------------------------------------------------------
	#@tool.timeit
	def refresh(self, fromtimer=False):
		#print("\n*-* RunTab.refresh")
		#Global.dump("*-* RunTab.refresh")
		#self.project.runs._check()
		if self._updating: return
		self._updating = True

		# Update messages
		self.pstatus["text"]    = ""
		self.pinput["text"]     = ""
		self.pdir["text"]       = ""
		self.ptime["text"]      = ""
		self.pstarted["text"]   = ""
		self.pelapsed["text"]   = ""
		self.premain["text"]    = ""
		self.peta["text"]       = ""
		self.prunremain["text"] = ""

		run = self.page.runList.focusedRun()
		if run is None or isinstance(run,Project.GroupInfo):
			self._updating = False
			self.setState(tk.DISABLED)
			return

		self.page.runList.refreshRuns()

		# Show active run
		if not self.page.selected or \
		   self.page.selected[0] not in self.project.runs():
			self._updating = False
			return

		run = self.page.selected[0]
		if not isinstance(run,Project.RunInfo):
			self._updating = False
			return

		# Find information
		if not run.family and (run("pid")>0
		   or run("status") in (Project.RunInfo.STATUS_RUNNING,
					Project.RunInfo.STATUS_TIMEOUT)):
			run.refresh(True)

		self.pstatus.config(text=Project.RunInfo.STATUS_STR[run("status")],
				foreground=Project.RunInfo.statusColor[run("status")])
		self.pinput["text"] = self.project.relpath(run.inputName())

		# Enable disable fields
		if run.family or run("status") \
		   not in (Project.RunInfo.STATUS_RUNNING, Project.RunInfo.STATUS_TIMEOUT):
			# ------------------------------------
			# Finally enable/disable fields
			# ------------------------------------
			if len(self.page.selected)==1:
				if self.page.selected[0].isdefault() or run("parent"):
					# Disable if child apart from rnd
					self.setState(tk.DISABLED)
					# Special treatment exe and rnd
					if run("parent"):
						self.rnd["state"] = tk.NORMAL
					else:
						self.exe["background"] = "White"
						self.exe["state"] = tk.NORMAL
						self.bel["state"] = tk.NORMAL
				else:
					self.setState(tk.NORMAL)
			elif len(self.page.selected)>1:
				if not fromtimer:
					self.setState(tk.DISABLED, True)
			else:
				self.setState(tk.NORMAL)
			self.cyclePBar.setProgress(0.0, 0.0, "")
			self.primaryPBar.setProgress(0.0, 0.0, "")
		else:
			self.setState(tk.DISABLED, True)
			self.cyclePBar.setProgress(0.0, 0.0, "")
			self.primaryPBar.setProgress(0.0, 0.0, "")

		# Individual run information
		if not run.family:
			if run("pid"):
				self.pdir["text"] = "%s%d" % (run.config().CYCLE,run("pid"))

			if run("status")== Project.RunInfo.STATUS_RUNNING:
				if run.cycle>0:
					self.cyclePBar.setLimits(run("prev"), run("last"))
					self.cyclePBar.setProgress(run.cycle)
				else:
					self.cyclePBar.setProgress(0.0, 0.0, "")

				nprim = run.handled + run.remaining
				if run.handled>0:
					self.ptime["text"] = tool.friendlyTime(run.timeperprim)
					self.primaryPBar.setLimits(0, nprim)
					self.primaryPBar.setProgress(run.handled+1)
					self.pelapsed["text"] = tool.friendlyTime(
						run.handled*run.timeperprim)
					cycle_remain = min(run.remaining*run.timeperprim,
							run.maxTime - run.handled*run.timeperprim)
					self.premain["text"] = tool.friendlyTime(cycle_remain)
					run_remain = cycle_remain + (run("last")-run.cycle) * \
							(min(nprim*run.timeperprim, run.maxTime)
								+ run.iniTime)
					self.prunremain["text"] = tool.friendlyTime(run_remain)
				else:
					self.primaryPBar.setProgress(0.0, 0.0, "")

				self.peta["text"] = time.strftime(
						bFileDialog._TIME_FORMAT,
						time.localtime(time.time()
						  + (run.remaining + (run("last")-run.cycle)*nprim)
						  * run.timeperprim))

			if run.startTime: self.pstarted["text"] = run.startTime

		else:	# For a family of runs
			startTime   = ""
			cyclesDone  = 0
			cyclesTotal = 0
			running     = 0
			handled     = 0
			remain      = 0
			for child in run.family:
				if not startTime:
					startTime = child.startTime
				else:
					startTime = min(startTime, child.startTime)
				if child("status")==Project.RunInfo.STATUS_RUNNING:
					running += 1
					handled += child.handled
					remain  += child.remaining
				runcycles = child("last") - child("prev")
				cyclesTotal += runcycles
				cyclesDone  += child.cycle - child("prev")
			if running>0:
				self.cyclePBar.setLimits(0, cyclesTotal, running)
				self.cyclePBar.setProgress(cyclesDone)
			if handled+remain>0:
				self.primaryPBar.setLimits(0, handled + remain)
				self.primaryPBar.setProgress(handled+1)
			self.pstarted["text"] = startTime
		self._updating = False

	# ----------------------------------------------------------------------
	# Refresh Status
	# ----------------------------------------------------------------------
	def refreshStatus(self):
		#print("*-* RunTab.refreshStatus")
		if self.flair is None: return
		# Count processes
		running = 0
		total   = 0
		for run in self.project.runs():
			if run.family: continue
			if run("status") == Project.RunInfo.STATUS_RUNNING: running += 1
			total += 1
		self.page.setStatus(f"Running {running} out of {total}")

	# ----------------------------------------------------------------------
	# Attach to selected runs
	# ----------------------------------------------------------------------
	def attach(self):
		#print("*-* RunTab.attach")
		self.page.runList.attach()
		self.refresh()

	# ----------------------------------------------------------------------
	# Check is anything is running or about to run.
	# @return True/False, suggested-refresh-Interval
	# ----------------------------------------------------------------------
	def _isAnythingRunning(self):
		dt = min(self._lastDt, Project.refreshInterval)
		running = False
		for run in self.project.runs():
			if run("status") == Project.RunInfo.STATUS_WAIT2ATTACH:
				dt = Project.refreshInterval // 10
				running = True
				break	# don't search any more
			if run("status") in (Project.RunInfo.STATUS_RUNNING,
					     Project.RunInfo.STATUS_TIMEOUT):
				if not running and dt < Project.refreshInterval:
					# do only the first time
					# progressively increase refresh interval
					dt += Project.refreshInterval // 10
				running = True
				break
		self._lastDt = dt
		return running, dt

	# ----------------------------------------------------------------------
	# When the window is unmapped, (e.g. change of desktop) then it stops
	# the refreshing and doesn't restart it
	# ----------------------------------------------------------------------
	def checkRefresh(self, event=None):
		#print("*-* RunTab.checkRefresh")
		if self._isAnythingRunning()[0]:
			self.timer()
		else:
			self.refresh(fromtimer=True)

	# ----------------------------------------------------------------------
	# Refresh timer function
	# ----------------------------------------------------------------------
	def timer(self, event=None):
		#print("*-* RunTab.timer")
		try:
			if self._timer: self.after_cancel(self._timer)
		except tk.TclError:
			pass
		self._timer = None
		if not self.winfo_ismapped(): return
		self.refresh(fromtimer=True)

		# check if all runs have finished or someone is waiting to attach
		running, dt = self._isAnythingRunning()
		if running:
			self._timer = self.after(dt, self.timer)
		else:
			self.refresh(fromtimer=True)

	# ----------------------------------------------------------------------
	def help(self):
		Manual.show("","Run:Run")

#===============================================================================
# Select Running Directory
#===============================================================================
class SelectRunningDir(tk.Toplevel):
	def __init__(self, master, run, dirlist):
		super().__init__(master)
		self.transient(master)

		l = tk.Label(self, text=f"Run: {run.name}",
			justify=tk.RIGHT, anchor=tk.NW, pady=2)
		l.pack(side=tk.TOP)

		l = tk.Label(self, text="Directories:",
			justify=tk.RIGHT, anchor=tk.NW, pady=2)
		l.pack(side=tk.TOP)

		self.listbox = tkExtra.ExListbox(self, borderwidth=0,
			selectmode=tk.BROWSE,
			selectborderwidth=0,
			background="White",
			takefocus=True,
			exportselection=tk.FALSE)
		self.listbox.pack(expand=tk.YES,fill=tk.BOTH)

		frame = tk.Frame(self)
		tk.Button(frame, text="Cancel", command=self.cancel).pack(side=tk.RIGHT)
		tk.Button(frame, text="Ok", command=self.ok).pack(side=tk.RIGHT)
		self.listbox.bind("<Double-1>",self.ok)
		self.bind("<Return>",   self.ok)
		self.bind("<KP_Enter>", self.ok)
		self.bind("<Escape>",   self.cancel)
		frame.pack(side=tk.BOTTOM)

		# Populate directories
		for d,_ in dirlist:
			self.listbox.insert(tk.END,d)

		self.listbox.activate(0)
		self.listbox.selection_set(0)

		self.sel = -1
		self.title("Select Running Dir")

	# ----------------------------------------------------------------------
	def show(self):
		self.focus_set()
		self.listbox.focus_set()
		self.transient(self.master)
		self.wait_window()
		return self.sel

	# ----------------------------------------------------------------------
	def ok(self, event=None):
		self.sel = self.listbox.index(tk.ACTIVE)
		self.destroy()

	# ----------------------------------------------------------------------
	def cancel(self, event=None):
		self.destroy()
