# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   5-Apr-2007

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import sys
import ssl
import time
import codecs
import tkinter as tk
from tkinter import ttk
from urllib.request import urlopen

import icons
import tkFlair
import tkExtra

from Project  import FLAIRVER_PAT
from fluka    import FlukaConfig
#from moira    import MoiraConfig
from RichText import RichText

# Issue https://gitlab.cern.ch/flair/flair/-/merge_requests/19
ssl._create_default_https_context = ssl._create_unverified_context

FLAIR_VERSION_TAG = "http://flair.web.cern.ch/flair/download/version.tag"
FLAIR_DOWNLOAD    = "http://cern.ch/flair/download.html"
FLUKA_VERSION_TAG = "http://flukafiles.web.cern.ch/flukafiles/Version.tag"
FLUKA_DOWNLOAD    = "https://fluka.cern/download/latest-fluka-release"

#===============================================================================
# Check for updates in flair and fluka
#===============================================================================
class CheckUpdateDialog(tk.Toplevel):
	def __init__(self, master, flair_version, fluka_version):
		super().__init__(master)
		self.transient(master)
		self.title("Check for updates")

		# Variables
		self.flair_version  = flair_version
		self.fluka_version  = fluka_version
		self.download_flair = False
		self.download_fluka = False

		# ----
		frame = tk.LabelFrame(self, text="Flair", padx=3, pady=5)
		frame.pack(side=tk.TOP, fill=tk.BOTH)

		l = tk.Label(frame, text="Installed Version:")
		l.grid(row=0, column=0, sticky=tk.E, pady=1)

		l = tk.Label(frame, text=flair_version, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		l.grid(row=0, column=1, sticky=tk.EW)
		tkExtra.Balloon.set(l, "Running version of flair")

		l = tk.Label(frame, text="Web Version:")
		l.grid(row=1, column=0, sticky=tk.E, pady=1)

		self.flair_webversion = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.flair_webversion.grid(row=1, column=1, sticky=tk.EW)
		tkExtra.Balloon.set(self.flair_webversion,
			"Version on the web site of flair")
		l = tk.Label(frame, text="Released on:")
		l.grid(row=2, column=0, sticky=tk.E, pady=1)

		self.flair_released = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.flair_released.grid(row=2, column=1, sticky=tk.EW)
		tkExtra.Balloon.set(self.flair_released,
			"Release date of the flair web version")

		frame.grid_columnconfigure(1, weight=1)

		# ----
		frame = tk.LabelFrame(self, text="Fluka", padx=3, pady=5)
		frame.pack(fill=tk.BOTH)

		l = tk.Label(frame, text="Installed Version:")
		l.grid(row=0, column=0, sticky=tk.E, pady=1)

		l = tk.Label(frame, text=fluka_version, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		l.grid(row=0, column=1, sticky=tk.EW)
		tkExtra.Balloon.set(l, "FLUKA version on the {FlukaDir} directory")

		l = tk.Label(frame, text="Web Version:")
		l.grid(row=1, column=0, sticky=tk.E, pady=1)

		self.fluka_webversion = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.fluka_webversion.grid(row=1, column=1, sticky=tk.EW)
		tkExtra.Balloon.set(self.fluka_webversion,
			"Version on the web site of fluka")

		l = tk.Label(frame, text="Released on:")
		l.grid(row=2, column=0, sticky=tk.E, pady=1)

		self.fluka_released = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.fluka_released.grid(row=2, column=1, sticky=tk.EW)
		frame.grid_columnconfigure(1, weight=1)
		tkExtra.Balloon.set(self.flair_released,
			"Release date of the FLUKA web version")

		# ----
		frame = tk.LabelFrame(self, text="Check Interval", padx=3, pady=5)
		frame.pack(fill=tk.BOTH)

		l = tk.Label(frame, text="Last Check:")
		l.grid(row=0, column=0, sticky=tk.E, pady=1)

		# Last check
		lastCheck = tkFlair.config.getInt(tkFlair._FLAIR_SECTION,"lastcheck",0)
		if lastCheck == 0:
			lastCheckStr = "unknown"
		else:
			lastCheckStr = time.asctime(time.localtime(lastCheck))

		l = tk.Label(frame, text=lastCheckStr, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		l.grid(row=0, column=1, sticky=tk.EW)
		tkExtra.Balloon.set(l, "Date of last checking")

		l = tk.Label(frame, text="Interval (days):")
		l.grid(row=1, column=0, sticky=tk.E, pady=1)

		checkInt = tkFlair.config.getInt(tkFlair._FLAIR_SECTION,"checkinterval",30)
		self.checkInterval = tk.IntVar()
		self.checkInterval.set(checkInt)

		s = tk.Spinbox(frame, text=self.checkInterval, from_=0, to_=365,
				background="White")
		s.grid(row=1, column=1, sticky=tk.EW)
		frame.grid_columnconfigure(1, weight=1)
		tkExtra.Balloon.set(s, "Days-interval to remind again for checking")

		# ----
		frame = tk.Frame(self)
		frame.pack(side=tk.BOTTOM,fill=tk.X)
		b = tk.Button(frame,
				text="Close",
				image=icons.get("x"),
				compound=tk.LEFT,
				command=self.later)
		b.pack(side=tk.RIGHT)

		self.checkButton = tk.Button(frame,
				text="Check Now",
				image=icons.get("GLOBAL"),
				compound=tk.LEFT,
				command=self.check)
		self.checkButton.pack(side=tk.RIGHT)
		tkExtra.Balloon.set(self.checkButton,
				"Check the web site for new versions of flair and FLUKA")

		self.bind('<Escape>', self.close)

		#x = master.winfo_rootx() + 200
		#y = master.winfo_rooty() + 50
		#self.geometry("+%d+%d" % (x,y))
		self.wait_window()

	# ----------------------------------------------------------------------
	def check(self):
		# Find latest version of flair
		self.flair_webversion.config(text="...waiting...")
		self.fluka_webversion.config(text="...waiting...")

		version  = None
		released = ""
		self.flair_webversion.config(
			text="Establishing connection\nhttp://flair.web.cern.ch/flair")
		self.flair_webversion.update_idletasks()
		try:
			m = FLAIRVER_PAT.match(urlopen(FLAIR_VERSION_TAG,
						timeout=10).read().decode(errors="replace"))
			if m:
				version, released = m.groups()
				self.flair_webversion.config(text=version)
				self.flair_released.config(text=released)
				if self.flair_version != version:
					self.download_flair = True
					self.flair_webversion.config(fg="DarkGreen")
					self.flair_released.config(fg="DarkGreen")
			else:
				self.flair_webversion.config(text="Error in connection",
								fg="Red")
		except Exception as err:
			self.flair_webversion.config(text=str(err), fg="Red")

		version  = None
		released = ""
		self.fluka_webversion.config(text="Establishing connection\nfluka.cern")
		self.fluka_webversion.update_idletasks()
		try:
			m = FlukaConfig.VERSION_PAT.match(
					urlopen(FLUKA_VERSION_TAG,
						timeout=10).read().decode(errors="replace"))
			if m:
				version, released = m.groups()
				self.fluka_webversion.config(text=version)
				self.fluka_released.config(text=released)
				if self.fluka_version != version:
					self.download_fluka = True
					self.fluka_webversion.config(fg="DarkGreen")
					self.fluka_released.config(fg="DarkGreen")
			else:
				self.fluka_webversion.config(text="Error in connection",
							fg="Red")
		except Exception as err:
			self.fluka_webversion.config(text=str(err), fg="Red")

		if self.download_flair or self.download_fluka:
			self.checkButton.config(text="Download", command=self.download)
			tkExtra.Balloon.set(self.checkButton,
					"Open web browser to download flair/FLUKA")
		else:
			self.checkButton.config(state=tk.DISABLED)
		#self.laterButton.config(state=tk.DISABLED)

		# Save today as lastcheck date
		tkFlair.config.set(tkFlair._FLAIR_SECTION,
			"lastcheck", str(int(time.time())))

	# ----------------------------------------------------------------------
	def later(self):
		# Save today as lastcheck date
		tkFlair.config.set(tkFlair._FLAIR_SECTION,
			"lastcheck", str(int(time.time())))
		self.close()

	# ----------------------------------------------------------------------
	def download(self):
		if self.download_flair:
			tkFlair.openurl(FLAIR_DOWNLOAD)
		if self.download_fluka:
			tkFlair.openurl(FLUKA_DOWNLOAD)

	# ----------------------------------------------------------------------
	def close(self, event=None):
		try:
			tkFlair.config.set(tkFlair._FLAIR_SECTION,
				"checkinterval", str(int(self.checkInterval.get())))
		except TypeError:
			pass
		self.destroy()

	#-----------------------------------------------------------------------
	# Check if interval has passed from last check
	#-----------------------------------------------------------------------
	@staticmethod
	def verify():
		lastCheck = tkFlair.config.getInt(tkFlair._FLAIR_SECTION,
						"lastcheck", 0)
		if lastCheck == 0:	# Unknown
			return False
		checkInt = tkFlair.config.getInt(tkFlair._FLAIR_SECTION,
						"checkinterval", 30)
		if checkInt == 0:	# never check again
			return True
		return lastCheck + checkInt*86400 > int(time.time())

#===============================================================================
# Display license and connect to server to register
#===============================================================================
class License(tk.Toplevel):
	"""xyrvqv"""
	#-----------------------------------------------------------------------
	def __init__(self, master, agreed=False):
		super().__init__(master)
		self.transient(master)
		self.title("Flair License")
		self._agreed = tk.NORMAL if agreed else tk.DISABLED

		# ----
		frame = tk.LabelFrame(self, text="Flair License",
				font="Helvetica,18",
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR,
				padx=3, pady=5)
		frame.pack(side=tk.TOP, fill=tk.BOTH, expand=tk.YES)

		self.text = RichText(frame,
					width=75,
					wrap=tk.WORD,
					takefocus=True)
		self.text.pack(side=tk.LEFT, fill=tk.BOTH, expand=tk.YES)
		self.scrollbar = ttk.Scrollbar(frame, orient=tk.VERTICAL,
					command=self.text.yview)
		self.scrollbar.pack(side=tk.RIGHT, fill=tk.Y)
		self.text.config(yscrollcommand=self._scrollText)

		#self.text.bind("<Up>",   lambda e,s=self.text : s.yview(tk.SCROLL, -1, tk.UNITS))
		#self.text.bind("<Down>", lambda e,s=self.text : s.yview(tk.SCROLL, +1, tk.UNITS))
		self.bind("<Up>",   lambda e,s=self.text : s.yview(tk.SCROLL, -1, tk.UNITS))
		self.bind("<Down>", lambda e,s=self.text : s.yview(tk.SCROLL, +1, tk.UNITS))
		self.text.focus_set()

		# ----
		frame = tk.Frame(self)
		frame.pack(side=tk.BOTTOM, fill=tk.X)

		tk.Label(frame, text="Scroll to the bottom of the LICENSE to activate the 'I AGREE'",
				foreground="DarkRed"). \
			grid(row=0, column=0, columnspan=3, sticky=tk.EW)

		self.disagreeBtn = tk.Button(frame,
				text=" I don't agree",
				font=("Helvetica","12"),
				image=icons.get("x"),
				compound=tk.LEFT,
				command=self.exit)
		self.disagreeBtn.grid(row=1, column=0, sticky=tk.NS)

		#self.downloadBtn = tk.Button(frame,
		#		text="I AGREE and download my free key",
		#		font=("Helvetica","14", "bold"),
		#		image=icons.get("download"),
		#		compound=tk.LEFT,
		#		command=self.download)
		#self.downloadBtn.grid(row=1, column=1, sticky=tk.NS)

		self.agreeBtn = tk.Button(frame,
				text="I AGREE",
				font=("Helvetica","14","bold"),
				image=icons.get("ok"),
				compound=tk.LEFT,
				command=self.iagree)
		self.agreeBtn.grid(row=1, column=2, sticky=tk.NS)

		frame.grid_columnconfigure(1, weight=1)

		self.text.setText(tkFlair.LICENSE)
		self.text.config(state=tk.DISABLED)
		self.text.see("1.0")

		self.protocol("WM_DELETE_WINDOW", self.close)
		self.bind('<Escape>', self.close)

		#x = master.winfo_rootx() + 200
		#y = master.winfo_rooty() + 50
		#self.geometry("+%d+%d" % (x,y))
		try:
			if not self.winfo_ismapped():
				self.wait_visibility()
			self.grab_set()
		except tk.TclError:
			pass
		self.wait_window()

	#-----------------------------------------------------------------------
	@staticmethod
	def key():
		return codecs.decode(License.__doc__,"rot13")

	#-----------------------------------------------------------------------
	@staticmethod
	def license():
		skey = tkFlair.config.getStr(tkFlair._FLAIR_SECTION,
				License.key(), "0.0")
		return map(int, skey.split("."))

	#-----------------------------------------------------------------------
	# Check if interval has passed from last check
	#-----------------------------------------------------------------------
	@staticmethod
	def verify():
		try:
			key,ver = License.license()
		except ValueError:
			return False
		if key == 0 or ver<tkFlair.LICENSE_VERSION: return False
		return key + 2*365*86400 > int(time.time())

	#-----------------------------------------------------------------------
	def _scrollText(self, *args):
		self.scrollbar.set(*args)
		# Check for end of page to enable the buttons
		lo,hi = self.scrollbar.get()
		state = tk.NORMAL if hi>0.975 else self._agreed
		self.agreeBtn.config(state=state)
		#self.downloadBtn.config(state=state)

	#-----------------------------------------------------------------------
	#def download(self, event=None):
	#	tkFlair.config.setStr(tkFlair._FLAIR_SECTION,
	#			self.key(),
	#			int(time.time()))
	#	# download a key
	#	self.destroy()

	#-----------------------------------------------------------------------
	def iagree(self, event=None):
		skey = f"{int(time.time())}.{tkFlair.LICENSE_VERSION}"
		tkFlair.config.setStr(tkFlair._FLAIR_SECTION, self.key(), skey)
		self.destroy()

	#-----------------------------------------------------------------------
	def exit(self, event=None):
		if self._agreed == tk.NORMAL:
			tkFlair.config.remove_option(tkFlair._FLAIR_SECTION, self.key())
			tkFlair.writeIni()
		self.destroy()
		sys.exit()

	#-----------------------------------------------------------------------
	def close(self, event=None):
		if self._agreed == tk.NORMAL:
			self.destroy()
		else:
			self.exit()
