#!/bin/env python3
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   28-Feb-2007

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import tkinter as tk
from tkinter import ttk

import Plot
import icons
import Manual
import tkFlair
import tkExtra
import BasePlot

from fluka import FlukaParticle

#===============================================================================
# USRBIN Data Plot Frame
#===============================================================================
class UserdumpPlotFrame(BasePlot.BasePlotFrame):
	def __init__(self, master, flair):
		super().__init__(master, flair)

		# --- Variables ---
		self.case    = -1
		self.mgfile  = ""
		self.limfile = ""
		self.dmin    = None
		self.dmax    = None

		# === File & Detector ===
		frame = tk.LabelFrame(self, text="Collision File ")
		frame.pack(side=tk.TOP, expand=tk.YES, fill=tk.BOTH, padx=3)

		row, col = 0, 0
		tk.Label(frame, text="File:").grid(row=row,
				column=col, sticky=tk.E)
		col += 1; span = 3
		self.datafile = tkExtra.FileEntry(frame,
						title="Load USERDUMP file",
						initialdir=self.project.dir(),
						filetypes=[
							("USERDUMP files","*dump*"),
							("All","*")],
						foreground="DarkBlue", background="White")
		self.datafile.bind("<<FileChanged>>", self.load)
		self.datafile.grid(row=row, column=col, columnspan=span, sticky=tk.EW)
		tkExtra.Balloon.set(self.datafile, "USERDUMP binary output file")
		self.widget["datafile"] = self.datafile

		# --
		col += span
		self.particle_list = tkExtra.ExListbox(frame,
				selectmode=tk.MULTIPLE,
				selectborderwidth=0,
				background="White",
				takefocus=True,
				height=6,
				exportselection=tk.FALSE)
		self.particle_list.grid(row=row, rowspan=4, column=col, sticky=tk.NSEW)

		col += 1
		sb = ttk.Scrollbar(frame, orient=tk.VERTICAL, takefocus=False,
				command=self.particle_list.yview)
		sb.grid(row=row, rowspan=4, column=col, sticky=tk.NS)
		self.particle_list.config(yscrollcommand=sb.set)
		self.widget["@particles"] = self.particle_list #,sb

		# ---
		row, col = row+1, 0
		tk.Label(frame, text="From Event:").grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		var = tk.IntVar()
		var.set(1)
		s = tk.Spinbox(frame, text=var, from_=1, to_=10000000,
				background="White",width=5)
		tkExtra.Balloon.set(s, "Starting event number")
		s.grid(row=row, column=col, sticky=tk.W)
		self.widget["from"] = s
		self.variable["from"] = var

		# ---
		col += 1
		tk.Label(frame, text="# Events:").grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		var = tk.IntVar()
		var.set(0)
		s = tk.Spinbox(frame, text=var, from_=0, to_=1000000,
				background="White",width=7)
		tkExtra.Balloon.set(s, "Total events to load")
		s.grid(row=row, column=col, sticky=tk.W)
		self.widget["n"] = s
		self.variable["n"] = var

		# ===
		row, col = row+1, 0
		tk.Label(frame, text="Emin:").grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		e = tkExtra.FloatEntry(frame, background="White")
		e.grid(row=row, column=col, sticky=tk.W)
		self.widget["emin"] = e

		# ---
		col += 1
		tk.Label(frame, text="Emax:").grid(row=row,
				column=col, sticky=tk.E)
		col += 1
		e = tkExtra.FloatEntry(frame, background="White")
		e.grid(row=row, column=col, sticky=tk.W)
		self.widget["emax"] = e

		frame.grid_columnconfigure(3, weight=2)	# <<<<-------
		frame.grid_columnconfigure(4, weight=1)

		# ============= Info =============
		infoframe = tk.Frame(self)
		infoframe.pack(side=tk.TOP, expand=tk.TRUE, fill=tk.BOTH, pady=3, padx=3)

		# Tabs
		self.tabPages = ttk.Notebook(infoframe)
		self.tabPages.pack(side=tk.LEFT, expand=tk.TRUE, fill=tk.BOTH)
		#self.widget["case"] = self.tabPages

		# ------- Geometry ---------
		frame = tk.LabelFrame(infoframe, text="Geometry ")
		frame.pack(side=tk.RIGHT, fill=tk.Y, padx=3)

		# ---
		row, col = 0, 0
		tk.Label(frame, text="Use:").grid(row=row, column=col, sticky=tk.E)

		col += 1
		self.geo = tkExtra.Combobox(frame, width=10)
		self.geo.insert(tk.END, Plot.GEO_NO)
		self.geo.set(Plot.GEO_NO)
		self.geo.grid(row=row, column=col, columnspan=2, sticky=tk.EW)
		self.widget["geo"] = self.geo

		# ---
		row, col = row+1, 0
		tk.Label(frame, text="Axes:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.geo_axes = tkExtra.Combobox(frame)
		self.geo_axes.fill(Plot.COORD)
		self.geo_axes.set(Plot.COORD[0])
		self.geo_axes.grid(row=row, column=col, sticky=tk.EW)
		self.widget["axes"] = self.geo_axes

		# ---
		#row, col = row+1, 0
		#tk.Label(frame, text="Plot:").grid(row=row, column=col, sticky=tk.E)
		#col += 1
		#self.plot_axes = tk.StringVar()
		#self.plot_axes.set(Plot.GP_AXES[0])
		#self.plot_axes_menu = tk.OptionMenu(frame, self.plot_axes,
		#			*Plot.GP_AXES)
		#self.plot_axes_menu.grid(row=row, column=col, sticky=tk.W)

		frame.grid_columnconfigure(1, weight=1)

		# ============= Continuous =============
		pageframe = tk.Frame(self.tabPages)
		self.tabPages.add(pageframe,
				text     = Plot.USERDUMP_CASE[0],
				image    = icons.get("USRTRACK"),
				compound = tk.LEFT)

		# ============= Point =============
		pageframe = tk.Frame(self.tabPages)
		self.tabPages.add(pageframe,
				text     = Plot.USERDUMP_CASE[1],
				image    = icons.get("point"),
				compound = tk.LEFT)

		# ============= Source =============
		pageframe = tk.Frame(self.tabPages)
		self.tabPages.add(pageframe,
				text     = Plot.USERDUMP_CASE[2],
				image    = icons.get("SOURCE"),
				compound = tk.LEFT)

		# ----------------
		frame = tk.LabelFrame(pageframe, text="Type ")
		frame.pack(side=tk.LEFT,fill=tk.Y)

		self.source_type = tk.StringVar()
		self.source_type.set(Plot.HISTOGRAM)
		b1 = tk.Radiobutton(frame, text=Plot.HISTOGRAM,
				variable=self.source_type, value=Plot.HISTOGRAM,
				command=self.typeChange, anchor=tk.W)
		tkExtra.Balloon.set(b1, "Draw one variable histogram")
		b1.pack(side=tk.TOP,fill=tk.X)

		b2 = tk.Radiobutton(frame, text=Plot.SCATTER,
				variable=self.source_type, value=Plot.SCATTER,
				command=self.typeChange, anchor=tk.W)
		tkExtra.Balloon.set(b2, "Draw a 2D scatter plot")
		b2.pack(fill=tk.X)
		self.widget["type"]   = b1,b2
		self.variable["type"] = self.source_type

		# ------- X ---------
		frame = tk.LabelFrame(pageframe, text="X ")
		frame.pack(side=tk.LEFT, expand=tk.YES, fill=tk.BOTH)

		row, col = 0, 0
		tk.Label(frame, text="Plot:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W, pady=0)
		col += 1
		s = tkExtra.Combobox(frame, width=10,
					command=self.histChange)
		s.fill(Plot.SOURCE_HIST)
		s.set(Plot.SOURCE_HIST[0])
		s.grid(row=row, column=col, columnspan=3, sticky=tk.EW)
		self.widget["x"] = s

		# ---
		row, col = row+1, 0
		tk.Label(frame, text="Min:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W, pady=0)
		col += 1
		self.hmin = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.hmin.grid(row=row, column=col, sticky=tk.EW)

		col += 1
		tk.Label(frame, text="Max:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W, pady=0)
		col += 1
		self.hmax = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.hmax.grid(row=row, column=col, sticky=tk.EW)

		# ----
		row, col = row+1, 0
		tk.Label(frame, text="Bins:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W, pady=0)
		col += 1
		self.bins = tkExtra.IntegerEntry(frame, background="White", width=10)
		tkExtra.Balloon.set(self.bins, "Number of bins to create histogram")
		self.bins.grid(row=row, column=col, sticky=tk.EW)
		self.widget["bins"] = self.bins

		# ----
		col += 1
		var = tk.IntVar()
		var.set(1)
		b = tk.Checkbutton(frame, text="Weighted", variable=var)
		tkExtra.Balloon.set(b, "Weight histogram with particle weight")
		b.grid(row=row, column=col, columnspan=2, sticky=tk.W)
		self.widget["weighted"] = b
		self.variable["weighted"] = var

		# Widgets to enable/disable
		self.x_widgets = [self.bins, b]

		frame.grid_columnconfigure(1,weight=1)
		frame.grid_columnconfigure(3,weight=1)

		# -------- Y --------
		frame = tk.LabelFrame(pageframe, text="Y ")
		frame.pack(side=tk.LEFT, expand=tk.YES, fill=tk.BOTH)

		row, col = 0, 0
		tk.Label(frame, text="Plot:", anchor=tk.W).grid(row=row,
				column=col, columnspan=3, sticky=tk.W, pady=0)
		col += 1
		s = tkExtra.Combobox(frame, width=10,
					command=self.histChange)
		s.fill(Plot.SOURCE_HIST)
		s.set(Plot.SOURCE_HIST[0])
		s.grid(row=row, column=col, columnspan=3, sticky=tk.EW)
		self.widget["y"] = s

		# Widgets to enable/disable
		self.y_widgets = [s]

		# ---
		row,col = row+1, 0
		tk.Label(frame, text="Min:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W, pady=0)
		col += 1
		self.h2min = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.h2min.grid(row=row, column=col, sticky=tk.EW)

		col += 1
		tk.Label(frame, text="Max:", anchor=tk.W).grid(row=row,
				column=col, sticky=tk.W, pady=0)
		col += 1
		self.h2max = tk.Label(frame, anchor=tk.W,
				foreground=tkFlair._ILABEL_FOREGROUND_COLOR,
				background=tkFlair._ILABEL_BACKGROUND_COLOR)
		self.h2max.grid(row=row, column=col, sticky=tk.EW)

		frame.grid_columnconfigure(1,weight=1)
		frame.grid_columnconfigure(3,weight=1)

		self.typeChange()

		# --- Particle List ---
		for p in FlukaParticle.list:
			if p != "": self.particle_list.insert(tk.END, p)
		self._datafile = None

		# ============= Continuous =============
		# FIXME add other plot types (do I need, or I do everything in GeometryEditor)
#		pageframe = tk.Frame(self.tabPages)
#		self.tabPages.add(pageframe,
#				text = "Continuous"
#				image =
#				compound=
#				)

		# ============= Point =============
		# FIXME add other plot types (do I need, or I do everything in GeometryEditor)
#		pageframe = tk.Frame(self.tabPages)
#		self.tabPages.add(pageframe,
#				text = "Point"
#				image =
#				compound=
#				)

		self.bindToggle()

	# ----------------------------------------------------------------------
	def type(self): return "USERDUMP"

	# ----------------------------------------------------------------------
	# Show Frame
	# ----------------------------------------------------------------------
	def set(self, plot):
		self.fillGeometry()
		super().set(plot)
		if self.plot is None: return

		try:
			case = Plot.USERDUMP_CASE.index(self.plot("case"))
		except:
			case = 2
		self.tabPages.select(case)
		name = self.project.name()
		self.datafile.initialdir = self.project.dir()
		self.datafile.setFiletypes([
					(f"{name} USERDUMP files",f"{name}*dump*"),
					("USERDUMP files","*dump*"),
					("All","*") ])
		self._datafile = self.plot("datafile")

		self.particle_list.selection_clear(0,tk.END)
		for p in self.plot("particles").split():
			self.particle_list.selection_set(
				FlukaParticle.list.index(p)-1)

		# Try to read file?
		self.mgfile  = ""
		self.limfile = ""
		self.dmin    = None
		self.dmax    = None
		self.fillGeometry()
		self.typeChange()

	# ----------------------------------------------------------------------
	def get(self, plot):
		super().get(plot)
		if plot is None: return

		# Enable all fields
		tkExtra.multiConfig(self.x_widgets, state=tk.NORMAL)
		tkExtra.multiConfig(self.y_widgets, state=tk.NORMAL)

		partlist = [self.particle_list.get(x)
				for x in self.particle_list.curselection()]
		plot["particles"] = " ".join(partlist)
		plot["case"]      = Plot.USERDUMP_CASE[self.tabPages.index(tk.CURRENT)]

		self.typeChange()

	# ----------------------------------------------------------------------
	def typeChange(self):
		if self.source_type.get() == Plot.HISTOGRAM:
			tkExtra.multiConfig(self.x_widgets, state=tk.NORMAL)
			tkExtra.multiConfig(self.y_widgets, state=tk.DISABLED)
		else:
			tkExtra.multiConfig(self.x_widgets, state=tk.DISABLED)
			tkExtra.multiConfig(self.y_widgets, state=tk.NORMAL)

	# ----------------------------------------------------------------------
	def histChange(self):
		if self.dmin:
			col = Plot.SOURCE_HIST.index(self.widget["x"].get())
			self.hmin.config(text="%10g"%(self.dmin[col]))
			self.hmax.config(text="%10g"%(self.dmax[col]))

			col = Plot.SOURCE_HIST.index(self.widget["y"].get())
			self.h2min.config(text="%10g"%(self.dmin[col]))
			self.h2max.config(text="%10g"%(self.dmax[col]))

	# ----------------------------------------------------------------------
	def fillGeometry(self):
		self.geo.clear()
		self.geo.insert(tk.END, Plot.GEO_NO)

		for plot in self.project.plots():
			if plot.type == "Geometry":
				self.geo.insert(tk.END, plot.name)

	# ----------------------------------------------------------------------
	def load(self, event=None):
		filename = self.datafile.filename()
		if filename:
			self._datafile = self.project.relpath(filename)
			self.datafile.delete(0,tk.END)
			self.datafile.insert(0,self._datafile)

	# ----------------------------------------------------------------------
	def help(self):
		Manual.show("","Userdump Plot")
