#!/bin/env python3
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   31-Oct-2006

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import io
import pickle
import tkinter as tk
from tkinter import ttk

import tool
import Data
import Plot
import icons
import Manual
import tkFlair
import tkExtra

import BasePlot
import PlotEngine

_X_OPTION_DEF = 2
_Y_OPTION_DEF = 0

_EMPTY = "<no name>"

#===============================================================================
# Residual Nuclei Plot Frame
#===============================================================================
class Usr1DPlotFrame(BasePlot.BasePlotFrame):
	def __init__(self, master, flair):
		super().__init__(master, flair)

		# === Detectors ===
		frame = tk.LabelFrame(self, text="Detectors ")
		frame.pack(side=tk.LEFT, expand=tk.YES, fill=tk.BOTH, padx=3)

		row, col = 0,0
		rowspan  = 7
		self.listbox = tkExtra.ExListbox(frame, borderwidth=0,
				selectmode=tk.EXTENDED,
				selectborderwidth=0,
				background="White",
				takefocus=True,
				width=15,
				height=10,
				exportselection=tk.FALSE)
		self.listbox.grid(row=row, rowspan=rowspan, column=col, sticky=tk.NSEW)
		self.widget["listbox"] = self.listbox

		col += 1
		sbv = ttk.Scrollbar(frame, orient=tk.VERTICAL, takefocus=False,
				command=self.listbox.yview)
		sbv.grid(row=row, rowspan=rowspan, column=col, sticky=tk.NS)
		self.listbox.config(yscrollcommand=sbv.set)

		col += 1
		b = tk.Button(frame, image=icons.get("rename"), takefocus=False,
				command=self.rename)
		b.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(b, "Rename detector")
		self.widget["rename"] = b

		row += 1
		b = tk.Button(frame, image=icons.get("add"), takefocus=False,
				command=self.add)
		b.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(b, "Add detector")
		self.widget["add"] = b

		row += 1
		b = tk.Button(frame, image=icons.get("del"), takefocus=False,
				command=self.delete)
		b.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(b, "Delete detector")
		self.widget["del"] = b

		row += 1
		b = tk.Button(frame, image=icons.get("clone"), takefocus=False,
				command=self.clone)
		b.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(b, "Clone detector")
		self.widget["clone"] = b

		row += 1
		b = tk.Button(frame, image=icons.get("up"), takefocus=False,
				command=self.moveUp)
		tkExtra.Balloon.set(b, "Move Up")
		b.grid(row=row, column=col, sticky=tk.EW)
		self.widget["up"] = b

		row += 1
		b = tk.Button(frame, image=icons.get("down"), takefocus=False,
				command=self.moveDown)
		tkExtra.Balloon.set(b, "Move Down")
		b.grid(row=row, column=col, sticky=tk.EW)
		self.widget["down"] = b

		frame.grid_columnconfigure(0, weight=1)
		frame.grid_rowconfigure(row+1, weight=1)

		# === Detectors ===
		labelframe = tk.LabelFrame(self, text="Detector Info ")
		labelframe.pack(side=tk.LEFT, expand=tk.YES, fill=tk.X, padx=3)

		# ---
		frame = tk.Frame(labelframe)
		frame.pack(side=tk.TOP, fill=tk.X)

		tk.Label(frame, text="File:", anchor=tk.E).pack(side=tk.LEFT)
		self.datafile = tkExtra.FileEntry(frame, width=32,
					title="Load USRxxx file",
					initialdir=self.project.dir(),
					multiple=True,
					filetypes=[
						("All Data files",("*_tab.lis","*.hist","*.dat")),
						("Data files"            , "*.dat"),
						("Histogram files"       , "*.hist")               ,
						("Fluka _tab.lis files"  , "*_tab.lis"),
						("All","*")],
					foreground="DarkBlue", background="White")
		self.datafile.bind("<<FileChanged>>", self.load)
		self.datafile.pack(side=tk.LEFT, expand=tk.YES, fill=tk.X)
		tkExtra.Balloon.set(self.datafile, "Histogram data file")
		self.widget["@datafile"] = self.datafile

		# ---
		tk.Label(frame, text=" Det:").pack(side=tk.LEFT)
		self.det = tkExtra.Combobox(frame, command=self.detChange, width=24)
		self.det.pack(side=tk.LEFT, fill=tk.X, expand=tk.YES)
		tkExtra.Balloon.set(self.det, "Select detector from the data file to plot")
		self.widget["@det"] = self.det

		# ====================================
		frame = tk.Frame(labelframe)
		frame.pack(fill=tk.X)

		# ====================================
		lf = tk.LabelFrame(frame, text="Show ")
		lf.pack(side=tk.LEFT)

		self.show_plot = tk.IntVar()
		self.show_plot.set(1)
		self.show_plot_button = tk.Checkbutton(lf, text="graph",
				variable=self.show_plot, anchor=tk.W, justify=tk.RIGHT)
		self.show_plot_button.pack(side=tk.TOP, fill=tk.X)
		tkExtra.Balloon.set(self.show_plot_button,
				"Include detector/graph in plot")
		self.widget["@show_plot"] = self.show_plot_button

		# ---
		self.show_key = tk.IntVar()
		self.show_key.set(1)
		self.show_key_button = tk.Checkbutton(lf, text="legend",
				variable=self.show_key, anchor=tk.W, justify=tk.RIGHT)
		self.show_key_button.pack(side=tk.TOP, fill=tk.X)
		tkExtra.Balloon.set(self.show_key_button,
				"Show key/legend of detector")
		self.widget["@show_key"] = self.show_key_button

		# ---
		lf = tk.LabelFrame(frame, text="Plot ")
		lf.pack(side=tk.RIGHT, expand=tk.YES, fill=tk.X)

		# ===
		row,col = 0,0
		tk.Label(lf, text="Type:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.plot_with = tkExtra.Combobox(lf, width=10)
		self.plot_with.fill(Plot.STYLE)
		self.plot_with.set(Plot.STYLE[Plot.STYLE_DEF])
		self.plot_with.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.plot_with, "Plotting style")
		self.widget["@plot_with"] = self.plot_with

		# ---
		col += 1
		tk.Label(lf, text=" X:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.x_norm = tkExtra.Combobox(lf, label=False, width=10, background="White")
		self.x_norm.fill(Plot.X_SCALE)
		self.x_norm.set(Plot.X_SCALE[0])
		self.x_norm.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.x_norm,
				"X axis, normalization factor or expression (eg. 3*$1-10)")
		self.widget["@x_norm"] = self.x_norm

		# ===
		row,col = row+1,0

		tk.Label(lf, text="Value:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.plot_value = tkExtra.Combobox(lf, width=10)
		self.plot_value.fill(Plot.Y_VALUE)
		self.plot_value.set(Plot.Y_VALUE[0])
		self.plot_value.grid(row=row, column=col, sticky=tk.EW)
		tkExtra.Balloon.set(self.plot_value, "Plotting Y value")
		self.widget["@plot_value"] = self.plot_value

		# ---
		col += 1
		tk.Label(lf, text=" Y:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.norm = tk.Entry(lf, background="White", justify=tk.LEFT)
		tkExtra.Balloon.set(self.norm,
				"Y axis, normalization factor or expression (eg. $2**2+10)")
		self.norm.grid(row=row, column=col, sticky=tk.EW)
		self.widget["@norm"] = self.norm

		lf.grid_columnconfigure(3, weight=1)

		# ====================================
		frame = tk.Frame(labelframe)
		frame.pack(fill=tk.X)

		# ====================================
		lf = tk.LabelFrame(frame, text="Options ")
		lf.pack(side=tk.LEFT)

		row,col = 0,0
		tk.Label(lf, text="Color:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.plot_lc = tkExtra.Combobox(lf, width=15)
		self.plot_lc.fill(PlotEngine._COLOR_NAMES)
		self.plot_lc.grid(row=row,column=col,sticky=tk.EW)
		tkExtra.Balloon.set(self.plot_lc, "Line color if gnuplot>=4.2")
		self.widget["@plot_lc"] = self.plot_lc

		# ---
		col += 1
		tk.Label(lf, text="Line width:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.plot_lw = tk.IntVar()
		self.plot_lw.set(1)
		self.plot_lw_spin = tk.Spinbox(lf, text=self.plot_lw, from_=1, to=6,
				background="White",width=3)
		self.plot_lw_spin.grid(row=row, column=col, sticky=tk.W)
		tkExtra.Balloon.set(self.plot_lw_spin, "Line width")
		self.widget["@plot_lw"] = self.plot_lw_spin

		# ---
		col += 1
		tk.Label(lf, text="Dash type:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.plot_dt = tk.IntVar()
		self.plot_dt.set(0)
		self.plot_dt_spin = tk.Spinbox(lf, text=self.plot_dt, from_=0, to=6,
				background="White",width=3)
		self.plot_dt_spin.grid(row=row, column=col, sticky=tk.W)
		tkExtra.Balloon.set(self.plot_dt_spin, "Dash type")
		self.widget["@plot_dt"] = self.plot_dt_spin

		# ===
		row, col = row+1, 0
		tk.Label(lf, text=" Point type:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.plot_pt = tkExtra.Combobox(lf, width=15)
		self.plot_pt.fill(PlotEngine.POINTS)
		self.plot_pt.grid(row=row, column=col, sticky=tk.W)
		tkExtra.Balloon.set(self.plot_pt, "Point type to use")
		self.widget["@plot_pt"] = self.plot_pt

		# ---
		col += 1
		tk.Label(lf, text="Point size:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.plot_ps = tk.IntVar()
		self.plot_ps.set(1)
		self.plot_ps_spin = tk.Spinbox(lf, text=self.plot_ps, from_=1, to=6,
				background="White",width=3)
		self.plot_ps_spin.grid(row=row, column=col, sticky=tk.W)
		tkExtra.Balloon.set(self.plot_ps_spin, "Point size")
		self.widget["@plot_ps"] = self.plot_ps_spin

		# ---
		col += 1
		tk.Label(lf, text="Axes:").grid(row=row, column=col, sticky=tk.E)
		col += 1
		self.axes = tkExtra.Combobox(lf, width=5)
		self.axes.fill(PlotEngine.AXES)
		self.axes.grid(row=row,column=col,sticky=tk.EW)
		tkExtra.Balloon.set(self.plot_lc, "Plotting axes to be used")
		self.widget["@axes"] = self.axes

		# ---- bindings -----
		self.listbox.bind('<Return>',		self.rename)
		self.listbox.bind('<KP_Enter>',		self.rename)
		self.listbox.bind('<<Rename>>',		self.rename)
		self.listbox.bind('<<Cut>>',		self.cut)
		self.listbox.bind('<<Copy>>',		self.copy)
		self.listbox.bind('<<Clone>>',		self.clone)
		self.listbox.bind('<<Paste>>',		self.paste)
		self.listbox.bind('<Delete>',		self.delete)
		self.listbox.bind('<Double-1>',		self.rename)
		self.listbox.bind('<<ListboxSelect>>',	self.selectDetector)
		self.listbox.bind('<<MoveUp>>',		self.moveUp)
		self.listbox.bind('<<MoveDown>>',	self.moveDown)

		# ---- variables -----
		self.current = None		# Current detector
		self.selectDetector()
		self._datafile = None		# Remember last data file shown
		self._datatime = 0		# -//- time stamp of file
		self.bindToggle()

	# ----------------------------------------------------------------------
	def type(self): return "USR-1D"

	# ----------------------------------------------------------------------
	# Show Frame
	# ----------------------------------------------------------------------
	def set(self, plot):
		super().set(plot)
		self.current = None
		# get current selection and reselect it by names
		selection = {self.listbox.get(i) for i in self.listbox.curselection()}
		self.clearInfo()
		self.listbox.delete(0,tk.END)
		name = self.project.name()
		self.datafile.initialdir = self.project.dir()
		self.datafile.setFiletypes( [ ("All Data files",("*_tab.lis","*.hist","*.dat")),
						("Data files"            , "*.dat"),
						("Histogram files"       , "*.hist")               ,
						("Fluka _tab.lis files"  , "*_tab.lis"),
						(f"{name} .dat files"    , f"{name}*.dat"),
						(f"{name} .hist files"   , f"{name}*.hist"),
						(f"{name} _tab.lis files", f"{name}*_tab.lis"),
						("All","*")])
		for i in range(int(plot("ndetectors"))):
			name = str(plot(f"name.{i}"))
			if not name: name = _EMPTY
			self.listbox.insert(tk.END, name)
			if name in selection:
				self.listbox.selection_set(i)
			if name == _EMPTY:
				self.listbox.itemconfig(tk.END, fg="Blue")

		if not self.listbox.curselection():
			self.listbox.selection_set(0)
		self.setState(tk.NORMAL)
		self.fillDetectorInfo()

	# ----------------------------------------------------------------------
	# WARNING:
	#    "state" can be "active" when mouse cursor is on
	#                   "normal"
	#                   "disabled"
	#    do not check for =="normal" but for != "disabled"
	# ----------------------------------------------------------------------
	def get(self, plot):
		super().get(plot)
		if not self.current: return
		if self.listbox["state"] != tk.DISABLED:
			plot["ndetectors"] = str(self.listbox.size())
		active = self.listbox.index(tk.ACTIVE)
		for cur in self.current:
			detname = str(self.listbox.get(cur))
			if detname == _EMPTY: detname = ""
			if self.listbox["state"] != tk.DISABLED:
				plot[f"name.{cur}"] = detname

			if self.datafile["state"] != tk.DISABLED:
				files = self.datafile.files()
				if files:
					plot[f"file.{cur}"] = self.project.relpath(files[0])

			if self.det._text["state"] != tk.DISABLED:
				det,block,name = self.detector()
				plot[f"det.{cur}"]   = str(det)
				plot[f"block.{cur}"] = str(block)
				newname = f"#{name}"
				if (not detname or detname[0]=="#") and detname!=newname:
					plot[f"name.{cur}"] = newname
					selected = self.listbox.selection_includes(cur)
					self.listbox.delete(cur)
					self.listbox.insert(cur,newname)
					if selected: self.listbox.selection_set(cur)
					if active==cur: self.listbox.activate(active)

			if self.show_plot_button["state"] != tk.DISABLED:
				plot[f"show.{cur}"] = str(self.show_plot.get())

			if self.show_key_button["state"] != tk.DISABLED:
				plot[f"key.{cur}"]  = str(self.show_key.get())

			if self.norm["state"] != tk.DISABLED:
				plot[f"norm.{cur}"] = self.norm.get()

			if self.plot_with["state"] != tk.DISABLED:
				plot[f"with.{cur}"]  = self.plot_with.get()

			if self.plot_value["state"] != tk.DISABLED:
				plot[f"y.{cur}"] = Plot.Y_VALUE.index(self.plot_value.get())

			if self.x_norm["state"] != tk.DISABLED:
				plot[f"xnorm.{cur}"] = self.x_norm.get()

			if self.plot_lc["state"] != tk.DISABLED:
				plot[f"lc.{cur}"] = self.plot_lc.get()

			if self.axes["state"] != tk.DISABLED:
				plot[f"axes.{cur}"] = self.axes.get()

			if self.plot_pt["state"] != tk.DISABLED:
				try: idx = PlotEngine.POINTS.index(self.plot_pt.get())
				except: idx = 0
				plot[f"pt.{cur}"] = idx

			if self.plot_lw_spin["state"] != tk.DISABLED:
				plot[f"lw.{cur}"] = str(self.plot_lw.get())

			if self.plot_dt_spin["state"] != tk.DISABLED:
				plot[f"dt.{cur}"] = str(self.plot_dt.get())

			if self.plot_ps_spin["state"] != tk.DISABLED:
				plot[f"ps.{cur}"] = str(self.plot_ps.get())

	# ----------------------------------------------------------------------
	def detChange(self):
		if self.current is None: return
		active = self.listbox.index(tk.ACTIVE)
		for cur in self.current:
			detname = self.listbox.get(cur)
			if detname and detname[0]!="#": continue
			det,block,name = self.detector()
			if not name: return
			newname = f"#{name}"
			if detname != newname:
				selected = self.listbox.selection_includes(cur)
				self.plot[f"name.{cur}"] = newname
				self.listbox.delete(cur)
				self.listbox.insert(cur,newname)
				if selected: self.listbox.selection_set(cur)
				if active == cur: self.listbox.activate(active)

	# ----------------------------------------------------------------------
	def selectDetector(self, event=None):
		self.get(self.plot)
		self.setState(tk.NORMAL)
		self.fillDetectorInfo()

	# ----------------------------------------------------------------------
	def setState(self, state):
		super().setState(state)
		self.current = list(map(int, self.listbox.curselection()))
		if len(self.current)!=1: state = tk.DISABLED

		self.datafile.config(state=state)
		self.det.config(state=state)
		self.show_plot_button.config(state=state)
		self.show_key_button.config(state=state)
		self.plot_with.config(state=state)
		self.plot_value.config(state=state)
		self.norm.config(state=state)
		self.x_norm.config(state=state)
		self.plot_lc.config(state=state)
		self.axes.config(state=state)
		self.plot_pt.config(state=state)
		self.plot_lw_spin.config(state=state)
		self.plot_dt_spin.config(state=state)
		self.plot_ps_spin.config(state=state)

	# ----------------------------------------------------------------------
	def fillDetectorInfo(self):
		if not self.current: return

		cur = self.current[0]
		if len(self.current)>1:
			# initial values
			idatafile = self.plot(f"file.{cur}")
			for cur in self.current:
				if idatafile != self.plot(f"file.{cur}"):
					# Fill detector only with numbers!
					for i in range(1,100):
						self.det.insert(tk.END, f"{i} ?")
					break
			else:
				self.loadDataFile(idatafile)

		else:
			datafile = self.plot(f"file.{cur}")
			if datafile:
				self.loadDataFile(datafile)
			else:
				self.clearInfo()

			self.show_plot.set(int(self.plot(f"show.{cur}")))
			self.show_key.set(int(self.plot(f"key.{cur}")))

			style = self.plot(f"with.{cur}")
			# Switch to histogram if not existing
			if style not in Plot.STYLE: style = "histogram"
			self.plot_with.set(style)

			try:
				yval = int(self.plot[f"y.{cur}"])
				if yval > len(Plot.Y_VALUE): yval = 0
			except:
				yval = 0
			self.plot_value.set(Plot.Y_VALUE[yval])

			self.norm.delete(0,tk.END)
			self.norm.insert(0,self.plot(f"norm.{cur}"))

			self.x_norm.set(self.plot(f"xnorm.{cur}"))

			self.plot_lc.set(self.plot(f"lc.{cur}"))
			self.axes.set(self.plot(f"axes.{cur}"))
			try:
				pt = int(self.plot(f"pt.{cur}")) % 14
			except:
				pt = 0
			self.plot_pt.set(PlotEngine.POINTS[pt])

			try:
				self.plot_lw.set(int(self.plot(f"lw.{cur}")))
			except:
				self.plot_lw.set(1)
			try:
				self.plot_dt.set(int(self.plot(f"dt.{cur}")))
			except:
				self.plot_dt.set(1)
			try:
				self.plot_ps.set(int(self.plot(f"ps.{cur}")))
			except:
				self.plot_ps.set(1)

		self.detector(int(self.plot(f"det.{cur}")),
			      int(self.plot(f"block.{cur}")))

	# ----------------------------------------------------------------------
	def clearInfo(self):
		self._datafile = None
		self._datatime = 0
		self.datafile.delete(0,tk.END)
		self.det.clear()

	# ----------------------------------------------------------------------
	def loadDataFile(self, datafile):
		if not datafile:
			self.clearInfo()
			return
		if self._datafile==datafile and \
		   self._datatime==tool.getmtime(datafile): return
		self._datafile = datafile
		self._datatime = tool.getmtime(datafile)
		self.datafile.set(datafile)

		self.det.clear()
		try:
			filefmt = Data.detectFormat(datafile)
			if filefmt == "phits":
				tab = Data.PhitsData(datafile, readdata=False)
			#elif filefmt == "tablis":
			else:
				tab = Data.Tablis(datafile, readdata=False)

			for det in tab.detector:
				if det.block is None:
					self.det.insert(tk.END, f"{det.num} {det.name}")
				else:
					self.det.insert(tk.END, f"{det.num}-{det.block+1} {det.name}")
			self.det.set(self.det.get(0), False)
		except Exception as ex:
			self.flair.notify("File error",
				f"Reading file {datafile!r}\n{ex}",
				tkFlair.NOTIFY_ERROR)

	# ----------------------------------------------------------------------
	def load(self, event=None):
		if self.datafile["state"] != tk.NORMAL: return
		files = self.datafile.files()
		if files:
			self.loadDataFile(self.project.relpath(files[0]))
			for fn in files[1:]:
				# clone the current detector and load the new file
				relfn = self.project.relpath(fn)
				self.clone()
				self.loadDataFile(relfn)

	# ----------------------------------------------------------------------
	# Set/Get detector=d/block=b
	# ----------------------------------------------------------------------
	def detector(self, d=None, b=None):
		if d is None:
			d = b = 0
			m = Data._detName.match(self.det.get())
			if m:
				db   = m.group(1)			# det-block
				name = m.group(2)
				if "-" in db:
					d,b = map(int,db.split("-"))	# det,block
				else:
					d = int(db)
				d -= 1				# convert to 0 based
				return d,b,name
			else:
				return 0,0,self.det.get()
		else:
			self.det.set("", False)
			if d < 0: return
			d += 1					# convert to 1 based

			for item in self.det.get(0,tk.END):
				db = item.split()[0]
				di = bi = 0
				if "-" in db:
					di,bi = map(int,db.split("-"))
				else:
					di = int(db)
				if d==di and b==bi:
					self.det.set(item, False)

	# ----------------------------------------------------------------------
	# Return active block of the detector
	# ----------------------------------------------------------------------
	def block(self):
		db = self.det.get()
		if "-" in db:
			return int(db.split("-")[1])
		else:
			return 0

	# ----------------------------------------------------------------------
	def add(self):
		self.get(self.plot)
		ndets = int(self.plot("ndetectors"))
		self.plot["ndetectors"] = ndets+1
		name = f"#Detector {ndets+1}"
		self.plot[f"name.{ndets}"] = name
		self.plot[f"det.{ndets}"] = 0
		self.listbox.insert(tk.END,name)
		self.listbox.selection_clear(0,tk.END)
		self.listbox.selection_set(tk.END)
		self.listbox.activate(tk.END)
		self.listbox.see(tk.END)
		self.current = None
		self.selectDetector()
		self.load()

	# ----------------------------------------------------------------------
	def rename(self, event=None):
		active = self.listbox.index(tk.ACTIVE)
		edit = tkExtra.InPlaceEdit(self.listbox)
		if edit and edit.value=="":
			self.listbox.set(active, _EMPTY)
			self.listbox.itemconfig(active, foreground="Blue")

	# ----------------------------------------------------------------------
	# find all vars
	# create dynamically the list of variables controlling the detector
	# ----------------------------------------------------------------------
	def _findDetectorVariables(self, det):
		lst = []
		for k in self.plot:
			try:
				ks = k.split(".")
				if int(ks[1])==det:
					lst.append(ks[0])
			except:
				pass
		return lst

	# ----------------------------------------------------------------------
	def _moveDetectorVariables(self, src, dst):
		lst = self._findDetectorVariables(src)
		# Make a copy of all variables, and delete the original
		for k in lst:
			vsrc = f"{k}.{src}"
			self.plot[f"{k}.{dst}"] = self.plot[vsrc]
			del self.plot[vsrc]

	# ----------------------------------------------------------------------
	# Delete list
	# ----------------------------------------------------------------------
	def delete(self, event=None):
		# protect against accidental delete
		if self.focus_get() is not self.listbox: return

		self.get(self.plot)
		self.current = None
		sel = list(map(int, self.listbox.curselection()))
		sel.reverse()
		ndets = int(self.plot("ndetectors"))

		for det in sel:
			for k in self._findDetectorVariables(det):
				del self.plot[f"{k}.{det}"]
			if det < self.listbox.size()-1:
				for d in range(det+1,ndets):
					self._moveDetectorVariables(d, d-1)
			ndets -= 1
			self.listbox.delete(det)

		self.plot["ndetectors"] = ndets
		self.selectDetector()
		return "break"

	# ----------------------------------------------------------------------
	# Copy detector in clipboard
	# ----------------------------------------------------------------------
	def copy(self, event=None):
		if self.focus_get() is not self.listbox: return
		self.get(self.plot)

		clip = {}
		ndets = 0
		for det in map(int, self.listbox.curselection()):
			for k in self._findDetectorVariables(det):
				clip[f"{k}.{ndets}"] = self.plot(f"{k}.{det}")
			ndets += 1
		clip["@n"] = ndets

		# Write to clipboard
		bio = io.BytesIO()
		pickler = pickle.Pickler(bio)
		pickler.dump(clip)
		self.clipboard_clear()
		self.flair.clipboardAppend(bio, tkFlair._USR1D_MIME)
		return "break"

	# ----------------------------------------------------------------------
	def cut(self, event=None):
		self.copy()
		self.delete()
		return "break"

	# ----------------------------------------------------------------------
	def paste(self, event=None):
		if self.focus_get() is not self.listbox: return
		self.get(self.plot)
		self.current = None

		if tkFlair._USR1D_MIME in self.flair.clipboardTargets():
			unpickler = self.flair.clipboardUnpickler(tkFlair._USR1D_MIME)
			try:
				clip = unpickler.load()
			except:
				self.flair.notify("Cannot paste",
						"Invalid content in clipboard",
						tkFlair.NOTIFY_WARNING)
				return
		else:
			self.flair.notify("Cannot paste",
					"Invalid content in clipboard",
					tkFlair.NOTIFY_WARNING)
			return

		self.listbox.selection_clear(0,tk.END)
		ndets = int(self.plot("ndetectors"))

		for i in range(clip.get("@n",0)):
			for k,v in clip.items():
				try:
					ks = k.split(".")
					if int(ks[1]) == i:
						self.plot[f"{ks[0]}.{ndets}"] = v
				except:
					pass
			self.listbox.insert(tk.END, str(self.plot(f"name.{ndets}")))
			self.listbox.selection_set(tk.END)
			self.listbox.activate(tk.END)
			self.listbox.see(tk.END)
			ndets += 1

		self.plot["ndetectors"] = ndets
		self.selectDetector()
		return "break"

	# ----------------------------------------------------------------------
	# Clone detector
	# ----------------------------------------------------------------------
	def clone(self, event=None):
		sel = list(map(int, self.listbox.curselection()))
		self.get(self.plot)
		self.current = None
		sel = list(map(int, self.listbox.curselection()))
		self.listbox.selection_clear(0,tk.END)
		ndets = int(self.plot("ndetectors"))
		for det in sel:
			for k in self._findDetectorVariables(det):
				self.plot[f"{k}.{ndets}"] = self.plot(f"{k}.{det}")
			self.listbox.insert(tk.END, str(self.plot(f"name.{ndets}")))
			self.listbox.selection_set(tk.END)
			self.listbox.activate(tk.END)
			self.listbox.see(tk.END)
			ndets += 1

		self.plot["ndetectors"] = ndets
		self.selectDetector()
		return "break"

	# ----------------------------------------------------------------------
	# Swap two items in the list
	# ----------------------------------------------------------------------
	def swap(self, a, b):
		self.listbox.swap(a,b)

		self._moveDetectorVariables(a, 999)
		self._moveDetectorVariables(b,   a)
		self._moveDetectorVariables(999, b)

	# ----------------------------------------------------------------------
	# Move up select items by one
	# ----------------------------------------------------------------------
	def moveUp(self, event=None):
		self.get(self.plot)
		self.current = None
		for i in map(int,self.listbox.curselection()):
			if i==0: continue
			prev = i-1
			if not self.listbox.selection_includes(prev):
				act = self.listbox.index(tk.ACTIVE)
				self.swap(prev,i)
				self.listbox.selection_set(prev)
				self.listbox.see(prev)
				if act == i: self.listbox.activate(prev)
			self.selectDetector()
		return "break"

	# ----------------------------------------------------------------------
	# Move down select items by one
	# ----------------------------------------------------------------------
	def moveDown(self, event=None):
		self.get(self.plot)
		self.current = None
		sz  = self.listbox.size()-1
		lst = list(map(int,self.listbox.curselection()))
		lst.reverse()
		for i in lst:
			if i >= sz: continue
			next_ = i+1
			if not self.listbox.selection_includes(next_):
				act = self.listbox.index(tk.ACTIVE)
				self.swap(i,next_)
				self.listbox.selection_set(next_)
				self.listbox.see(next_)
				if act == i: self.listbox.activate(next_)
			self.selectDetector()
		return "break"

	# ----------------------------------------------------------------------
	def help(self):
		Manual.show("","1D Plot")
