# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   8-Feb-2019

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

__all__ = [ "input", "NotSupported" ]

from .convert import conversionClass, NotSupported
from .fluka2moira import Fluka2Moira	# needed to register
from .fluka2mcnp  import Fluka2Mcnp	#       -//-
from .mcnp2fluka  import Mcnp2Fluka

#-------------------------------------------------------------------------------
# input conversion method that converts one type to another, based
# on the registered converters
#-------------------------------------------------------------------------------
def input(inp, mode, defines=None, **kwargs):
	if mode in ("",inp.mode):
		newinp = inp.clone()
		newinp.enableDefines(defines)
		return newinp
	class_ = conversionClass(inp.mode, mode)
	inp.preprocess(defines)
	return class_(inp).convert(**kwargs)

#-------------------------------------------------------------------------------
# @return option list for a possible mode
#-------------------------------------------------------------------------------
def options(inp, mode):
	return conversionClass(inp.mode, mode).options
