# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   24-Oct-2018

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

from log import error
from deck import Card

CONVERSION_CLASSES = {}

class NotSupported(Exception): pass

#-------------------------------------------------------------------------------
# Register conversion class
#-------------------------------------------------------------------------------
def register(src, dst, class_):
	global CONVERSION_CLASSES
	key = f"{src}:{dst}"
	CONVERSION_CLASSES[key] = class_

#-------------------------------------------------------------------------------
def conversionClass(src, dst):
	try:
		return CONVERSION_CLASSES[f"{src}:{dst}"]
	except KeyError:
		raise NotSupported(
			f"Input mode change {src} "
			f"\N{RIGHTWARDS ARROW} {dst} is not supported")

#===============================================================================
# Basic abstract class for conversion from one format to another
#===============================================================================
class Convert:
	src = ""
	dst = ""

	options = []		# user selectable options as in the ListboxDialog

	#-----------------------------------------------------------------------
	def __init__(self, inp):
		self.input  = inp
		self.output = None

	#-----------------------------------------------------------------------
	def convert(self, **kwargs):
		# Initialize card used, just in case
		for card in self.input.cardlist: card[Card.USED] = False
		return None

	#-----------------------------------------------------------------------
	def notImplemented(self, card, what, msg="", notimpl="not implemented"):
		if isinstance(what,int):
			error(card.tag,
				f"what[{what}]={card.what(what)}",
				notimpl,msg)
		else:
			error(card.tag,what,notimpl,msg)

	#-----------------------------------------------------------------------
	def used(self, card):
		card[Card.USED] = True

	#-----------------------------------------------------------------------
	def isused(self, card):
		return card[Card.USED]

	#-----------------------------------------------------------------------
	def add(self, card, enable=True):
		card.setEnable(enable)
		self.output.add(card)

	#-----------------------------------------------------------------------
	def addClone(self, card, disable=False):
		clone = card.clone()
		if disable: clone.setEnable(False)
		self.add(card)
