# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   23-Nov-2018

import os
import re

from deck import DeckConfig

#===============================================================================
# A container class to keep the programs
# It should contain only static and class methods
#===============================================================================
class FlukaConfig(DeckConfig):
	SECTION      = "Fluka"		# Configuration section name
	version      = "Unknown"
	released     = ""
	directory    = ""		# Executable installation directory
	user         = {}		# Description of user routines
	CYCLE        = "fluka_"		# prefix of temporary cycle directory
	# fluka programs, the name is the default program
	DEFAULT_PRG  = "rfluka"		# [flair.ini] Main application program/script
	DEFAULT_EXE  = "fluka"		# [flair.ini]
	DEFAULT_EXE2 = "flukadpm"
	DEFAULT_LIB  = "libfluka.a"
	EXT          = ".inp"		# default input extension
	EXTENSIONS   = [EXT,".fluka"]
	programs     = { "prg"        : "",
			 "exe"        : "",
			 "exe2"       : "",
			 "fff"        : "",
			 "detsuw"     : "",
			 "usbsuw"     : "",
			 "ustsuw"     : "",
			 "usxsuw"     : "",
			 "usbmax"     : "",
			 "usrsuw"     : "",
			 "usbrea"     : "",
			 "gplevbin"   : "",
			 "detect"     : "detsuw",
			 "resnucle"   : "usrsuw",
			 "usrbin"     : "usbsuw",
			 "usrbdx"     : "usxsuw",
			 "usrtrack"   : "ustsuw",
			 "usryield"   : "usysuw",
			 "usrcoll"    : "ustsuw",
		}
	VERSION_PAT   = re.compile(r".*^Version: (\d.*?)$.Date: *(.*?)$",
				re.MULTILINE | re.DOTALL)
	VERSION2_PAT  = re.compile(r".*^The latest version is.*?: (\d+..*?)$.*?: (.*?)$",
				re.MULTILINE | re.DOTALL)
	CYCLEDIR_PAT  = re.compile(rf"^{CYCLE}(\d+)$")
	CYCLEDIR2_PAT = re.compile(rf"^(.*/?){CYCLE}(\d+)$")

	#-----------------------------------------------------------------------
	# Find FLUKA version
	#-----------------------------------------------------------------------
	@staticmethod
	def findVersion(directory=None):
		if directory is None:
			d = FlukaConfig.directory
		else:
			d = directory
		try:
			with open(os.path.join(d,"Version.tag"),"r") as f:
				m = FlukaConfig.VERSION_PAT.match(f.read())
				version  = None
				released = None
				if m:
					version, released = m.groups()
				else:
					m = FlukaConfig.VERSION2_PAT.match(f.read())
					if m:
						version, released = m.groups()
				if version is not None:
					FlukaConfig.version = version
					FlukaConfig.released = released
			return version
		except IOError:
			return "Unknown"

	#-----------------------------------------------------------------------
	# Load from config
	#-----------------------------------------------------------------------
	@staticmethod
	def loadConfig(config):
		section  = FlukaConfig.SECTION
		FlukaConfig.setDir(config.getStr(section, "directory", ""))
		FlukaConfig.setCmd("prg",     config.getStr(section, "executable",FlukaConfig.DEFAULT_EXE))
		FlukaConfig.setCmd("exe",     config.getStr(section, "program",   FlukaConfig.DEFAULT_PRG))
		FlukaConfig.setCmd("fff",     config.getStr(section, "fff",     ""))
		FlukaConfig.setCmd("usbsuw",  config.getStr(section, "usbsuw",  ""))
		FlukaConfig.setCmd("ustsuw",  config.getStr(section, "ustsuw",  ""))
		FlukaConfig.setCmd("usxsuw",  config.getStr(section, "usxsuw",  ""))
		FlukaConfig.setCmd("usrsuw",  config.getStr(section, "usrsuw",  ""))
		FlukaConfig.setCmd("usysuw",  config.getStr(section, "usysuw",  ""))
		FlukaConfig.setCmd("detsuw",  config.getStr(section, "detsuw",  ""))
		FlukaConfig.setCmd("usbmax",  config.getStr(section, "usbmax",  ""))
		FlukaConfig.setCmd("usbrea",  config.getStr(section, "usbrea",  ""))
		FlukaConfig.setCmd("gplevbin",config.getStr(section, "gplevbin",""))
