# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   12-Dec-2014

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

#import ntuple
import histogram

_COLOR_NAMES = ["",
		"antiquewhite",
		"aquamarine",
		"beige",
		"bisque",
		"black",
		"blue",
		"brown",
		"chartreuse",
		"coral",
		"cyan",
		"dark-blue",
		"dark-cyan",
		"dark-goldenrod",
		"dark-gray",
		"dark-green",
		"dark-khaki",
		"dark-magenta",
		"dark-olivegreen",
		"dark-orange",
		"dark-red",
		"dark-salmon",
		"dark-spring-green",
		"dark-turquoise",
		"gold",
		"goldenrod",
		"gray",
		"green",
		"greenyellow",
		"honeydew",
		"khaki",
		"lemonchiffon",
		"light-blue",
		"light-coral",
		"light-cyan",
		"light-goldenrod",
		"light-gray",
		"light-green",
		"light-pink",
		"light-salmon",
		"magenta",
		"navy",
		"olive",
		"orange",
		"orange-red",
		"orchid",
		"pink",
		"plum",
		"purple",
		"red",
		"royalblue",
		"salmon",
		"sandybrown",
		"seagreen",
		"skyblue",
		"slategray",
		"steelblue",
		"turquoise",
		"violet",
		"white",
		"yellow"
]

POINTS = [	"dot",
		"+",
		"x",
		"*",
		"square",
		"square-filled",
		"circle",
		"circle-filled",
		"triangle-up",
		"triangle-up-filled",
		"triangle-down",
		"triangle-down-filled",
		"diamond",
		"diamond-filled"
	]

AXES = [	"",
		"x1y1",
		"x1y2",
		"x2y1",
		"x2y2"
	]

#===============================================================================
# Abstract Plot Engine class
#===============================================================================
class PlotEngine:
	_version = None

	"""
	Interface class to any plot engine
	"""
	def __init__(self):
		pass

	# ----------------------------------------------------------------------
	# Check if the process has ended and return the returncode else None
	# ----------------------------------------------------------------------
	def poll(self):
		return None

	# ----------------------------------------------------------------------
	def close(self):
		pass

	# ----------------------------------------------------------------------
	def reset(self):
		pass

	# ----------------------------------------------------------------------
	def quit(self):
		pass

	# ----------------------------------------------------------------------
	def title(self, title, options=""):
		pass

	# ----------------------------------------------------------------------
	def label(self, axis, label, options=""):
		pass

	# ----------------------------------------------------------------------
	def xlabel(self, label, opts=""):  self.label("x", label,opts)
	def ylabel(self, label, opts=""):  self.label("y", label,opts)
	def zlabel(self, label, opts=""):  self.label("z", label,opts)
	def cblabel(self, label, opts=""): self.label("cb",label,opts)

	# ----------------------------------------------------------------------
	def grid(self, on=True):
		pass

	# ----------------------------------------------------------------------
	def aspect(self, ratio):
		pass

	# ----------------------------------------------------------------------
	def tics(self, axis, options=""):
		pass

	# ----------------------------------------------------------------------
	def range(self, axis, amin="", amax=""): pass
	def xrange(self,  amin="", amax=""): self.range("x",  amin, amax)
	def yrange(self,  amin="", amax=""): self.range("y",  amin, amax)
	def zrange(self,  amin="", amax=""): self.range("z",  amin, amax)
	def cbrange(self, amin="", amax=""): self.range("cb", amin, amax)

	# ----------------------------------------------------------------------
	def plot(self, cmd, data=None):
		pass

	# ----------------------------------------------------------------------
	def splot(self, cmd, data=None):
		pass

	# ----------------------------------------------------------------------
	def draw(self, data, options=None):
		"""Generic drawing of data with options"""
		if isinstance(data, (histogram.H1, data,histogram.H2)):
			data.draw(self, options)
		else:
			raise Exception(f"Don't know how to handle datatype {type(data)}")

	# ----------------------------------------------------------------------
	def replot(self):
		pass

	# ----------------------------------------------------------------------
	def escape(self, s):
		"""Escape characters in string"""
		return s.replace("'","''")

	# ----------------------------------------------------------------------
	def output(self, output=None):
		pass

	# ----------------------------------------------------------------------
	def set(self, cmd=""):
		pass

	# ----------------------------------------------------------------------
	def version(self):
		"""return version number of plotting engine"""
		return PlotEngine._version

	# ----------------------------------------------------------------------
	def log(self, axis="y", log=True):
		pass

	# ----------------------------------------------------------------------
	def xlog(self):  self.log("x")
	def ylog(self):  self.log("y")
	def zlog(self):  self.log("z")
	def cblog(self): self.log("cb")

	# ----------------------------------------------------------------------
	def linear(self, axis):
		self.log(axis,False)

	def xlin(self):  self.linear("x")
	def ylin(self):  self.linear("y")
	def zlin(self):  self.linear("z")
	def cblin(self): self.linear("cb")

	# ----------------------------------------------------------------------
	def getRange(self, axis="x"):
		return 0.,1.

	# ----------------------------------------------------------------------
	def getRange2(self, axes):
		return 0., 1., 0., 1.

	# ----------------------------------------------------------------------
	# Change plot engine directory to path
	# ----------------------------------------------------------------------
	def cd(self, path):
		pass

	# ----------------------------------------------------------------------
	# Convert color name to something acceptable by the engine
	# To be overridden if needed
	# ----------------------------------------------------------------------
	def color(self, name):
		return name

	# ----------------------------------------------------------------------
	# Convert marker name to something acceptable by the engine
	# To be overridden if needed
	# ----------------------------------------------------------------------
	def marker(self, name):
		return name
