# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   29-Nov-2009

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import os
import time
import stat
import fnmatch
import tkinter as tk
from tkinter import ttk
from tkinter import messagebox

import icons
import tkExtra

if "_" not in dir(): _ = lambda x: x

_DIR_TYPE     = " <DIR>"
_FILE_TYPE    = "-file-"
_LINK_TYPE    = "-link-"
_BACKUP_TYPE  = "-backup-"
_TIME_FORMAT  = "%Y.%m.%d %H:%M:%S"
DISABLE_FILE  = "DarkGray"

COLORS = {
		"gz":	"DarkBlue",
		"tgz":	"DarkBlue",
		"zip":	"DarkBlue",
		"tbz":	"DarkBlue",
		"rpm":	"DarkBlue",
		"deb":	"DarkBlue",

		"flair":"DarkGreen",
		"fluka":"#109010",
		"inp":	"#109010",
		"out":	"#109010",
		"log":	"#109010",
		"err":	"DarkRed",
		"m":	"Blue",
		"mcnp":	"Blue",
		"mac":  "DarkCyan",
		"macro":"DarkCyan",
		"moira":"DarkCyan",

		"hist":	"#107010",
		"dat":	"#107010",
		"lis":	"#107010",
		"trk":	"#107010",
		"cll":	"#107010",
		"dtc":	"#107010",
		"bdx":	"#105050",
		"yie":	"#105050",
		"rnc":	"#105050",
		"bnn":	"#205050",

		"py":	"Brown",
		"f":	"Brown",
		"F":	"Brown",
		"for":	"Brown",
		"FOR":	"Brown",
		"c":	"Brown",
		"C":	"Brown",
		"cc":	"Brown",
		"cpp":	"Brown",
		"a":	"Brown",
		"so":	"Brown",

		"eps":	"Purple",
		"ps":	"Purple",
		"gif":	"Purple",
		"png":	"Purple",
		"jpg":	"Purple",
		"bmp":	"Purple",
		"tif":	"Purple",

		"vxl":  "DarkGreen",
		"dcm":  "DarkCyan",

		"ngc":  "Brown",
		"nc" :  "Brown",

		"probe": "DarkOrange",
		"stl"  : "DarkOrange",

		_LINK_TYPE:	"DarkCyan",
		_BACKUP_TYPE:	"DarkGray",
		_DIR_TYPE:	"DarkBlue",

		"core" : "Red",
	}

FILETYPE = {
		"gz":	"Package gzip",
		"tgz":	"Package tgz",
		"zip":	"Package zip",
		"tbz":	"Package tbz",
		"rpm":	"Package rpm",
		"deb":	"Package deb",

		"flair":"FLAIR",
		"fluka":"Fluka input",
		"inp":	"Fluka input",
		"out":	"Output",
		"log":	"Log",
		"err":	"Error",
		"lis":	"Listing",
		"m":	"Mcnp input",
		"mcnp":	"Mcnp input",
		"mac":  "Geant4 macro",
		"macro":"Geant4 macro",
		"moira":"Moira geometry input",

		"py":	"Python",
		"f":	"Fortran",
		"F":	"Fortran",
		"for":	"Fortran",
		"FOR":	"Fortran",
		"c":	"C",
		"C":	"C",
		"cc":	"C++",
		"cpp":	"C++",
		"a":	"Lib a",
		"so":	"Lib so",

		"eps":	"Image eps",
		"gif":	"Image gif",
		"jpg":	"Image jpg",
		"png":	"Image png",
		"bmp":	"Image bmp",
		"ps":	"Image ps",
		"tif":	"Image tif",

		"vxl":  "Voxel",
		"dcm":  "Dicom",

		_LINK_TYPE   : _LINK_TYPE,
		_BACKUP_TYPE : _BACKUP_TYPE,
		_DIR_TYPE    : _DIR_TYPE,
	}

DESCRIPTION = {
		"eps"  : "Encapsulated Postscript",
		"ps"   : "Postscript",
		"png"  : "Portable Network Graphics",
		"svg"  : "Scalable vector graphics",
		"gif"  : "Graphic Interchange Format",
		"jpg"  : "Joint motion picture format",
		"jpeg" : "Joint motion picture format",
		"pdf"  : "Portable document format",
	}

#-------------------------------------------------------------------------------
def fileTypeColor(filename):
	fn = os.path.basename(filename)
	dot = fn.rfind(".")
	if dot>=0:
		ext = fn[dot+1:].lower()
	else:
		ext = _FILE_TYPE
	color  = None		# Default

	try: s = os.lstat(filename)
	except: return ext,color

	isdir = stat.S_ISDIR(s.st_mode)
	islnk = stat.S_ISLNK(s.st_mode)

	if isdir:
		color = COLORS.get(_DIR_TYPE)
		ext   = FILETYPE.get(_DIR_TYPE)

	elif islnk:
		color = COLORS.get(_LINK_TYPE)
		ext   = FILETYPE.get(_LINK_TYPE)

	elif fn[-1] == "~":
		color = COLORS.get(_BACKUP_TYPE)

	elif fn == "core" or fn.startswith("core."):
		color = COLORS.get("core")

	else:
		color = COLORS.get(ext,color)
		ext   = FILETYPE.get(ext,ext)

	return ext,color

#===============================================================================
# FileDialog
#===============================================================================
class FileDialog(tk.Toplevel):
	_active   = False	# Avoid re-entrance of the dialog if by accident
				# someone double clicks a button
	_title    = None	# overridden by super-classes
	sort      = None
	width     = -1
	height    = -1
	#sash      = None
	newfolder = None
	_history  = []

	# ----------------------------------------------------------------------
	def __init__(self, title=None,
			master=None,
			initialdir=None,
			initialfile=None,
			defaultextension=None,
			multiple=False,
			filetypes=None,
			**kw):

		super().__init__(master, class_="FileDialog")
		self.transient(master)
		self.title(title)

		FileDialog._active = True

		if title is None: title = self._title

		self.dirframe = tk.Frame(self)
		self.dirframe.pack(side=tk.TOP, fill=tk.X)
		tk.Label(self.dirframe, text=_("Directory:")).grid(
			row=0, column=0)

		self.downButton = tk.Label(self.dirframe,
				text="\N{BLACK DOWN-POINTING TRIANGLE}",
				padx=3, pady=1, relief=tk.RAISED)
		self.downButton.bind("<Button-1>", self.history)
		self.downButton.grid(row=0, column=99)

		if FileDialog.newfolder is None:
			FileDialog.newfolder = icons.get("newfolder")

		tk.Button(self.dirframe, image=FileDialog.newfolder,
			padx=3, pady=3,
			command=self.newFolder).grid(
			row=0, column=100)
		self.dirframe.grid_columnconfigure(98, weight=1)
		self.buttons = []

		self.multiple = multiple
		selectmode = tk.EXTENDED if multiple else tk.BROWSE
		self.fileList = tkExtra.ColorMultiListbox(self,
			((_("Name"), 30, None),
			 (_("Type"), 12, None),
			 (_("Size"),  8, None),
			 (_("Date"), 17, None)),
			 height=20,
			 selectmode=selectmode)
		self.fileList.pack(expand=tk.YES, fill=tk.BOTH)
		self.fileList.setPopupMenu(
			[(_('Rename'),     0, self.rename),
			 (_('Delete'),     0, self.delete),
			 (_('New Folder'), 0, self.newFolder) ])

		self.fileList.bindList("<Double-1>",      self.double)
		self.fileList.bindList('<Return>',        self.double)
		self.fileList.bindList('<F2>',            self.rename)
		self.fileList.bindList("<Key-BackSpace>", self.upDir)
		self.fileList.bind("<<ListboxSelect>>",   self.select)
		self.fileList.bind("<<ListboxSort>>",     self.sortChanged)
		#self.fileList.bind("<Configure>",         self.resize)

		frame = tk.Frame(self)
		frame.pack(side=tk.BOTTOM, fill=tk.X)

		l = tk.Label(frame, text=_("File name:"))
		l.grid(row=0, column=0, sticky=tk.E)
		self.filename = tk.Entry(frame, background="White")
		self.filename.grid(row=0, column=1, sticky=tk.EW)
		self.ok = tk.Button(frame, text=_("Open"), command=self.openFilename)
		self.ok.grid(row=0, column=2, sticky=tk.EW)

		l = tk.Label(frame, text=_("Files of type:"))
		l.grid(row=1, column=0, sticky=tk.E)

		self.typeCombo = tkExtra.Combobox(frame, command=self.typeChange)
		self.typeCombo.grid(row=1, column=1, sticky=tk.NSEW)

		self.filter = None
		self.filetypes = {}
		if filetypes:
			if isinstance(filetypes[0],str):
				filetypes = (filetypes,)
			self.filter = filetypes[0]
			for desc,ext in filetypes:
				if isinstance(ext,str):
					s = "%s (%s)"%(desc,ext)
					ext = (ext,)
				else:
					s = "%s (%s)"%(desc, ",".join(ext))
				self.typeCombo.insert(tk.END, s)
				self.filetypes[s] = ext

		b = tk.Button(frame, text=_("Cancel"), command=self.cancel)
		b.grid(row=1, column=2, sticky=tk.EW)
		frame.grid_columnconfigure(1, weight=1)

		self.bind("<Return>", self.openFilename)
		self.bind("<Escape>", self.cancel)
		self.protocol("WM_DELETE_WINDOW", self.close)

		# Flags
		self.hidden = False	# Show hidden files
		self.links  = True	# Show links
		self.dirs   = True	# Show directories
		self.files  = True	# Show files
		self.seldir = False	# Select directory instead of file
		self.selFile = ""	# Selected files

		# popup history
		self._popup = None
		self._historyOldLen = len(FileDialog._history)

		# Variables
		# 1st set the path if any
		if initialdir:
			self.path = os.path.abspath(initialdir)
		else:	# otherwise to the current directory
			try:
				self.path = os.getcwd()
			except FileNotFoundError:
				self.path = os.path.expanduser("~")

		if initialfile:
			# if a file is specified and has a different path
			initialfile = os.path.abspath(initialfile)
			d,n = os.path.split(initialfile)
			if d != "": self.path = d
			self.filename.insert(0, n)
			self.filename.select_range(0,tk.END)
		FileDialog.append2History(self.path)
		if master:
			tkExtra.placeWindow(self, master)

	# ----------------------------------------------------------------------
	def show(self):
		self.buttonPath(self.path)
		# Find best match for current extension
		ext = os.path.splitext(self.filename.get())[1]
		for desc,lstext in self.filetypes.items():
			found = False
			for e in lstext:
				if ext in e:
					self.typeCombo.set(desc)
					found = True
					break
			if found: break
		else:
			self.typeCombo.set(self.typeCombo.get(0))	# will fill the files

		self.focus_set()
		self.filename.focus_set()
		try:
			# If an error message shows up during filling
			# the window will be already visible and wait_visbility
			# will behave like the wait_window below and then create an error
			if not self.winfo_ismapped():
				self.wait_visibility()
			if FileDialog.width > 0:
				self.geometry(f"{FileDialog.width}x{FileDialog.height}")
			self.grab_set()
		except tk.TclError:
			pass
		try:
			self.wait_window()
		except tk.TclError:
			pass
		return self.selFile

	# ----------------------------------------------------------------------
	# Restore sash size on creation
	# ----------------------------------------------------------------------
	#def resize(self, event):
	#	# --- Begin XMING workaround ---
	#	if os.environ.get("XMING") is not None:
	#		FileDialog.sash = None
	#	# --- End XMING workaround ---
	#	if FileDialog.sash is not None:
	#		FileDialog.sash.reverse()
	#		# XXX XXX ERROR: paneframe doesn't update the width/height
	#		# so all sash placement is wrong
	#		self.fileList.paneframe.update_idletasks()
	#		if FileDialog.sash is not None:
	#			n = len(FileDialog.sash)-1
	#			for i,x in enumerate(FileDialog.sash):
	#				try:
	#					self.fileList.paneframe.sash_place(n-i,x,1)
	#				except tk.TclError:
	#					pass
	#			FileDialog.sash = None

	# ----------------------------------------------------------------------
	# Create buttons for the path
	# ----------------------------------------------------------------------
	def buttonPath(self, path):
		path = path.split(os.sep)
		if path[0] == "": path[0] = os.sep
		if path[-1] == "": del path[-1]
		lp = len(path)
		lb = len(self.buttons)

		i = 0
		same = True
		while i < min(lp, lb):
			b = self.buttons[i]
			if path[i] != b["text"]:
				b["text"] = path[i]
				same = False
			i += 1

		if lb < lp:	# Create new buttons
			while i < lp:
				btn = tk.Button(self.dirframe,
						text=path[i],
						command=lambda s=self,b=i:s.button(b),
						padx=1)
				self.buttons.append(btn)
				i += 1
				btn.grid(row=0, column=i)
				btn.bind("<Return>", self._dirbutton_invoke)
				btn.bind("<Left>",   self._dirbutton_prev)
				btn.bind("<Right>",  self._dirbutton_next)

		elif lp < lb and not same:	# Use existing buttons
			while i < lb:
				self.buttons[i].grid_forget()
				i += 1
			del self.buttons[lp:]

		for i in range(lp):
			self.buttons[i].configure(
					foreground = "black",
					activeforeground = "black")
		self.buttons[lp-1].configure(
					foreground = "blue",
					activeforeground = "blue")
		for i in range(lp, len(self.buttons)):
			self.buttons[i].configure(
					foreground="darkgray",
					activeforeground = "darkgray")

	# ----------------------------------------------------------------------
	def _dirbutton_invoke(self, event):
		event.widget.invoke()
		return "break"

	# ----------------------------------------------------------------------
	def _dirbutton_prev(self, event):
		self.event_generate("<Shift-Tab>")
		return "break"

	# ----------------------------------------------------------------------
	def _dirbutton_next(self, event):
		self.event_generate("<Tab>")
		return "break"

	# ----------------------------------------------------------------------
	def button(self, b):
		path = [ x["text"] for x in self.buttons[0:b+1] ]
		if path[0] == os.sep:
			path = os.sep + os.sep.join(path[1:])
		else:
			path = os.sep.join(path)
		if path=="": path=os.sep
		self.fileList.focus_set()
		self.changePath(path)

	# ----------------------------------------------------------------------
	def history(self, event=None):
		if self._popup is not None:
			self._historyDestroy()
			return
		self.downButton["relief"] = tk.SUNKEN
		self._popup = tk.Toplevel(self)
		self._popup.transient(self)
		self._popup.overrideredirect(1)
		self._popup.withdraw()
		self._popup.bind('<Escape>',	self._historyDestroy)
		self._popup.bind('<FocusOut>',	self._historyFocusOut)

		x = self.buttons[0].winfo_rootx()
		y = self.buttons[0].winfo_rooty() + self.buttons[0].winfo_height() - 2
		w = self.downButton.winfo_rootx() + self.downButton.winfo_width() - x
		h = self.fileList.winfo_height() - 20
		self._popup.geometry('%dx%d+%d+%d' % (w,h,x,y))

		sb = ttk.Scrollbar(self._popup, orient=tk.VERTICAL, takefocus=False)
		sb.pack(side=tk.RIGHT, fill=tk.Y)
		self._popupList = tk.Listbox(self._popup,
			background="White",
			selectmode=tk.BROWSE,
			takefocus=True,
			yscrollcommand=sb.set)
		self._popupList.pack(side=tk.LEFT, fill=tk.BOTH, expand=tk.YES)
		self._popupList.bind("<ButtonRelease-1>", self._historyClick)
		self._popupList.bind("<Return>", self._historyClick)
		sb.config(command=self._popupList.yview)
		for h in sorted(FileDialog._history):
			self._popupList.insert(tk.END, h)
		self._popupList.selection_set(0)
		self._popupList.activate(0)

		self.grab_release()
		try:
			self._popup.deiconify()
			self._popup.lift()
			self._popup.focus_set()
			self._popupList.focus_set()
			self._popup.wait_visibility()
		except tk.TclError:
			pass

	# ----------------------------------------------------------------------
	def _historyFocusOut(self, event):
		try:
			f = self.focus_get()
		except KeyError:
			pass
		else:
			if f in (self._popup, self._popupList):
				return
		self._historyDestroy()

	# ----------------------------------------------------------------------
	def _historyClick(self, event):
		try:
			sel = self._popupList.curselection()[0]
			self.changePath(self._popupList.get(sel))
			self._historyDestroy()
		except:
			pass

	# ----------------------------------------------------------------------
	def _historyDestroy(self, event=None):
		self.downButton["relief"] = tk.RAISED
		self._popup.destroy()
		self._popup = None
		try:
			self.grab_set()
		except tk.TclError:
			pass
		self.focus_set()
		return "break"

	# ----------------------------------------------------------------------
	def upDir(self, event):
		if not tkExtra.SearchBox.search:
			self.changePath(os.path.dirname(self.path))
		else:
			event.widget._key(event)

	# ----------------------------------------------------------------------
	def changePath(self, path):
		if path[-1] != os.sep: path += os.sep
		path = os.path.abspath(path)
		if not os.path.isdir(path):
			messagebox.showerror(_("Error"),
				_("Cannot access path \"%s\"")%(path),
				parent=self)
			return
		self.buttonPath(path)
		self.path = path
		self.fill()
		FileDialog.append2History(self.path)

	# ----------------------------------------------------------------------
	def fill(self, path=None):
		self.fileList.delete(0,tk.END)
		tkExtra.SearchBox.reset()

		if path is None: path = self.path

		# Populate list but sorted
		try:
			for fn in os.listdir(path):
				if not self.hidden and fn[0]==".": continue
				filename = os.path.join(path, fn)
				ext, color = fileTypeColor(filename)

				try: s = os.lstat(filename)
				except: continue

				size  = 0
				islnk = stat.S_ISLNK(s.st_mode)
				if islnk:
					try: s = os.stat(filename)
					except: continue
				isdir = stat.S_ISDIR(s.st_mode)

				if self.filter is not None and not isdir and not islnk:
					match = False
					for pat in self.filter:
						if fnmatch.fnmatch(fn, pat):
							match = True
							break
				else:
					match = True

				if isdir:
					if not self.dirs: continue
				elif islnk:
					if not self.links: continue
				else:
					size = s.st_size

				if match:
					self.fileList.insert(tk.END, (fn, ext, size,
						 time.strftime(_TIME_FORMAT,
							time.localtime(s.st_mtime))))
					if not self.files and not isdir:
						self.fileList.setColor(tk.END, DISABLE_FILE)
					elif color:
						self.fileList.setColor(tk.END, color)
		except OSError:
			messagebox.showerror(_("Error"),
				_("Error listing folder \"%s\"")%(path),
				parent=self)

		if FileDialog.sort is None:
			self.fileList.sort(0, False)		# First short by name
			# Move all directories to top
			self.fileList.sort(1, False)		# then by type
			FileDialog.sort = None
		else:
			self.fileList.restoreSort(FileDialog.sort)

		# Find item to select
		fn = self.filename.get()
		if self.seldir: fn = os.path.basename(fn)
		for i in range(self.fileList.size()):
			if fn == self.fileList.listbox(0).get(i):
				self.fileList.see(i)
				self.fileList.activate(i)
				self.fileList.selection_set(i)
				break
		else:
			self.fileList.see(0)
			self.fileList.activate(0)
			self.fileList.selection_set(0)

	# ----------------------------------------------------------------------
	def sortChanged(self, event=None):
		FileDialog.sort = self.fileList.saveSort()

	# ----------------------------------------------------------------------
	def open(self, fn):
		if self.seldir and self.path == fn:
			self.selFile = self.path

		# Single file selection
		elif fn.find('","')<0:
			# Check for path
			try:
				filename = os.path.join(self.path, fn)
				s = os.stat(filename)
				if stat.S_ISDIR(s.st_mode):
					self.changePath(filename)
					self.filename.delete(0,tk.END)
					return
			except OSError:
				pass

			# Check for a pattern
			if fn.find('*')>=0 or fn.find('?')>=0:
				self.filter = (fn,)
				self.fill()
				return

			# Check for extension
			if self.filter:
				fn,ext = os.path.splitext(filename)
				if ext == "":
					ffext = os.path.splitext(self.filter[0])[1]
					if ffext!="":
						filename = fn+ffext

			if self.multiple:
				self.selFile = [filename]
			else:
				self.selFile = filename

		# Multiple file selection
		else:
			self.selFile = [os.path.join(self.path, f)
					for f in fn[1:-1].split('","')]

		if self.check():
			# Delete all temporary directories and keep only the last one
			if len(FileDialog._history) > self._historyOldLen:
				del FileDialog._history[self._historyOldLen:]
			FileDialog.append2History(self.path)
			self.close()

	# ----------------------------------------------------------------------
	# Open the filename entered in the entry box
	# ----------------------------------------------------------------------
	def openFilename(self, event=None):
		fn = self.filename.get()
		# Single file selection?
		if fn=="":
			self.select()
			fn = self.filename.get()
			if fn=="": return
		self.open(fn)

	# ----------------------------------------------------------------------
	def check(self): return True

	# ----------------------------------------------------------------------
	def cancel(self, event=None):
		if self._popup is not None:
			self._historyDestroy()
		else:
			self.selFile = ""
			self.close()

	# ----------------------------------------------------------------------
	def close(self):
		#bd = self.fileList.paneframe["bd"]
		FileDialog._active = False
		FileDialog.width   = self.winfo_width()
		FileDialog.height  = self.winfo_height()
		#FileDialog.sash    = [self.fileList.paneframe.sash_coord(i)[0]-bd
		#			for i in range(len(self.fileList.listboxes())-1)]
		tkExtra.SearchBox.reset()
		self.grab_release()
		self.destroy()

	# ----------------------------------------------------------------------
	def double(self, event):
		sel = self.fileList.curselection()
		if len(sel)!=1: return
		item = self.fileList.get(sel[0])
		if item[1] == _DIR_TYPE:
			self.changePath(os.path.join(self.path, item[0]))
			return "break"
		elif item[1] == _LINK_TYPE:
			# maybe a directory?
			path = os.path.join(self.path, item[0])
			try:
				s = os.stat(path)
				if stat.S_ISDIR(s.st_mode):
					self.changePath(path)
					return "break"
			except: pass
		self.openFilename()

	# ----------------------------------------------------------------------
	# Select current file from listbox
	# ----------------------------------------------------------------------
	def select(self, event=None):
		sel = self.fileList.curselection()

		if len(sel)==1:
			item = self.fileList.get(sel[0])[0]
			fn = os.path.join(self.path, item)
			if self.seldir:
				try:
					s = os.stat(fn)
					if not stat.S_ISDIR(s.st_mode):
						fn = os.path.dirname(fn)
				except OSError:
					pass
				self.filename.delete(0, tk.END)
				self.filename.insert(0, fn)
			else:
				try:
					s = os.stat(fn)
					if not stat.S_ISDIR(s.st_mode):
						self.filename.delete(0, tk.END)
						self.filename.insert(0, item)
				except OSError:
					pass
		else:
			lget = self.fileList.get
			files = [f"\"{lget(i)[0]}\"" for i in sel]

			if files:
				self.filename.delete(0, tk.END)
				self.filename.insert(0, ",".join(files))

			elif self.seldir:
				self.filename.delete(0, tk.END)
				self.filename.insert(0, self.path)

	# ----------------------------------------------------------------------
	def typeChange(self, event=None):
		pat = self.typeCombo.get()
		self.filter = self.filetypes.get(pat,None)
		self.fill()
		if self.filter is None or self.seldir: return

		# Change extension if needed
		first = None
		filename = self.filename.get()
		if filename == "" or "," in filename: return
		fn,ext = os.path.splitext(filename)
		for i in self.filter:
			e = os.path.splitext(i)[1]
			if first is None and e: first = e
			if e == ext: return
		if first:
			# not found, change the filename to the first extension
			self.filename.delete(0, tk.END)
			self.filename.insert(0, fn+first)

	# ----------------------------------------------------------------------
	def newFolder(self):
		self.fileList.insert(tk.END, (_("NewFolder"), _DIR_TYPE, 0,
					 time.strftime(_TIME_FORMAT,
					 time.localtime(time.time()))))
		self.fileList.see(tk.END)
		self.fileList.selection_clear(0,tk.END)
		self.fileList.selection_set(tk.END)
		self.fileList.activate(tk.END)
		edit = tkExtra.InPlaceEdit(self.fileList.listbox(0))
		if edit.value:
			try:
				os.mkdir(os.path.join(self.path, edit.value))
			except OSError:
				messagebox.showerror(_("Error"),
					_("Error creating folder \"%s\"")%(edit.value),
					parent=self)
				self.fileList.delete(tk.END)
				return
			self.fileList.selection_set(tk.END)
			self.fileList.see(tk.END)
			self.fileList.setColor(tk.END, COLORS.get(_DIR_TYPE))
			self.select()
		else:
			try:
				self.fileList.delete(tk.END)
			except tk.TclError:
				pass

	# ----------------------------------------------------------------------
	def rename(self, event=None):
		fn = self.fileList.listbox(0).get(tk.ACTIVE)
		edit = tkExtra.InPlaceEdit(self.fileList.listbox(0))
		if edit.value and edit.value != fn:
			try:
				os.rename(os.path.join(self.path, fn),
					os.path.join(self.path, edit.value))
			except OSError:
				messagebox.showerror(_("Error"),
					_("Error renaming \"%s\" to \"%s\"")
						%(fn, edit.value),
					parent=self)
		self.select()

	# ----------------------------------------------------------------------
	def delete(self):
		sel = list(map(int,self.fileList.curselection()))
		sel.reverse()
		if not sel: return
		try:
			for i in sel:
				fn = self.fileList.listbox(0).get(i)
				filename = os.path.join(self.path, fn)
				s = os.lstat(filename)
				if stat.S_ISDIR(s.st_mode):
					os.rmdir(filename)
				else:
					os.remove(filename)
				self.fileList.delete(i)
		except OSError as ex:
			messagebox.showerror(_("Error"),
					_("Error deleting file \"%s\"\n%s")%(fn,str(ex)),
					parent=self)
		self.select()

	#-----------------------------------------------------------------------
	@staticmethod
	def append2History(path):
		if not path: return
		if path not in FileDialog._history:
			FileDialog._history.append(path)

#===============================================================================
class OpenDialog(FileDialog):
	_title = _("Open")

	# ----------------------------------------------------------------------
	# Check if file exist
	# ----------------------------------------------------------------------
	def check(self):
		if isinstance(self.selFile, list):
			for f in self.selFile:
				if not os.path.isfile(f):
					messagebox.showwarning(_("File does not exist"),
						_(f"File {f!r} does not exist"),
						parent=self)
					self.selFile = ""
					return False
		elif not os.path.isfile(self.selFile):
			messagebox.showwarning(_("File does not exist"),
				_("File \"%s\" does not exist")%(self.selFile),
				parent=self)
			self.selFile = ""
			return False
		return True

#===============================================================================
class SaveAsDialog(FileDialog):
	_title = _("Save As")

	# ----------------------------------------------------------------------
	def __init__(self, **kw):
		FileDialog.__init__(self, **kw)
		self.ok["text"] = _("Save")

	# ----------------------------------------------------------------------
	def check(self):
		if os.path.isfile(self.selFile):
			ans = messagebox.askyesno(_("File already exists"),
				_(f"Overwrite existing file {self.selFile!r}?"),
				parent=self)
			if not ans:
				self.selFile = ""
				return False
		return True

#===============================================================================
class DirectoryDialog(FileDialog):
	_title = _("Choose Directory")

	#-----------------------------------------------------------------------
	def __init__(self, **kw):
		FileDialog.__init__(self, **kw)
		self.files  = False
		self.seldir = True
		self.filename.insert(0, self.path)

	# ----------------------------------------------------------------------
	def changePath(self, path):
		path = os.path.abspath(path)
		FileDialog.changePath(self, path)
		self.filename.delete(0, tk.END)
		self.filename.insert(0, path)

#===============================================================================
def askfilename(**options):
	"""Ask for a filename"""
	if FileDialog._active: return ""
	return FileDialog(**options).show()

def askopenfilename(**options):
	"""Ask for a filename to open"""
	if FileDialog._active: return ""
	return OpenDialog(**options).show()

def askopenfilenames(**options):
	"""Ask for a multiple filenames to open"""
	if FileDialog._active: return ()
	options["multiple"] = True
	return OpenDialog(**options).show()

def asksaveasfilename(**options):
	"""Ask for a filename to save as"""
	if FileDialog._active: return ""
	return SaveAsDialog(**options).show()

def askdirectory(**options):
	"""Ask for a directory"""
	if FileDialog._active: return ""
	return DirectoryDialog(**options).show()

#===============================================================================
if __name__ == "__main__":
	import sys

	root = tk.Tk()
	root.withdraw()
	icons.init()
	initdir = None
	if len(sys.argv)>1:
		initdir = os.path.abspath(sys.argv[1])
	#print askdirectory()

	print(asksaveasfilename(title=_("Open"),
			initialdir=initdir,
#			initialfile="test.f",
			filetypes=(("All","*"),
				("Python", "*.py"),
				("Flair",("*.flair", "*.inp")))))
	#import tkFileDialog
	#print tkFileDialog.asksaveasfilename( title="Open flair project",
	#		initialdir=initdir,
	#		filetypes=(("All","*"),
	#			("Python", "*.py"),
	#			("Flair",("*.flair", "*.inp"))))
