#
# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   24-Nov-2018

import configparser

import tkinter as tk
import tkinter.font as tkFont

#===============================================================================
# Static class providing all in/out functions for the configuration management
#===============================================================================
class Config(configparser.RawConfigParser):
	#-----------------------------------------------------------------------
	def read(self, inifiles, *args, **kwargs):
		self.optionxform=str	# preserve case
		super().read(inifiles, *args, **kwargs)

	#-----------------------------------------------------------------------
	def getInt(self, section, name, default):
		try:
			return self.getint(section, name)
		except:
			self.set(section, name, str(default))
			return default

	#-----------------------------------------------------------------------
	def getFloat(self, section, name, default):
		try:
			return self.getfloat(section, name)
		except:
			self.set(section, name, str(default))
			return default

	#-----------------------------------------------------------------------
	def getStr(self, section, name, default=None):
		try:
			return self.get(section, name)
		except:
			self.set(section, name, str(default))
			return default

	#-----------------------------------------------------------------------
	def getBool(self, section, name, default):
		try:
			return self.getboolean(section, name)
		except:
			self.set(section, name, str(bool(default)))
			return default

	#-----------------------------------------------------------------------
	def set(self, section, name, value):
		super().set(section, name, str(value))
	setStr   = set
	setInt   = set
	setFloat = set

	#-----------------------------------------------------------------------
	def setBool(self, section, name, value):
		self.set(section, name, str(bool(value)))

	#-----------------------------------------------------------------------
	# FONTS
	#-----------------------------------------------------------------------
	# Return a font from a string
	#-----------------------------------------------------------------------
	@staticmethod
	def makeFont(name, value=None, root=None):
		try:
			font = tkFont.Font(root=root, name=name, exists=True)
		except tk.TclError:
			font = tkFont.Font(root=root, name=name)
			font.delete_font = False
		except AttributeError:
			return None
		if value is None: return font
		if isinstance(value, tuple):
			font.configure(family=value[0], size=value[1])
			try:
				font.configure(weight=value[2])
			except:
				pass
			try:
				font.configure(slant=value[3])
			except:
				pass
		return font

	#-----------------------------------------------------------------------
	# Get font from configuration
	#-----------------------------------------------------------------------
	def getFont(self, section, name, default=None, root=None):
		try:
			value = self.get(section, name)
		except:
			font = Config.makeFont(name, default, root)
			self.setFont(section, name, font)
			return font

		if isinstance(value, str):
			value = tuple(value.split(','))

		if isinstance(value, tuple):
			font = Config.makeFont(name, value, root)
			if font is not None: return font
		return value

	#-----------------------------------------------------------------------
	# Set font in configuration
	#-----------------------------------------------------------------------
	def setFont(self, section, name, font):
		if font is None: return
		if isinstance(font,str):
			self.set(section, name, font)
		elif isinstance(font,tuple):
			self.set(section, name, ",".join(map(str,font)))
		else:
			self.set(section, name, "%s,%s,%s" %
				(font.cget("family"),font.cget("size"),font.cget("weight")))

	#-----------------------------------------------------------------------
	def updateFont(self, section, name, font):
		self.setFont(section, name, font)
		self.getFont(section, name)

	#-----------------------------------------------------------------------
	# COLORS
	#-----------------------------------------------------------------------
	def getColor(self, section, name, default):
		try:
			return self.get(section, name)
		except:
			self.setColor(section, name, default)
			return default

	#-----------------------------------------------------------------------
	def setColor(self, section, name, value):
		self.set(section, name, value)

	#-----------------------------------------------------------------------
	# set color from integer
	#-----------------------------------------------------------------------
	def setIColor(self, section, name, value):
		self.setColor(section, name, f"#{value:06X}")

	#-----------------------------------------------------------------------
	# return color as Integer
	#-----------------------------------------------------------------------
	def getIColor(self, section, name, default):
		return int(str(self.getColor(section, name,f"#{default:06X}"))[1:],16)
