# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   20-Apr-2006

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import os
import select
import subprocess

#-------------------------------------------------------------------------------
# Execute a system command using pipes
# FIXME to unblock the output buffer
#-------------------------------------------------------------------------------
def system(cmd, inp=None, cwd=None, shell=False):
	"""
	execute a command by piping the input (list of strings) and
	returning the return code and a list of string of the output
	of the command. Newline character is not required in the input list
	and is removed from the output
	"""

	try:
		pipe = subprocess.Popen(cmd,
					shell=shell,
					cwd=cwd,
					stdin=subprocess.PIPE,
					stdout=subprocess.PIPE,
					stderr=subprocess.STDOUT,
					close_fds=True)
	except OSError:
		return 1,[f"Cannot execute command: {cmd}"]

	out = []
	try:
		rc = 0
		if inp:
			for x in inp:
				ready = select.select([pipe.stdout], [], [], 0)
				while ready[0]:
					# Use the os.read instead of the pipe.stdout.readline
					# to avoid blocking situations
					# readline waits for the \n or EOF
					data = os.read(pipe.stdout.fileno(), 1024)
					if not data: break
					out.append(data.decode(errors="replace"))
					ready = select.select([pipe.stdout], [], [], 0)
				pipe.stdin.write(f"{x}\n".encode())
				pipe.stdin.flush()
		pipe.stdin.close()
		while True:
			line = pipe.stdout.readline()
			if not line: break
			out.append(line.decode(errors="replace"))
	except OSError:
		rc = 1

	out = "".join(out).splitlines()
	return rc, out
