# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   24-Jun-2006

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import tkinter as tk
from tkinter import messagebox

import tkExtra
import tkDialogs
from log import say

#=============================================================================
# TextEditor
#=============================================================================
class TextEditor(tk.Frame):
	def __init__(self, master, *arg, **kw_args):
		super().__init__(master)
		self.txt = tk.Text(self, *arg, **kw_args)
		self.txt.grid(row=0, column=0, sticky='nsew')
		sbv = tkExtra.AutoScrollbar(self,
				takefocus=False)
		sbv.grid(row=0, column=1, sticky='ns')
		sbh = tkExtra.AutoScrollbar(self,
				takefocus=False,
				orient=tk.HORIZONTAL)
		sbh.grid(row=1, column=0, sticky='ew')

		self.txt.config(  yscrollcommand=sbv.set,
				  xscrollcommand=sbh.set )
		sbv.config(command=self.txt.yview)
		sbh.config(command=self.txt.xview)

		self.grid_rowconfigure(0, weight=1)
		self.grid_columnconfigure(0, weight=1)

	# ----------------------------------------------------------------------
	def cget(self,item):
		if not hasattr(self.txt,item):
			return super().cget(self.txt,item)
		else:
			return getattr(self.txt,item)
	__getitem__ = cget

	# ----------------------------------------------------------------------
	def configure(self,cnf=None,**kw):
		if kw: cnf=tk._cnfmerge((cnf,kw))
		for key in cnf.keys():
			if not hasattr(self.txt,key):
				self.txt.configure(cnf)
			else:
				setattr(self.txt,key,cnf[key])
	config=configure

	# ----------------------------------------------------------------------
	# Use this method to get the parent widget of the frame
	# ----------------------------------------------------------------------
	def __call__(self): return self.txt

	# --------------------------------------------------------------------
	def findDialog(self, event=None):
		try: txt = self.txt.selection_get()
		except:
			txt = ""
		fd = tkDialogs.FindReplaceDialog(self, replace=False)
		fd.show(self.find, None, None, txt)
		try:
			self.txt.tag_delete("find")
		except tk.TclError:
			pass

	# --------------------------------------------------------------------
	def replaceDialog(self, event=None):
		try: txt = self.txt.selection_get()
		except: txt = ""
		fd = tkDialogs.FindReplaceDialog(self,replace=True)
		fd.show(self.find, self.replace, self.replaceAll, txt)
		self.txt.tag_delete("find")

	# --------------------------------------------------------------------
	def find(self, findStr=None, matchCase=None):
		if findStr:
			TextEditor.save_findStr   = findStr
			TextEditor.save_matchCase = matchCase
		else:
			try:
				findStr   = TextEditor.save_findStr
				matchCase = TextEditor.save_matchCase
			except:
				return False

		self.txt.tag_remove(tk.SEL,"0.0",tk.END)
		self.txt.tag_delete("find")
		try:
			index = self.txt.search(findStr,
				"insert + 1 chars",
				nocase = matchCase)
			self.txt.mark_set('insert', index)
			endex = f"insert + {len(findStr)} chars"
			self.txt.tag_add(tk.SEL, index, endex)
			self.txt.tag_add("find", index, endex)
			self.txt.tag_configure("find",
					background = '#800000',
					foreground = '#FFFFFF')
			self.txt.see(index)
			self.txt.update_idletasks()
			self.found = True
			return True
		except:
			messagebox.showinfo("Not Found",
				f"Target \"{findStr}\" not found")
			self.found = False
			return False

	# --------------------------------------------------------------------
	def findNext(self, event=None):
		self.find()

	# --------------------------------------------------------------------
	def replace(self, findStr, replaceStr, matchCase):
		if not self.found:
			if not self.find(findStr, matchCase): return
		index = self.txt.index(tk.SEL_FIRST)
		self.txt.delete(tk.SEL_FIRST, tk.SEL_LAST)
		self.txt.insert(index, replaceStr)
		endex = f"{index} + {len(replaceStr)} chars"
		self.txt.tag_add(tk.SEL, index, endex)
		self.txt.tag_add("find", index, endex)
		self.found = False

	# --------------------------------------------------------------------
	def replaceAll(self, findStr, replaceStr, matchCase):
		say("TextEditor::replaceAll", findStr, replaceStr, matchCase)

	# ----------------------------------------------------------------------
	def delete(self, index1, index2=None): return self.txt.delete(index1, index2)
	def insert(self, index, chars, *args): return self.txt.insert(index, chars, *args)

#=============================================================================
# Testing...
if __name__ == "__main__":
	root = tk.Tk()
	editor = TextEditor(root)
	editor.pack(side=tk.LEFT,expand=1,fill=tk.BOTH)
	root.bind_class(root,'<Control-Key-q>',lambda e,r=root: r.destroy())
	root.bind_class(root,'<Control-Key-f>',editor.findDialog)
	root.bind_class(root,'<Control-Key-r>',editor.replaceDialog)
	root.bind_class(root,'<F3>',editor.findNext)

	root.mainloop()
