#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author:	Vasilis.Vlachoudis@cern.ch
# Date:	15-May-2019

import sys
import platform
#from enum import Enum, unique

#===============================================================================
# Tk event state modifiers from
# https://infohost.nmt.edu/tcc/help/pubs/tkinter/web/event-handlers.html
#===============================================================================
class STATE:	# (Enum)
	SHIFT    = 0X0001
	CAPSLOCK = 0X0002
	CONTROL  = 0X0004
	LEFTALT  = 0X0008
	NUMLOCK  = 0X0010
	RIGHTALT = 0X0080
	BUTTON1  = 0X0100
	BUTTON2  = 0X0200
	BUTTON3  = 0X0400
	ALT       = LEFTALT | RIGHTALT
	CTRLSHIFT = CONTROL | SHIFT
	ALTSHIFT  = ALT | SHIFT

	# check for Vista
	if sys.platform == "cygwin":
		s = platform.system().split("-")
		if s[1][0] == "6":
			ALT = 0x20000
	elif sys.platform == "win32":
		try:
			if sys.getwindowsversion()[0] == 6:
				ALT = 0x20000
		except AttributeError:
			pass

#===============================================================================
# Tk event keysym names and codes
# keycode	For KeyPress or KeyRelease events, this attribute is set to a
#		numeric code that identifies the key. However, it does not
#		identify which of the characters on that key were produced, so
#		that “x” and “X” have the same .keyCode value. For the possible
#		values of this field, see Section 54.5, “Key names”.
#
# .keysym	For KeyPress or KeyRelease events involving a special key, this
#		attribute is set to the key's string name, e.g., 'Prior' for
#		the PageUp key. See Section 54.5, “Key names” for a complete
#		list of .keysym names.
#
# .keysym_num	For KeyPress or KeyRelease events, this is set to a numeric
#		version of the .keysym field. For regular keys that produce a
#		single character, this field is set to the integer value of the
#		key's ASCII code. For special keys, refer to Section 54.5, “Key
#		names”.
#===============================================================================
class KEYSYM:
	pass
#Alt_L	64	65513	The left-hand alt key
#Alt_R	113	65514	The right-hand alt key
#BackSpace	22	65288	backspace
#Cancel	110	65387	break
#Caps_Lock	66	65549	CapsLock
#Control_L	37	65507	The left-hand control key
#Control_R	109	65508	The right-hand control key
#Delete	107	65535	Delete
#Down	104	65364	down arrow
#End	103	65367	end
#Escape	9	65307	Esc
#Execute	111	65378	SysReq
#F1	67	65470	Function key F1
#F2	68	65471	Function key F2
#Fi	66+i	65469+i	Function key Fi
#F12	96	65481	Function key F12
#Home	97	65360	home
#Insert	106	65379	insert
#Left	100	65361	left arrow
#Linefeed	54	106	Linefeed (control-J)
#KP_0	90	65438	0 on the keypad
#KP_1	87	65436	1 on the keypad
#KP_2	88	65433	2 on the keypad
#KP_3	89	65435	3 on the keypad
#KP_4	83	65430	4 on the keypad
#KP_5	84	65437	5 on the keypad
#KP_6	85	65432	6 on the keypad
#KP_7	79	65429	7 on the keypad
#KP_8	80	65431	8 on the keypad
#KP_9	81	65434	9 on the keypad
#KP_Add	86	65451	+ on the keypad
#KP_Begin	84	65437	The center key (same key as 5) on the keypad
#KP_Decimal	91	65439	Decimal (.) on the keypad
#KP_Delete	91	65439	delete on the keypad
#KP_Divide	112	65455	/ on the keypad
#KP_Down	88	65433	down arrow on the keypad
#KP_End	87	65436	end on the keypad
#KP_Enter	108	65421	enter on the keypad
#KP_Home	79	65429	home on the keypad
#KP_Insert	90	65438	insert on the keypad
#KP_Left	83	65430	left arrow on the keypad
#KP_Multiply	63	65450	× on the keypad
#KP_Next	89	65435	PageDown on the keypad
#KP_Prior	81	65434	PageUp on the keypad
#KP_Right	85	65432	→ on the keypad
#KP_Subtract	82	65453	- on the keypad
#KP_Up	80	65431	Up arrow on the keypad
#Next	105	65366	PageDown
#Num_Lock	77	65407	NumLock
#Pause	110	65299	pause
#Print	111	65377	PrintScrn
#Prior	99	65365	PageUp
#Return	36	65293	The enter key (control-M)
#Right	102	65363	Right arrow
#Scroll_Lock	78	65300	ScrollLock
#Shift_L	50	65505	The left-hand shift key
#Shift_R	62	65506	The right-hand shift key
#Tab	23	65289	The tab key
#Up	98	65362	Up arrow
