# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   16-Feb-2021

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

__all__ = [	"Mcnp",
		"McnpGeometry",
		"McnpCard",
		"McnpCardInfo",
		"McnpConfig",
		"McnpLayout"]

import configparser

from .input  import Mcnp, McnpGeometry
from .card   import McnpCard, McnpCardInfo
from .layout import McnpLayout
from .config import McnpConfig

DATABASE = "db/mcnp.ini"
GEOMDB   = "db/mcnp_geometry.ini"
__loaded = False

#-------------------------------------------------------------------------------
# Initialize classes:
#	McnpCardInfo
#	McnpParticle
#-------------------------------------------------------------------------------
def init(filename=None, g4name=None, nistname=None):
	from deck import Card, dbpath, readCardini

	global __loaded
	if __loaded: return
	__loaded = True

	Card.register_class(Mcnp.MODE, McnpCard)

	# Load card database
	if filename is None:
		filename = (dbpath(x) for x in (DATABASE,GEOMDB))

	# read card database and prepare CardInfo classes
	cardini = configparser.RawConfigParser()
	cardini.optionxform = str	# make case sensitive
	if not cardini.read(filename, encoding="UTF-8"): return False

	# parse cards
	readCardini(cardini, Mcnp.MODE, McnpCardInfo)

	del cardini
	return True

#===============================================================================
# Load only once
#init()
