# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   16-Feb-2021

import os
import re

from deck import DeckConfig

#===============================================================================
# A container class to keep the programs
# It should contain only static and class methods
#===============================================================================
class McnpConfig(DeckConfig):
	SECTION     = "Mcnp"		# Configuration section name
	version     = "Unknown"
	released    = ""
	CYCLE       = "mcnp_"		# prefix of temporary cycle directory
	directory   = ""		# Executable installation directory
	datapath    = ""		# DATAPATH directory
	DEFAULT_PRG = "mcnp"		# [flair.ini Main application program/script
	DEFAULT_EXE = "mcnp"		# [flair.ini
	EXT         = ".mcnp"		# default input extension
	EXTENSIONS  = [EXT, ".m"]
	programs    = { "prg"      : "",
			"exe"      : "",
		}

	CYCLEDIR_PAT = re.compile(rf"^{CYCLE}(\d+)$")

	#-----------------------------------------------------------------------
	@staticmethod
	def loadXSdir():
		from .input import Mcnp
		datapath = McnpConfig.datapath if McnpConfig.datapath else os.getenv("DATAPATH")
		if not datapath: return
		try:
			with open(os.path.join(datapath,"xsdir"),"r") as f:
				del Mcnp.zaid[:]
				del Mcnp.sab[:]
				exist = set()
				Mcnp.zaid.append("")
				Mcnp.sab.append("")
				for line in f:	# skip until directory line
					if line.rstrip()=="directory": break
				for line in f:
					if line.startswith("     "): continue
					zaid_full = line.split(maxsplit=1)[0]
					zaid = zaid_full.split(".")[0]
					if zaid_full[-1]=="t":
						Mcnp.sab.append(zaid_full)
					else:
						if zaid not in exist:
							exist.add(zaid)
							Mcnp.zaid.append(zaid)
						Mcnp.zaid.append(zaid_full)
		except (FileNotFoundError, PermissionError):
			pass

	#-----------------------------------------------------------------------
	# Load from config
	#-----------------------------------------------------------------------
	@staticmethod
	def loadConfig(config):
		section = McnpConfig.SECTION
		McnpConfig.setDir(config.getStr(section, "directory", ""))
		McnpConfig.setCmd("prg", config.getStr(section, "program",   McnpConfig.DEFAULT_PRG))
		McnpConfig.setCmd("exe", config.getStr(section, "executable",McnpConfig.DEFAULT_EXE))
		McnpConfig.datapath = config.getStr(section, "datapath",  McnpConfig.datapath)
		McnpConfig.loadXSdir()
