# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   23-Nov-2018

import re

from deck import DeckConfig

#===============================================================================
# A container class to keep the programs
# It should contain only static and class methods
#===============================================================================
class MoiraConfig(DeckConfig):
	SECTION     = "Moira"		# Configuration section name
	version     = "Unknown"
	released    = ""
	directory   = ""		# Executable installation directory
	CYCLE       = "moira_"		# prefix of temporary cycle directory
	DEFAULT_PRG = "moira.sh"	# [flair.ini] Main application program
	DEFAULT_EXE = "moira"		# [flair.ini] executable
	EXT         = ".moira"		# default input extension
	EXTENSIONS  = [EXT, ".macro", ".mac"]
	programs    = { "prg"       : "",
		        "exe"       : "",
		        "merge.py"  : ""
		}
	CYCLEDIR_PAT  = re.compile(rf"^{CYCLE}(\d+)$")
	CYCLEDIR2_PAT = re.compile(rf"^(.*/?){CYCLE}(\d+)$")

	#-----------------------------------------------------------------------
	# Load from config
	#-----------------------------------------------------------------------
	@staticmethod
	def loadConfig(config):
		section = MoiraConfig.SECTION
		MoiraConfig.setDir(config.getStr(section, "directory", ""))
		MoiraConfig.setCmd("prg",      config.getStr(section, "program",   MoiraConfig.DEFAULT_PRG))
		MoiraConfig.setCmd("exe",      config.getStr(section, "executable",MoiraConfig.DEFAULT_EXE))
		MoiraConfig.setCmd("merge.py", config.getStr(section, "merge.py", ""))
