# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   24-Oct-2018

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

__all__ = [	"Penelope",
		"PenelopeCard",
		"PenelopeCardInfo",
		"PenelopeConfig",
		"PenelopeLayout"]

import configparser

from .input  import Penelope
from .card   import PenelopeCard, PenelopeCardInfo
from .layout import PenelopeLayout
from .config import PenelopeConfig

DATABASE = "db/penelope.ini"
__loaded = False

#-------------------------------------------------------------------------------
# Initialize classes:
#	PenelopeCardInfo
#	PenelopeParticle
#-------------------------------------------------------------------------------
def init(filename=None, g4name=None, nistname=None):
	from deck import Card, dbpath, readCardini

	global __loaded
	if __loaded: return
	__loaded = True

	Card.register_class(Penelope.MODE, PenelopeCard)

	# Load card database
	if filename is None:
		filename = dbpath(DATABASE)

	# read card database and prepare CardInfo classes
	cardini = configparser.RawConfigParser()
	cardini.optionxform = str	# make case sensitive
	if not cardini.read(filename, encoding="UTF-8"): return False

	# parse cards
	readCardini(cardini, Penelope.MODE, PenelopeCardInfo)

	del cardini
	return True

#===============================================================================
# Load only once
#init()
