# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   9-Nov-2018

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

from deck import Layout

from .card import PenelopeCard
from .input import Penelope

#===============================================================================
# Layout class for the specific input/canvas display
#===============================================================================
class PenelopeLayout(Layout):
	MODE = PenelopeCard.MODE

	# ----------------------------------------------------------------------
	def init(self):
		order = [
			#("STAB",      None),
			#("MATERIAL", _MATERIAL_layout),
			#("BODY"    , _BODY_layout  ),
		]
		self._init_set(order, Penelope.MODE)

	# ----------------------------------------------------------------------
	# Enable/disable caching of lists
	# ----------------------------------------------------------------------
	def cache(self, on):
		super().cache(on)
		if on:
			# WARNING body refers to surface
			#     and region to bodies/modules
			self._cache["body"]     = self.bodyList()
			self._cache["region"]   = self.regionList()
			self._cache["material"] = self.materialList()
			self._cacheTypedef()

	# ----------------------------------------------------------------------
	# Return material list, excluding material if defined
	#-----------------------------------------------------------------------
	def materialList(self, e=None):
		try: return self._cache["material"]
		except KeyError: pass
		lst = [""] + [(f"{x} [{i}]",i) for i,x in enumerate(self.input.materialList())]
		return lst
