# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   16-Feb-2021

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

__all__ = [	"Phits",
		"PhitsCard",
		"PhitsCardInfo",
		"PhitsConfig",
		"PhitsLayout"]

import configparser

from .input    import Phits
from .particle import PhitsParticle
from .card     import PhitsCard, PhitsCardInfo
from .layout   import PhitsLayout
from .config   import PhitsConfig

DATABASE = "db/phits.ini"
GEOMDB   = "db/mcnp_geometry.ini"
__loaded = False

#-------------------------------------------------------------------------------
# Initialize classes:
#	PhitsCardInfo
#	PhitsParticle
#-------------------------------------------------------------------------------
def init(filename=None, g4name=None, nistname=None):
	from deck import Card, dbpath, readCardini

	global __loaded
	if __loaded: return
	__loaded = True

	Card.register_class(Phits.MODE, PhitsCard)

	# Load card database
	if filename is None:
		filename = (dbpath(x) for x in (DATABASE,GEOMDB))

	# read card database and prepare CardInfo classes
	cardini = configparser.RawConfigParser()
	cardini.optionxform = str	# make case sensitive
	if not cardini.read(filename, encoding="UTF-8"): return False

	# parse cards
	readCardini(cardini, Phits.MODE, PhitsCardInfo)

	# Particles
	section = "@particles"
	for pid, name in cardini.items(section):
		if pid[:3] == "pid":
			tag, idx = pid.split(".")
			#mass     = cardini.getfloat(section,"mass."+idx)
			#comment  = cardini.get(section,"comment."+idx)
			#try:
			#	pdg = int(cardini.get(section,"pdg."+idx))
			#except:
			#	pdg = 0
			PhitsParticle.add(int(idx), name) #, mass, comment, pdg)
	PhitsParticle.makeLists()

	del cardini
	return True

#===============================================================================
# Load only once
#init()
