# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   16-Feb-2021

__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

from deck import Layout

from .input    import Phits
from .card     import PhitsCard
from .particle import PhitsParticle

#===============================================================================
# Layout class for the specific input/canvas display
#===============================================================================
class PhitsLayout(Layout):
	MODE = PhitsCard.MODE

	# ----------------------------------------------------------------------
	def init(self):
		order = [
			#("STAB",      None),
			#("MATERIAL", _MATERIAL_layout),
			#("CELL",      _CELL_layout),
			#("CELL_LIKE",      _CELL_LIKE_layout),
		]
		self._init_set(order, Phits.MODE)

	# ----------------------------------------------------------------------
	# Enable/disable caching of lists
	# ----------------------------------------------------------------------
	#def cache(self, on):
	#	super().cache(on)

	# ----------------------------------------------------------------------
	# Automatically detect type from range
	# ----------------------------------------------------------------------
	def iType(self, name, rng):
		try:
			if rng=="cell":
				return (name, PhitsLayout.cellList)
			elif rng=="zaid":
				return (name, Phits.zaid)
			elif rng=="sab":
				return (name, Phits.sab)
			else:
				return super().iType(name,rng)
		except (TypeError,IndexError):
			return name

	# ----------------------------------------------------------------------
	def particleList(self):
		return PhitsParticle.list

	#-----------------------------------------------------------------------
	def cellList(self):
		return self.typedefList("cell")
