# -*- coding: utf-8 -*-
#
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# Please look at the supplied documentation for the user's
# license
#
# Author: Vasilis.Vlachoudis@cern.ch
# Date:   11-Nov-2022

#===============================================================================
# PhitsParticle class
#===============================================================================
class PhitsParticle:
	_db           = {}	# bi-directional particle dictionary particle<->id
	list          = []	# list of particles

	# ----------------------------------------------------------------------
	def __init__(self, idx, name, mass=0.0, comment=None, pdg=0):
		self.id      = idx
		self.name    = name
		self.mass    = mass
		self.comment = comment
		self.pdg     = pdg

	# ----------------------------------------------------------------------
	# add particle
	# ----------------------------------------------------------------------
	@staticmethod
	def add(idx, name, mass=0.0, comment=None, pdg=0):
		from .card import PhitsCard
		p = PhitsParticle(idx, name, mass, comment, pdg)
		PhitsParticle._db[idx]  = p
		PhitsParticle._db[name] = p

		# Populate also _baseLocalDict for the function evaluation
		if mass>0.0:
			n = name
			if name[-1] == "-":
				n = n[:-1]+"n"
			elif name[-1] == "+":
				n = n[:-1]+"p"
			if n.find("-")>=0:
				n = n.replace("-","")
			PhitsCard.globalDict[f"m{n}"] = mass
		return p

	# ----------------------------------------------------------------------
	#def __getitem__(self, idx):
	#	return PhitsParticle.particles[idx]

	# ----------------------------------------------------------------------
	@staticmethod
	def get(idx):
		return PhitsParticle._db.get(idx)

	# ----------------------------------------------------------------------
	# create lists
	# ----------------------------------------------------------------------
	@staticmethod
	def makeLists():
		if PhitsParticle.list: return
		PhitsParticle.list.append("")
		# Go through in ordered by id list
		for i in sorted([PhitsParticle._db[x].id
				for x in PhitsParticle._db
					if isinstance(x,str)]):
			part = PhitsParticle._db[i]
			PhitsParticle.list.append((part.name, part.id))
